# Copyright (C) 2004 Philippe Normand <phil@base-art.net>
#
# This file is part of EaseXML (http://easexml.base-art.net)
#
# Under PSF License (see COPYING)
# $HeadURL: http://svn.base-art.net/public/easexml/trunk/EaseXML/MixedList.py $
# $Id: MixedList.py 108 2005-07-30 13:16:19Z phil $


from TypedList import TypedList
from main import XMLObject
import copy
import classregistry
from types import StringType, UnicodeType

class MixedList(TypedList):
    """ Multiple Type storage List

        This kind of list can store every XMLObject type declared
        in its parent `ChoiceNode`. Plus Strings.

        Example:

        ::

          class Foo(XMLObject):
              pass

          class Bar(XMLObject):
              mix = ChoiceNode(['#PCDATA','Foo'], noLimit=True)

        In ``mix`` node, a backend MixedList will be created. It will
        ensure items stored are either strings ('#PCDATA') or ``Foo``
        instances.

    """

    def walkOnXMLObject(self, typeList, registry, callback, *args, **kw):
        if '#PCDATA' in typeList and callback('#PCDATA',*args, **kw):
            return True
        for className in typeList:
            theClass = classregistry.registry(registry).getClass(className)
            for nodeName, nodeInstance in theClass.__nodes__.iteritems():
                if nodeInstance.getType()  == 'ChoiceNode':
                    for classRef in nodeInstance.getItemType():
                        if callback(classRef, *args, **kw):
                            return True
                elif nodeInstance.getType() in ['ListNode','ItemNode']:
                    if callback(nodeInstance.getType(), *args, **kw):
                        return True
        return False


    def checkItem(self, it):
        typeMismatch = False
        alternatives = self._xmlList.getItemType()
        if type(it) in [StringType, UnicodeType]:
            if not self.walkOnXMLObject(alternatives,
                                        self._xmlList.getRegistry(),
                                        self._compareTypes, it):
                typeMismatch = True

        elif not isinstance(it, XMLObject):
            typeMismatch = True
        elif it.getClassName() not in alternatives:
            if not self.walkOnXMLObject(alternatives,
                                        self._xmlList.getRegistry(),
                                        self._compareTypes, it):
                typeMismatch = True

        if typeMismatch:
            raise TypeError("""\
%s type required for %s. Got %s instead""" % (`alternatives`,
                                              `self._xmlList.getName()`,
                                              `type(it)`))

        if isinstance(it,XMLObject):
            if it.getClassName() == self._xmlList.getParentType():
                it = copy.deepcopy(it)
            it.setParentNode(self._xmlList)
        return it
