# Copyright (C) 2004 Philippe Normand <phil@base-art.net>
#
# This file is part of EaseXML (http://easexml.base-art.net)
#
# Under PSF License (see COPYING)
# $HeadURL$
# $Id$

from parser import GenericParser, ParseError
try:
    import cElementTree as ElementTree
except ImportError:
    from elementtree import ElementTree
import time

class DummyElement:

    def __init__(self):
        self.text = ""
        self.tag = ""

class Parser(GenericParser):

    def __init__(self, data, encoding):
        self.encoding = encoding
        if data == '':
            self.tree = None
        else:
            self.tree = ElementTree.fromstring(data)
        self.children = {}
        self.parents = {}
        
    def getTree(self):
        assert self.tree is not None
        return self.tree
        
    def toXml(self, tree):
        return ElementTree.tostring(tree)

    def isChild(self, node, parentNodeName, tree):
        key = (tree,self.getNodeName(node))
        
        result = self.parents.get(key,None)
        if result is not None:
            return result
        
        result = False
        if self.getNodeName(tree) == parentNodeName:
            result = node in self.getChildren(tree)
        else:
            for child in self.getChildren(tree):
                if self.isChild(node, parentNodeName, child):
                    result = True
                    break
        self.parents[key] = result
        return result
    
    def getChildren(self, tree):
        result = self.children.get(tree,None)
        if result is not None:
            return result
        children = tree.getchildren()
        result = []
        txt = tree.text
        if txt:
            txt = txt.strip()
            if txt:
                elem = DummyElement()
                elem.text = txt
                result.append(elem)
        for child in children:
            tail = child.tail
            result.append(child)
            if tail:
                tail = tail.strip()
                elem = DummyElement()
                elem.text = tail
                result.append(elem)
        self.children[tree] = result
        return result

    def isTextNode(self, node):
        return hasattr(node, 'text')
    
    def getNodeValue(self, node):
        result = u''
        if self.isTextNode(node) and node.text is not None:
            result = node.text
        return result

    def getNodeName(self, node):
        return node.tag
    
    def getDirectChildrenWithName(self,parent, name):
        """ Fetch *direct* sub-nodes of a `parent` TREE tree. These
            nodes must have a name matching `name`.

            Return a list of DOM Nodes.
        """
        return [ node for node in self.getChildren(parent)
                 if self.getNodeName(node) == name ]

    def getValueForAttribute(self, node, attrName):
        if attrName == 'xmlns:rdf':
            import pdb; pdb.set_trace()
        try:
            result = node.attrib[attrName]
        except KeyError:
            raise AttributeError
        return result

    def getComments(self, tree):
        # XXX
        return ''
            
    getText = getNodeValue
    
    def getCDATA(self, tree):
        tail = tree.text
        if not tail.strip():
            try:
                tail = tree.getchildren()[0].tail
            except:
                tail = tree.tail
        return tail
            
    def textNodeName(self):
        return ''
