
from parser import GenericParser, ParseError
from xml.dom.minidom import parseString

class Parser(GenericParser):

    def __init__(self, data, encoding):
        if data == '':
            self.dom = None
        else:
            try:
                self.dom = parseString(data).documentElement
            except Exception,ex:
                raise ParseError(ex, data, encoding=encoding)

    def getTree(self):
        assert self.dom is not None
        return self.dom
        
    def finish(self):
        if self.dom:
            self.dom.unlink()

    def isTextNode(self, node):
        return node.nodeType == node.TEXT_NODE

    def isChild(self, node, parentNodeName, tree):
        return node.parentNode.nodeName == parentNodeName

    def toXml(self, tree):
        return tree.toxml()

    def getChildren(self, tree):
        return tree.childNodes

    def getNodeType(self, node):
        return node.nodeType

    def getNodeValue(self, node):
        return node.data

    def getNodeName(self, node):
        return node.nodeName

    def getDirectChildrenWithName(self,parent, name):
        """ Fetch *direct* sub-nodes of a `parent` DOM tree. These
            nodes must have a name matching `name`.

            Return a list of DOM Nodes.
        """
        return [ node for node in parent.childNodes
                 if node.nodeType == node.ELEMENT_NODE and \
                 node.nodeName == name ]

    def getValueForAttribute(self, node, attrName):
        return node.getAttributeNode(attrName).value

    def getComments(self, tree):
        return ''.join([ node.data for node in tree.childNodes
                         if node.nodeType == node.COMMENT_NODE ])
            
    def getText(self, tree):
        return ''.join([ node.data for node in tree.childNodes
                         if node.nodeType == node.TEXT_NODE ])

    def getCDATA(self, tree):
        return ''.join([ node.data for node in tree.childNodes
                         if node.nodeType == node.CDATA_SECTION_NODE ])

    def textNodeName(self):
        return '#text'
