
from parser import GenericParser, ParseError
try:
    import pyRXPU as pyRXP
except ImportError:
    import pyRXP

for opt in ['ReturnProcessingInstructions',
            'ReturnComments','ReturnCDATASectionsAsTuples']:
    pyRXP.parser_flags[opt] = 1

class Parser(GenericParser):

    def __init__(self, data, encoding):
        self.encoding = encoding
        if data == '':
            self.tree = None
        else:
            try:
                self.tree = pyRXP.Parser().parse(data)
            except pyRXP.Error,ex:
                raise ParseError(ex, data, encoding=encoding)

##         import pprint
##         if self.tree is not None:
##             pprint.pprint( self.tree)
        
    def getTree(self):
        assert self.tree is not None
        return self.tree
        
    def toXml(self, tree):
        normal = tree[0].startswith('<')
        if not normal:
            result = tree[0]
        else:
            result = '<%s' % tree[0]

        if tree[1] is not None:
            for attr, val in tree[1].iteritems():
                result += ' %s="%s"' % (attr, val)

        if tree[2] is not None:
            if normal:
                result += '>'
            for child in tree[2]:
                if type(child) == type(()):
                    result += self.toXml(child)
                else:
                    result += child
            if normal:
                result += '</%s>' % tree[0]
            elif tree[0] == '<![CDATA[':
                result += ']]'
            elif tree[0] == '<!--':
                result += '-->'
        else:
            result += '/>'
        return result

    def isTextNode(self, node):
        child = node[2]
        return type(child) == type([]) and type(child[0]) == type('')

    def isChild(self, node, parentNodeName, tree):
        if self.getNodeName(tree) == parentNodeName:
            return node in self.getChildren(tree)
        else:
            for child in self.getChildren(tree):
                if self.isChild(node, parentNodeName, child):
                    return True
        return False

    def getChildren(self, tree):
        children = tree[2]
        if children is None:
            return []
        return children

##     def getNodeType(self, node):
##         return node.nodeType

    def getNodeValue(self, node):
        if self.isTextNode(node):
            return unicode(node[2][0], self.encoding)
        return u''

    def getNodeName(self, node):
        return node[0]

    def getDirectChildrenWithName(self,parent, name):
        """ Fetch *direct* sub-nodes of a `parent` TREE tree. These
            nodes must have a name matching `name`.

            Return a list of DOM Nodes.
        """
        return [ node for node in self.getChildren(parent)
                 if self.getNodeName(node) == name ]

    def getValueForAttribute(self, node, attrName):
        attrs = node[1]
        if attrs is None:
            attrs = {}
        try:
            return attrs[attrName]
        except KeyError:
            raise AttributeError

    def getComments(self, tree):
        return ''.join([ self.getNodeValue(node) for node in self.getChildren(tree)
                         if self.getNodeName(node) == pyRXP.commentTagName ])
            
##     def getText(self, tree):
##         return ''.join([ self.getNodeValue(node) for node in self.getChildren(tree)
##                          if self.isTextNode(node) ])
    getText = getNodeValue

    
    def getCDATA(self, tree):
        return ''.join([ self.getNodeValue(node) for node in self.getChildren(tree)
                         if self.getNodeName(node) == pyRXP.CDATATagName ])
