import sys
#BUGS: ENTITY declatations not handeld correctly

from tag_parser import TokenParser,Handler
from xliff_builders import *
from transolution.xmlutils import createNodeAttr,NodeCreator,createDocument
from misc import unescapeCharRefEnt,unescapeHtmlEnt,unescape
from transolution.path_mod import path

class TagInfo(object):
    def __init__(self,name,type,grouped,trans_attrs,other_attrs):
        self.name = name
        self.type = type
        self.grouped = grouped
        self.trans_attrs = trans_attrs
        self.other_attrs = other_attrs
        
    def __str__(self):
        r = 'Tag=%s Type=%s Grouped=%s'%(self.name,self.type,self.grouped)
        r += 'Trans attrs=%s Other attrs=%s'%(self.trans_attrs,self.other_attrs)
        return r

def readConfig(fname):
    from ConfigParser import ConfigParser
    config = ConfigParser()
    config.readfp(file(fname))
    c = {}
    tags = config.items('Tags')
    c['tags'] = [(tag.replace('&colon;',':'), set) for (tag,set) in tags]
    c['default_tagstyle'] = config.get('FilterSettings','DefaultTagStyle')
    c['keep_whitespace'] = config.getboolean('FilterSettings','KeepLineBreaks')
    return c

##config = readConfig('STOffice6.ini')
##config = readConfig('transolution_HTML.ini')

def CreateTagInfoDict(tags):
    '@tags is a tuple (tag,value) from a ini file'
    tag_dict = {}
    for tag,flags in tags:
        flags = flags.split(',')
        trans_attrs = []
        other_attrs = []
        other_flags = []
        a = None
        for flag in flags:
            if flag.startswith('Translatable Attributes:'):
                a = trans_attrs.append
                a(flag[len('Translatable Attributes:'):])
            elif flag.startswith('Other Attributes:'):
                a = other_attrs.append
                a(flag[len('Other Attributes:'):])
            else:
                if a is None: other_flags.append(flag)
                else: a(flag)
        tag_dict[tag] = TagInfo(tag,flags[0],'Group' in other_flags,trans_attrs,other_attrs)
    return tag_dict

# token types
class PH(unicode):
    pass
class BPT(unicode):
    pass
class EPT(unicode):
    pass
class BIT(unicode):
    pass
class EIT(unicode):
    pass
class TEXT(unicode):
    pass
class EXTERNAL(unicode):
    pass

def normalize(tokens):
    'Normalize TEXT and EXTERNAL tokens'
    ret = []
    for token in tokens:
        if (token.__class__ in (TEXT,EXTERNAL)):
            if ret and (ret[-1].__class__ is token.__class__):
                ret[-1] = ret[-1].__class__(ret[-1] + token)
                continue
            else:
                ret.append(token)
        else:
            ret.append(token)
    return ret

import re
re_ent =  re.compile(r'&.+?;')

def markupEntities(tokens):
    'Markup entities in text tokens as PH'
    for token in tokens:
        if token.__class__ is TEXT:
            matches = re_ent.finditer(token)
            prev_end = 0
            for m in matches:
                yield TEXT(token[prev_end:m.start()])
                yield PH(token[m.start():m.end()])
                prev_end = m.end()
            yield TEXT(token[prev_end:])
        else:
            yield token

def fixOrpans(tokens):
    'Check and markup isolated tags'
    b_orphans = []
    e_orphans = []
    for i,token in enumerate(tokens):
        if token.__class__ is BPT:
            b_orphans.append(i)
        elif token.__class__ is EPT:
            try: b_orphans.pop()
            except IndexError:
                e_orphans.append(i)

    for i,token in enumerate(tokens):
        if i in b_orphans:
            yield BIT(token)
        elif i in e_orphans:
            yield EIT(token)
        else:
            yield token

ST_INT,ST_EXT = 1,2
class Sgml2XliffHandlerBase(Handler):
    def __init__(self,doc,datatype,tag_dict,keep_whitespace,default_tagstyle,skipwords):
        Handler.__init__(self)
        self.doc = doc
        self.datatype = datatype
        self.tag_dict = tag_dict
        self.keep_whitespace = keep_whitespace
        self.default_tagstyle = default_tagstyle
        self.skipwords = skipwords
        
        self.tu_id = 0
        self.tu_nodes = []
        self.id_stack = []
        self.tokens = []
        self.sklnodes = []
        self.state_stack = []
        self.ce = NodeCreator(self.doc).createElement
        self.textfilers = []

        self.textfilers.append(unescapeCharRefEnt)
        if datatype == "html":
            self.textfilers.append(unescapeHtmlEnt)
        elif datatype == "xml":
            self.textfilers.append(unescape)
            
    def createXliff(self,tokens):
        tmp_tokens = []
        tokens = normalize(tokens)
        tokens = markupEntities(tokens)
        skipwords = self.skipwords
        for token in tokens:
            if not (token.__class__ is EXTERNAL):
                # add internal tokens
                tmp_tokens.append(token)
            else:
                if tmp_tokens:
                    # sentence segmenatate tokens
                    segments = segmentate(tmp_tokens,skipwords)
                    for seg in segments:
                        if seg.__class__ is EXTERNAL:
                            self.addExternal(seg)
                        else:
                            # dont add empty TU segments
                            if ''.join(seg):
                                seg = fixOrpans(seg)
                                self.addTU(seg)
                tmp_tokens = []
                # Add External
                self.addExternal(token)

    def addExternal(self,text):
        node = self.doc.createElementNS(None,'formatting')
        node.appendChild(self.doc.createTextNode(text))
        self.sklnodes.append(node)

    def addTU(self,tokens):
        # create a tu and source nodes
        tu_node = TuNodeBuilder(self.doc,str(self.tu_id),translate='yes').node
        self.tu_nodes.append(tu_node)
        source_node = self.doc.createElementNS(None,'source')
        tu_node.appendChild(source_node)

        # add all tokens to source
        id_stack = []
        tag_id = 1
        for token in tokens:
            if token.__class__ is TEXT:
                node = self.doc.createTextNode(token)
            elif token.__class__ is BPT:
                node = self.ce('bpt',{'id':unicode(tag_id)},[unicode(token),])
                id_stack.append(tag_id)
                tag_id += 1
            elif token.__class__ is EPT:
                try: id = id_stack.pop()
                except IndexError,e:
                    print "ID error", token,self.tu_id
                    id = -1
                node = self.ce('ept',{'id':unicode(id)},[unicode(token),])
            elif token.__class__ is BIT:
                node = self.ce('it',{'id':unicode(tag_id),'pos':'open'},[unicode(token),])
                tag_id += 1
            elif token.__class__ is EIT:
                node = self.ce('it',{'id':unicode(tag_id),'pos':'close'},[unicode(token),])
                tag_id += 1
            elif token.__class__ is PH:
                node = self.ce('ph',{'id':unicode(tag_id)},[unicode(token),])
                tag_id += 1
            else:
                raise ValueError('unsupported token class:%s'%(type(token),) )
            #Add created node to source
            source_node.appendChild(node)

        # Add tu place holder to skeleton
        self.addSklTuPlaceHolder(self.tu_id)
        self.tu_id += 1

    def addSklTuPlaceHolder(self,id):
        node = self.doc.createElementNS(None,'tu-placeholder')
        createNodeAttr(node,'id',unicode(id))
        self.sklnodes.append(node)

class Sgml2XliffHandler(Sgml2XliffHandlerBase):
    def __init__(self,*args,**kwargs):
        Sgml2XliffHandlerBase.__init__(self,*args,**kwargs)
        
    def _isInternal(self,tag):
        try: ret = self.tag_dict[tag.lower()].type == 'Internal'
        except KeyError,e:
            ret = self.default_tagstyle == 'Internal'
        return ret

    def startDocument(self):
        pass
    def declaration(self,token):
        self.tokens.append(EXTERNAL(token))
    def xmlDeclaration(self,token):
        self.tokens.append(EXTERNAL(token))
    def processingInstruction(self,token):
        self.tokens.append(EXTERNAL(token))
    def error(self,token):
        self.tokens.append(EXTERNAL(token))


    def localize(self):
        if self.state_stack and self.state_stack[0] is ST_EXT:
            ret = False
        else:
            ret = True
        return ret

    def emptyTag(self,token):
        tag = token[:-2].split()[0][1:]
        if self.localize() and self._isInternal(tag):
            self.tokens.append(PH(token))
        else:
            self.tokens.append(EXTERNAL(token))

    def startTag(self,token):
        tag = token[:-1].split()[0][1:]
        try:
            if self.tag_dict[tag.lower()].grouped:
                if self._isInternal(tag):
                    self.state_stack.append(ST_INT)
                else:
                    self.state_stack.append(ST_EXT)
        except KeyError,e:
            pass

        # external?
        if self.localize() and self._isInternal(tag):
            self.tokens.append(BPT(token))
        else:
            self.tokens.append(EXTERNAL(token))
            
    def endTag(self,token):
        tag = token[2:-1]
        try:
            if self.tag_dict[tag.lower()].grouped:
                self.state_stack.pop()
        except KeyError,e:
            pass

        # External?
        if self.localize() and self._isInternal(tag):
            self.tokens.append(EPT(token))
        else:
            # external tag
            self.tokens.append(EXTERNAL(token))

    def text(self,token):
        if (self.localize()) and token.strip():
            for filter in self.textfilers:
                token = filter(token)
            if not self.keep_whitespace:
                #the X'es prevents any valid start/end whitespace between inline tags to be removed.
                token = ' '.join(("X"+ token+"X").split())[1:-1]#.strip()
            self.tokens.append(TEXT(token))
        else:
            self.tokens.append(EXTERNAL(token))

    def endDocument(self):
        self.createXliff(self.tokens)            

##re_seg =  re.compile(r'(\w{2,}[\.\!\?\:])(\s+|$)')
##re_seg =  re.compile(r'\w{2,}[\.\!\?\:]')
re_seg =  re.compile(r'([\.\!\?\:]\"?)(?=\s+|$)')

def getNextSegend(tokens,skipwords):
    # TODO: if a . is surrounded with tags eg. "If &self;.&classattr; is"
    #       it will segment.
    # precalc text
    texts = [token for token in tokens if token.__class__ is TEXT ]
    text = ''.join(texts)

    # get the segment positions
    segend = None
    start = 0
    max = len(text)
    # get the segment positions
    matches = re_seg.finditer(text)

    for match in matches:
        segend = match.end(1)
        lastpart = text[:segend].split()[-1]
        # if last part before segend is shorter then 1 char or in skipwords dont segment
        if len(lastpart) < 3 or lastpart in skipwords:
            continue
        yield segend
    if segend != max:
        yield max


def segmentate(tokens,skipwords):
    if not tokens:
        return []
    segments = []
    texts = []

    seg_tokens = []
    segends = getNextSegend(tokens,skipwords)
    # get first segend
    try:
        segend = segends.next()
    except StopIteration,e:
        segend = max


    tokens = iter(tokens)
    token = tokens.next()
    text_index = 0
    try:
        while True:
            if token.__class__ is TEXT:
                start = text_index
                text_index += len(token)
                # is there a segment end in token?
                if text_index > segend:
                    offset = segend - start
                    seg_tokens.append(token.__class__(token[:offset]))
                    # add segment
                    ws,seg_tokens = fixWhiteSpace(seg_tokens)
                    if ws: segments.append(ws)
                    segments.append(seg_tokens)
                    # reset segment
                    seg_tokens = []
                    # Set Next token to the rest
                    token = token.__class__(token[offset:])
                    # set current text index counter to segment end
                    text_index = segend
                    # get next segment end index
                    try:
                        segend = segends.next()
                    except StopIteration,e:
                        segend = max
                else:
                    # no segment end in token so just add it
                    seg_tokens.append(token)
                    token = tokens.next()
            else:
                seg_tokens.append(token)
                token = tokens.next()
    except StopIteration,e:
        if seg_tokens:
            ws,seg_tokens = fixWhiteSpace(seg_tokens)
            if ws: segments.append(ws)
            segments.append(seg_tokens)
                

    return segments

re_ws =  re.compile(r'^\s+')
def fixWhiteSpace(tokens):
    '''split any leding whitespace from @tokens and make it EXTERNAL
       @returns - tuple(external,remainingtokens)
    '''
    tokens = list(normalize(tokens))
    tmp_tokens = []
    token = tokens[0]
    m = re_ws.search(token)
    whitespace = None

    if (token.__class__ is TEXT) and m:
        whitespace = EXTERNAL(token[:m.end()])
        tmp_tokens.append(TEXT(token[m.end():]))
    else:
        tmp_tokens.append(token)

    if len(tokens) > 1:
        for token in tokens[1:]:
            tmp_tokens.append(token)
    return whitespace,tmp_tokens

def parseArgs():
    import sys
    from optparse import OptionParser

    usage = "usage: %prog [options] inifile path"

    op = OptionParser(usage=usage)
    op.add_option('-e', '--encoding',default='utf-8',
                  action='store', type='string', dest='encoding',
                  help='set source file encoding')
    op.add_option('-r', '--recursive',
                  action='store_true', dest='recursive',
                  help='Process files recursive.')
    op.add_option('-f', '--fmask',
                  action='store', type='string', dest='fmask',
                  help='File mask to use when running recursive.')
    op.add_option('-l', '--slang',default='en-US',
                  action='store', type='string', dest='slang',
                  help='Language of source document(s).')
    op.add_option('-s', '--skipwords',
                  action='store', type='string', dest='skipwords',
                  help='a file containing a the words not to segment after. Example: vs. Mr.')
    op.add_option('-z', '--xlz',
                  action='store_true', dest='xlz',
                  help='Create a xlz (zip file containing xliff and skeleton files).')


    options, args = op.parse_args(sys.argv[1:])
    try:
        inifile = path(args[0])
        pname = path(args[1])
    except IndexError,e:
        op.error("Invalid number if arguments, use -h for help")

        
    if not inifile.isfile():
        op.error("inifile path invalid:%s"%(inifile,))
    elif options.recursive:
        if not options.fmask:
            op.error("must give filemask option to when recurse")
        else:
            fnames = pname.walkfiles(options.fmask)
    else:
        if options.fmask:
            if not pname.isdir():
                op.error("must give a directory path when option fmask is given")
            else:
                fnames = pname.listdir(options.fmask)
        else:
            fnames = [path(f) for f in args[1:]]

    if options.skipwords:
        options.skipwords = [w.strip() for w in path(options.skipwords).lines(encoding='utf-8')]
    else:
        options.skipwords = []

    options.inifile = inifile
    options.fnames = fnames
    return options

def main():
    import sys
    import xliff_builders
    import xml_lexing
    from datetime import date
    tool_name = 'Transolution Generic sgml2xliff'
    tool_id = 'http://transolution.python-hosting.com'

    options = parseArgs()

    filterconfig = readConfig(options.inifile)
    tag_dict = CreateTagInfoDict(filterconfig['tags'])
    default_tagstyle = filterconfig['default_tagstyle']
    keep_whitespace = filterconfig['keep_whitespace']
    
    datatypes = {'.html':'html','.htm':'html','.xml':'xml','.sgml':'html','.sgm':'html'}

    encoding = options.encoding
    
    for fname in options.fnames:
        print fname
        try:
            datatype = datatypes[fname.ext]
        except KeyError,e:
            raise SystemExit('Unsupported content type:%s'%(fname.ext))
        print "datatype:%s"%(datatype,)

        if options.xlz:
            xlf_filename = path( 'content.xlf')
            skl_filename = path('skeleton.skl')
            fxlz = path(fname + '.xlz')
        else:
            xlf_filename = path(fname.splitext()[0] + '.xlf')
            skl_filename = path(fname.splitext()[0] + '.skl')

        outf = file(xlf_filename,'w')
        sklfile = file(skl_filename,'w')

        xlfdoc = createDocument('xliff', publicId="-//XLIFF//DTD XLIFF//EN",systemId="http://www.oasis-open.org/committees/xliff/documents/xliff.dtd")
        skldoc = createDocument('tt-xliff-skl', publicId="-//SUN-TRANSTECH//DTD XLIFF SKELETON//EN", systemId="http://www.ireland/~timf/tt-xliff-skl.dtd")
        createNodeAttr(xlfdoc.documentElement ,'version','1.1')
        createNodeAttr(skldoc.documentElement ,'version','1.0')


        # create Xliff
        builder = XliffBuilder(xlfdoc)

        ce = NodeCreator(xlfdoc).createElement
        filenode = FileNodeBuilder(xlfdoc,fname,options.slang,datatype,date=date.today().isoformat())

        # create header with skeleton info
        sklnode = ce('skl',childs=[ce('external-file',{'href':path(skl_filename).basename()}) ])
        propnode = ce('prop',{'type':'source-encoding'},[unicode(encoding),])
        propgroupnode = ce('prop-group',{'name':'sourceinfo'},[propnode,])
        toolnode = ce('tool',{'tool-id':tool_id,'tool-name':tool_name},[propgroupnode,])
        headernode = ce('header',childs=[sklnode,toolnode])
        filenode.addHeader(headernode)

        body = BodyNodeBuilder(xlfdoc)
        filenode.addBody(body)
        builder.addFile(filenode)

        # create skeleton dom
        skeleton = SkeletonBuilder(skldoc,fname,options.slang,'xml',xlf_filename)


        print "Parsing"
        #parse sgml/xml    
        tokens = xml_lexing.lexxml(path(fname).text(encoding))

        print "  Building XLIFF"
        handler = Sgml2XliffHandler(xlfdoc,datatype,tag_dict,keep_whitespace,default_tagstyle,options.skipwords)
        parser = TokenParser(handler)
        parser.parse(tokens)

        for tu_node in handler.tu_nodes:
            body.addTu(tu_node)
        for skl_node in handler.sklnodes:
            skeleton.addToBody(skl_node)
        print "  Writing XLIFF"
        transolution.xmlutils.toxml(xlfdoc,outf)
        transolution.xmlutils.toxml(skeleton.doc,sklfile)
        outf.close()
        sklfile.close()
        if options.xlz:
            # create a xlz archive.
            import zipfile
            xlz = zipfile.ZipFile(fxlz,'w',zipfile.ZIP_DEFLATED)
            xlz.write(xlf_filename,'content.xlf')
            xlz.write(skl_filename,'skeleton.skl')
            xlz.close()
            xlf_filename.remove()
            skl_filename.remove()
        print "  Done"
        del xlfdoc
        del skldoc

if __name__ == "__main__":
    main()