'Backconvert xliff files to tagged format.'
from transolution.xliffeditor.document.segments import TuSegment,External
from transolution.xliffeditor.xliffparser_ng import XliffDocument,XliffException
from transolution.xmlutils import XmlError,getText
from transolution.xmlutils import getNodeAttr,setNodeAttr,getNodes,getElementsByTagName

from transolution.path_mod import path

from xml.sax.saxutils import escape

def getTextAndEscape(nodelist,recursive=True):
    rc = []
    if hasattr(nodelist,"nodeType"):
        raise ValueError("not a nodelist")

    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            if node.parentNode.nodeName in ('btp','ept','ph'):
                rc.append(node.data)
            else:
                rc.append(escape(node.data))
        elif node.nodeType == node.ELEMENT_NODE:
            if recursive:
                rc += getText(node.childNodes)

    return ''.join(rc)

def getEncoding(headernode):
    'Get tool specific encoding information'
    try:
        tool_node, = [node for node in getNodes(headernode,'tool','+')]
        prop_node, = [node for node in getNodes(tool_node,'prop-group','+') if getNodeAttr(node,'name') == 'sourceinfo']
    except TypeError,e:
        encoding = None
    else:
        enc_node = [node for node in getNodes(prop_node,'prop','+') if getNodeAttr(node,'type') == 'source-encoding'][0]
        encoding = getText(enc_node.childNodes)
    return encoding

def parseArgs():
    import sys
    from optparse import OptionParser

    usage = "usage: %prog [options] path"

    op = OptionParser(usage=usage)
    op.add_option('-r', '--recursive',
                  action='store_true', dest='recursive',
                  help='Process files recursive.')
    op.add_option('-i', '--inifile',
                  action='store', type='string', dest='inifile',
                  help='filter inifile to use')
    op.add_option('-e', '--encoding',
                  action='store', type='string', dest='encoding',
                  help='set out file encoding')
    op.add_option('-f', '--fmask',
                  action='store', type='string', dest='fmask',
                  help='File mask to use when running recursive.')


    options, args = op.parse_args(sys.argv[1:])
    try:
        pname = path(args[0])
    except IndexError,e:
        op.error("Invalid number if arguments, use -h for help")
    
    if options.inifile:
        options.inifile = path(options.inifile)
        if not options.inifile.isfile():
            op.error("inifile path invalid:%s"%(inifile,))
    if options.recursive:
        if not options.fmask:
            op.error("must give filemask option to when recurse")
        else:
            fnames = pname.walkfiles(options.fmask)
    else:
        if options.fmask:
            if not pname.isdir():
                op.error("must give a directory path when option fmask is given")
            else:
                fnames = pname.listdir(options.fmask)
        else:
            fnames = [path(f) for f in args]

    options.fnames = fnames
    return options


def main():
    import sys
    import codecs
    
    contenttypes = {'.html':'html','.htm':'html','.xml':'xml'}
    options = parseArgs()
    encoding = options.encoding
    defult_encoding = 'utf-8'
    
    for fname in options.fnames:
        print 'Processing:%s -->'%(fname,)
        xliff = XliffDocument(fname)
        xliff.parse()
        for filenode in xliff.files:
            if not encoding:
                # Try and get encoding property from xliff file
                encoding = getEncoding(filenode.header.node)
                if not encoding:
                    encoding = defult_encoding
            filenode.parseSkeleton()
            doc = xliff.xliffdoc
            outfile = path(fname.parent) / path(filenode.getOriginal()).name
            print '  ' + outfile
            outfile_obj = codecs.open(outfile,"w",encoding)

            for seg in filenode.generateSegments():
                if seg.__class__ is TuSegment:
                    text = getTextAndEscape(seg.getNode().childNodes)
                    outfile_obj.write(text)
                elif seg.__class__ is External:
                    text = getText(seg.getNode().childNodes)
                    outfile_obj.write(text)
            outfile_obj.close()

if __name__ == "__main__":
    main()