import sys
import transolution.xmlutils
from transolution.xmlutils import NodeCreator,createNodeAttr,EMPTY_NAMESPACE,dom_impl

##try:
##    from Ft.Xml import EMPTY_NAMESPACE
##    from Ft.Xml.Domlette import cDomlette as impl
##    
##except ImportError,e:
##    print "MINIDOM"
##    from xml.dom.minidom import getDOMImplementation
##    impl = getDOMImplementation()
##    EMPTY_NAMESPACE = None
##else:

##impl = transolution.xmlutils.dom_impl
##doc = impl.createDocument(EMPTY_NAMESPACE, "tt-xliff-skl", None)


def args2Attr(**kwargs):
    'convert underscore to hyphens as keyword args cant contain hyphen'
    new_kwargs = {}
    for key,val in kwargs.items():
        new_kwargs[key.replace('_','-')] = val
    return new_kwargs

class XliffError(Exception):pass

class SkeletonBuilder(NodeCreator):
    def __init__(self,doc,original, source_language, datatype,xliff):
        NodeCreator.__init__(self,doc)
        ce = self.createElement
        node = ce('file',{'source-language':source_language,
                          'original':original,
                          'datatype':datatype}
                  )
##        node = self.doc.createElementNS(None,'file')
##        createNodeAttr(node,'source-language',source_language)
##        createNodeAttr(node,'original',original)
##        createNodeAttr(node,'datatype',datatype)


        createNodeAttr(node,'datatype',datatype)

        self.doc.documentElement.appendChild(node)
        self.filenode = node        

##        node = self.doc.createElementNS(None,'header')

        self.body_node = self.doc.createElementNS(None,'body')
        
        self.filenode.appendChild(self.body_node)

    def addToBody(self,node):
        self.body_node.appendChild(node)
        


class XliffBuilder(NodeCreator):
    def __init__(self,doc):
        NodeCreator.__init__(self,doc)

##        self.xlfdoc = impl.createDocument(EMPTY_NAMESPACE, "xliff", None)

    def addFile(self,o):
        self.doc.documentElement.appendChild(o.node)

class FileNodeBuilder(NodeCreator):
    def __init__(self,doc,original,source_language,datatype,**kwargs):
        NodeCreator.__init__(self,doc)
        self.node = None
        self.header_node = None
        self.body_node = None
        self._createFileNode(original,source_language,datatype,**kwargs)
        
    def _createFileNode(self, original, source_language, datatype, **kwargs):
        optional_args = ['tool', 'tool-id', 'date', 'xml:space', 'ts', 'category', 'target-language', 'product-name', 'product-version', 'build-num']

        # convert kwargs
        kwargs = args2Attr(**kwargs)

        # create filenode
        node = self.node = self.doc.createElementNS(None,'file')
        # add required attrs
        createNodeAttr(node,'original',original)
        createNodeAttr(node,'source-language',source_language)
        createNodeAttr(node,'datatype',datatype)

        # add optional attrs
        for attr,val in kwargs.items():
            if attr not in optional_args:
                raise XliffError('Illegal file node attribute: %s'%(attr,))
            createNodeAttr(node,attr,val)

    def addHeader(self,node):
        # create filenode
        self.header_node = node
        self.node.appendChild(self.header_node)
        
    def addBody(self,o):
        # create filenode
        assert not (self.header_node is None),'header node must be added first'
        assert self.body_node is None,'file can only have one header node'
        self.body_node = o.node
        self.node.appendChild(self.body_node)

class HeaderNodeBuilder(NodeCreator):
    def __init__(self,doc):
        NodeCreator.__init__(self,doc)
        self.node = None
        self._createNode()
        
    def _createNode(self):
        # create filenode
        self.node = self.doc.createElementNS(None,'header')
        
    def addSkl(self,o):
        # create filenode
        self.node.appendChild(o.node)
        
class BodyNodeBuilder(NodeCreator):
    def __init__(self,doc):
        NodeCreator.__init__(self,doc)
        self.node = None
        self._createNode()
        
    def _createNode(self):
        # create filenode
        self.node = self.createElement('body')
        
    def addTu(self,node):
        # create filenode
        self.node.appendChild(node)
        

class TuNodeBuilder(NodeCreator):
    optional_args = [
        "approved", "translate", "reformat", "xml:space","datatype",
        "ts", "phase-name", "restype", "resname", "extradata", "help-id",
        "menu", "menu-option", "menu-name", "coord", "font", "css-style",
        "style", "exstyle", "extype", "maxbytes", "minbytes",  "size-unit",
        "maxheight", "minheight", "maxwidth", "minwidth", "charclass"
        ]
    def __init__(self,doc,id,**kwargs):
        NodeCreator.__init__(self,doc)
        self.node = None
        self.source_node = None
        self.target_node = None
        self._createNode(id,**kwargs)
        
    def _createNode(self, id, **kwargs):
        # convert kwargs
        kwargs = args2Attr(**kwargs)

        # create filenode
        self.node = self.createElement('trans-unit',attrs=kwargs)
        # add required attrs
        createNodeAttr(self.node,'id',id)
        # add optional attrs
        for attr,val in kwargs.items():
            if attr not in self.optional_args:
                raise XliffError('Illegal file node attribute: %s'%(attr,))
##            createNodeAttr(node,attr,val)


if __name__ == "__main__":
    # createDocument(namespaceURI, qualifiedName, doctype):
    builder = XliffBuilder()
    doc = builder.xlfdoc
    filenode = FileNodeBuilder(doc,'test.xml','en-US','xml',date='2004-12-15')
    header = HeaderNodeBuilder(doc)
    body = BodyNodeBuilder(doc)
    for id in range(10):
        tu = TuNodeBuilder(doc,str(id),translate='yes')
        tu.addSource(SourceNodeBuilder(doc))
        tu.addTarget(TargetNodeBuilder(doc))
        body.addTu(tu)

    filenode.addHeader(header)
    filenode.addBody(body)

    builder.addFile(filenode)

    transolution.xmlutils.toxml(builder.xlfdoc,sys.stdout)

