#! /usr/bin/env python
#Here's the code for the unit tests, which also serves as usage examples.  It should go in a file named test_observer.py:
import unittest
from observer import Observable, Observer

EVENT_FOO, EVENT_UPDATE = range(2)

class Stuff(Observable):

    def __init__(self):
        Observable.__init__(self)
        self._data = None

    def setData(self, data):
        self._data = data
        self.notifyObservers(None, data)

    def setDataWithUpdateEvent(self, data):
        self._data = data
        self.notifyObservers(EVENT_UPDATE, data)

    def setDataWithFooEvent(self, data):
        self._data = data
        self.notifyObservers(EVENT_FOO, data)


class StuffWatcher(Observer):

    def __init__(self, observable=None, cbname=None, events=None):
        Observer.__init__(self, observable, cbname, events)
        self._data = None
        self._updateData = None
        self._reportData = None

    def update(self, observable, event, msg):
        self._data = msg
        self._updateData = msg

    def report(self, observable, event, msg):
        self._data = msg
        self._reportData = msg


class TestCase_01_Observer(unittest.TestCase):

    def test_01_noMethod(self):
        observable = Stuff()
        observer1 = StuffWatcher(observable, "foo")
        self.assertRaises(NotImplementedError, observable.setData, 10)

    def test_02_simple(self):

        observable = Stuff()
        observer1 = StuffWatcher(observable)
        observer2 = StuffWatcher(observable)
        self.assertEqual(len(observable._observers), 2)

        observable.setData(10)
        self.assertEqual(observer1._data, 10)
        self.assertEqual(observer2._data, 10)

        observable.setData(20)
        self.assertEqual(observer1._data, 20)
        self.assertEqual(observer2._data, 20)

        del observer1
        self.assertEqual(len(observable._observers), 1)

        observable.setData(30)
        self.assertEqual(observer2._data, 30)

        observer3 = StuffWatcher()
        self.assertEqual(len(observable._observers), 1)
        observer3.subscribeToObservable(observable)
        self.assertEqual(len(observable._observers), 2)
 
        observable.setData(40)
        self.assertEqual(observer2._data, 40)
        self.assertEqual(observer3._data, 40)

        del observer2
        self.assertEqual(len(observable._observers), 1)

        observable.setData(50)
        self.assertEqual(observer3._data, 50)

    def test_03_specificEvents(self):

        observable = Stuff()
        observer1 = StuffWatcher(observable, events=EVENT_UPDATE)
        observer2 = StuffWatcher(observable, events=EVENT_FOO)

        observable.setDataWithUpdateEvent(10)
        self.assertEqual(observer1._data, 10)
        self.assertEqual(observer2._data, None)

        observable = Stuff()
        observer1 = StuffWatcher(observable, events=(EVENT_UPDATE, EVENT_FOO))

        observable.setDataWithUpdateEvent(10)
        self.assertEqual(observer1._data, 10)

        observable.setDataWithFooEvent(20)
        self.assertEqual(observer1._data, 20)

    def test_03_multipleObservables(self):
        observable1 = Stuff()
        observable2 = Stuff()
        observer = StuffWatcher()

        observer.subscribeToObservable(observable1, "update", EVENT_UPDATE)
        observer.subscribeToObservable(observable2, "report", EVENT_FOO)
        self.assertEqual(len(observable1._observers), 1)
        self.assertEqual(len(observable2._observers), 1)

        observable1.setDataWithUpdateEvent(10)
        self.assertEqual(observer._data, 10)
        self.assertEqual(observer._updateData, 10)
        self.assertEqual(observer._reportData, None)

        observable1.setDataWithFooEvent(20)
        self.assertEqual(observer._data, 10)
        self.assertEqual(observer._updateData, 10)
        self.assertEqual(observer._reportData, None)

        observable2.setDataWithUpdateEvent(30)
        self.assertEqual(observer._data, 10)
        self.assertEqual(observer._updateData, 10)
        self.assertEqual(observer._reportData, None)

        observable2.setDataWithFooEvent(40)
        self.assertEqual(observer._data, 40)
        self.assertEqual(observer._updateData, 10)
        self.assertEqual(observer._reportData, 40)


if __name__ == "__main__":
    unittest.main()
