#!/usr/bin/env python

import sys
import codecs
from Ft.Xml.Domlette import implementation, PrettyPrint
from Ft.Xml import EMPTY_NAMESPACE,XML_NAMESPACE
import poparse

poenc = "utf-8"
src_lang = u"en"
trg_lang = u"sv-se"

def addTMXHeader(doc,parent,entry):
    header = doc.createElementNS(EMPTY_NAMESPACE,"header")
    if len(entry.xlatComment) > 0:
        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
        str_note = u"".join(entry.xlatComment).strip()
        note_txt = doc.createTextNode(str_note)
        note.appendChild(note_txt)
        header.appendChild(note)
    if len(entry.autoComment) > 0:
        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
        str_note = u"".join(entry.autoComment).strip()
        note_txt = doc.createTextNode(str_note)
        note.appendChild(note_txt)
        header.appendChild(note)
    if len(entry.refComment) > 0:
        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-ref")
        str_prop = u"".join(entry.refComment).strip()
        prop_txt = doc.createTextNode(str_prop)
        prop.appendChild(prop_txt)
        header.appendChild(prop)
    if len(entry.procComment) > 0:
        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-pi")
        str_prop = u"".join(entry.procComment).strip()
        prop_txt = doc.createTextNode(str_prop)
        prop.appendChild(prop_txt)
        header.appendChild(prop)
    parent.appendChild(header)
    
    


def  addTMXTu(doc,parent,entry):
    tu = doc.createElementNS(EMPTY_NAMESPACE,"tu")
    if len(entry.xlatComment) > 0:
        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
        str_note = u"".join(entry.xlatComment).strip()
        note_txt = doc.createTextNode(str_note)
        note.appendChild(note_txt)
        tu.appendChild(note)
    if len(entry.autoComment) > 0:
        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
        str_note = u"".join(entry.autoComment).strip()
        note_txt = doc.createTextNode(str_note)
        note.appendChild(note_txt)
        tu.appendChild(note)
    if len(entry.refComment) > 0:
        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-ref")
        str_prop = u"".join(entry.refComment).strip()
        prop_txt = doc.createTextNode(str_prop)
        prop.appendChild(prop_txt)
        tu.appendChild(prop)
    if len(entry.procComment) > 0:
        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-pi")
        str_prop = u"".join(entry.procComment).strip()
        prop_txt = doc.createTextNode(str_prop)
        prop.appendChild(prop_txt)
        tu.appendChild(prop)
    tuv = doc.createElementNS(EMPTY_NAMESPACE,"tuv")
    tuv.setAttributeNS(XML_NAMESPACE,'xml:lang',src_lang)
    seg = doc.createElementNS(EMPTY_NAMESPACE,"seg")
    seg_text = doc.createTextNode(entry.msgid)
    seg.appendChild(seg_text)
    tuv.appendChild(seg)
    tu.appendChild(tuv)
    tuv = doc.createElementNS(EMPTY_NAMESPACE,"tuv")
    tuv.setAttributeNS(XML_NAMESPACE,'xml:lang',trg_lang)
    seg = doc.createElementNS(EMPTY_NAMESPACE,"seg")
    seg_text = doc.createTextNode(entry.msgstr)
    seg.appendChild(seg_text)
    tuv.appendChild(seg)
    tu.appendChild(tuv)
    
    parent.appendChild(tu)

try:
    if len(sys.argv) != 3:
        raise Exception, 'Usage: po2xml pofile.po tmxfile.tmx'
    
    fd = codecs.open(sys.argv[1],"r",poenc)
    
    impl = implementation
    dom = impl.createDocument(EMPTY_NAMESPACE, "tmx", None)
    doc = dom.documentElement
    doc.setAttributeNS(EMPTY_NAMESPACE,"version","1.4")
    
    numents = 0
    have_header = False
    body = None
    
    for entry in poparse.getPoEntries(fd):
        if len(entry.msgid) == 0:
            if have_header is not True:
                addTMXHeader(dom,doc,entry)
                have_header = True
                print "Got po header\n"
            else:
                print "Invalid po file: Got extra header\n"
        else:
            if body is None:
                body = dom.createElementNS(EMPTY_NAMESPACE,"body")
                doc.appendChild(body)
            numents = numents + 1
            addTMXTu(dom,body,entry)
    print "Got " + str(numents) + " po entries"
    fd.close()
    tmxf = open(sys.argv[2],"w")
    PrettyPrint(dom,stream=tmxf)
    tmxf.close()
    
except Exception, theError:
    print theError
