import sys
port = 6315

from transolution.xmlutils import toxml,parseString

from difflib import ndiff
from transolution.tm.lookup import fuzzyLookup
# DONE! TODO: a GTK textvindow to show match diff 
# TODO: TMX Load/Save
# DONE! TODO: Freestanding server communicatin through xml-rpc
# TODO: markers are not copied into result target as they lack id
# TODO: markers and group tags should not be treated as tags

def getPlainText(nodelist):
    text_nodes = [node.data for node in nodelist if node.nodeType == node.TEXT_NODE]
    return ''.join(text_nodes)


class TMException(Exception):pass

class TMServer(object):
    def __init__(self):
        self.tus = {}

    def version(self):
        return 'Dirty little TM server v0.01'

    def addTU(self,source,target):
        'add source and target xml strings, index them as plain text without tag info'
        source = parseString(source.encode('utf-8')).firstChild
        target = parseString(target.encode('utf-8')).firstChild
        stext = getPlainText(source.childNodes)
        print "addTU:",stext.encode('utf-8')
####        source = shallowNodeCopy(source)
####        target = shallowNodeCopy(target)
        self.tus[stext] = source,target
        return True

    def lookup(self,lookup):
        'return best match'
        lookup = parseString(lookup.encode('utf-8')).firstChild
        stext = getPlainText(lookup.childNodes)
        try:
            #Try find exact match
            tm_source,tm_target = self.tus[stext]
            quality=1.0
            return quality,toxml(tm_source),toxml(tm_target)
        except KeyError,e:
            #Try find fuzzy match
            for m in fuzzyLookup( stext, self.tus.keys(),cutoff=0.7):
                quality,s = m
                tm_source,tm_target = self.tus[s]
                return quality,toxml(tm_source),toxml(tm_target)
        raise TMException('No match found')
##import sys
##sys.path.insert(0, '..')
class TMException(Exception):pass

def main():
    import SimpleXMLRPCServer
    tm = TMServer()
    server = SimpleXMLRPCServer.SimpleXMLRPCServer(("localhost", port))
    server.register_instance(tm)

    #Go into the main listener loop
    print "TM server listening on port",port
    server.serve_forever()
if __name__ == '__main__':
    main()

##    xml1 = r'''<seg>first <bpt id="1" >xx</bpt>in tag<ept id="1">xx</ept> some more text</seg>'''
##    xml2 = r'''<seg>first source <bpt id="1" >&lt;tag&gt;</bpt>in tag<ept id="1">&lt;/tag&gt;</ept> some more text</seg>'''
##    xml3 = r'''<seg>first source in tag some more text</seg>'''
##    tm = TM()
##    
##    print tm.addTU(xml1,xml2)
##    print "--------"
##    print tm.lookup(xml3)
