import pygtk
#tell pyGTK, if possible, that we want GTKv2
pygtk.require("2.0")

import gtk
import gtk.glade
import gobject
import transolution.tm.server
from transolution.xliffeditor.gtk_utils import CustomHandler,askOk,messageDialog,aboutDialog,cb_get_active_text,entryDialog,create_tag_table,fileChooserDialog
import os

# I18n support
import gettext
domain='transolution'
localePath=gettext.bindtextdomain(domain)
gettext.install(domain, localePath, unicode=1)
gtk.glade.bindtextdomain(domain, localePath)

#   columns
(
  COLUMN_SELECTED,
  COLUMN_TMX,
) = range(2)


def get_active_text(combobox):
    model = combobox.get_model()
    active = combobox.get_active()
    if active < 0:
        return None
    return model[active][0]



class TmGuiBase(CustomHandler):
    def __init__(self, parent=None):
        CustomHandler.__init__(self)
        # Load Glade GUI
        windowname="MainWindow"
        # Create the toplevel window
        print transolution.tm.server.gladefile
        self.wTree=gtk.glade.XML(transolution.tm.server.gladefile,windowname,domain)
        self.window = self.wTree.get_widget(windowname)
        
        self.window.connect('destroy', lambda *w: gtk.main_quit())
        # enable Drag/Drop
        self.window.drag_dest_set(gtk.gdk.ACTION_DEFAULT, [], 0)
        self.window.connect('drag_drop', self.target_drag_drop)
        self.window.connect('drag_data_received', self.target_drag_data_received)

        w = self.wTree.get_widget
        # get the widgets we need access to
        self.gui_log = w("logview")
        self.source_lang_cb = w("source_lang")
        self.target_lang_cb = w("target_lang")
        self.tmx_list_view = w("tmx_list_view")
        self.server_port_entry = w("server_port_entry")

        # connect
        self.wTree.signal_autoconnect(self)
        self.window.show_all()

    def getComboBox(self,str1, str2, int1, int2):
        return gtk.combo_box_entry_new_text()

    def target_drag_drop(self, widget, context, x, y, time):
        print 'drop'
        if context.targets:
            widget.drag_get_data(context, context.targets[0], time)
            return True
        return False

    def target_drag_data_received(self, widget, context, x, y, selection, targetType,time):
        if selection.format == 8:
            for fname in selection.data.split('\n'):
                print fname.strip()
            context.finish(True, False, time)
        else:
            context.finish(False, False, time)

    def getTaskComboBox(self,str1, str2, int1, int2):
        'custom widget provider'
        return gtk.combo_box_new_text()

    def getListStore(self,str1, str2, int1, int2):
        'custom widget provider'
        liststore = gtk.ListStore(bool, str)
        view = gtk.TreeView(liststore)
        self.__add_columns(view)
        view.set_enable_search(True)

        self.tmx_list_store = liststore
        return view

    def __add_columns(self, treeview):
        model = treeview.get_model()

        # column for select toggle
        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.selected_toggled, model)
        column = gtk.TreeViewColumn('Use', renderer, active=COLUMN_SELECTED)
        treeview.append_column(column)

        # column for job number
        column = gtk.TreeViewColumn('TMX file paths', gtk.CellRendererText(),
                                    text=COLUMN_TMX)
        column.set_sort_column_id(COLUMN_TMX)
        treeview.append_column(column)

import sys
from transolution.path_mod import path
from transolution import getLangCodes
import estreen

class TmGui(TmGuiBase):
    def __init__(self, *args, **kwargs):
        TmGuiBase.__init__(self,*args,**kwargs)
        self.last_open_folder = None
        self.log = self.gui_log.get_buffer()
        sys.stdout = self
        sys.stderr = self
        self.server_port_entry.set_text(str(estreen.port))

        try:
            drop_down_values = ['%s (%s)'%(lcid,country) for lang,country,lcid in list(getLangCodes())]
            drop_down_values.sort()
        except IOError,e:
            drop_down_values = None
        for v in drop_down_values:
            self.source_lang_cb.append_text(v)
            self.target_lang_cb.append_text(v)

    def onAddTMX(self,*args):
        filefilters = (("TMX Files (.tmx)",("*.tmx",)),
                       ("All files",("*",),))
        fnames = fileChooserDialog(title="Add...", filefilters=filefilters,folder=self.last_open_folder)
        if fnames:
            self.last_open_folder = os.path.split(fnames[0])[0]
            for fname in fnames:
                self.tmx_list_store.append([True,fname])
        return True

    def onRemoveTMX(self,*args):
        'Remove a note'
        #TODO: implement delete note
        selection = self.tmx_list_view.get_selection()
        model, iter = selection.get_selected()
        if iter:
            self.tmx_list_store.remove(iter)
        return True

    def write(self,text):
        'write to log window'
        gtk.threads_enter()
        self.log.insert(self.log.get_end_iter(),text)
        self.log.place_cursor(self.log.get_end_iter())
        self.gui_log.scroll_mark_onscreen(self.log.get_insert())
        gtk.threads_leave()
        
    def selected_toggled(self, cell, path, model):
        # get toggled iter
        iter = model.get_iter((int(path),))
        selected = model.get_value(iter, COLUMN_SELECTED)

        # do something with the value
        selected = not selected

        # set new value
        model.set(iter, COLUMN_SELECTED, selected)


    def onRunClicked(self,*args):
        print "Run Thread!"
        class InvalidTMServerOptions(Exception): pass
        class Options:
            slang = None
            tlang = None
            fnames = None
            port = None
            def validate(self):
                try:
                    self.port = int(self.port)
                except (TypeError,ValueError),e:
                    raise InvalidTMServerOptions('Port number not an integer:%s'%(self.port,))

                if not (self.slang and self.tlang):
                    raise InvalidTMServerOptions('Source and Target language must be set')
                elif self.slang == self.tlang:
                    raise InvalidTMServerOptions('Source and Target language cant be the same')
                for fname in self.fnames:
                    if not path(fname).isfile():
                        raise InvalidTMServerOptions('TMX path not valid:%s'%(fname,))
                    
        options = Options()
        options.slang = self.source_lang_cb.child.get_text().strip()
        options.tlang = self.target_lang_cb.child.get_text().strip()
        # get all TMX files in list view that are active
        options.fnames = [row[COLUMN_TMX] for row in self.tmx_list_store if row[COLUMN_SELECTED]]
        options.port = self.server_port_entry.get_text()
        
        try:
            options.validate()
        except InvalidTMServerOptions,e:
            self.alertException(e,'Error')
        else:
            self.runTmThread(options)
        return True

    def runTmThread(self,*args):
        import threading
        import estreen as tmserver
        server_thread = threading.Thread(target = tmserver.main, args = args)
##        pybthread.setDaemon(False)
        server_thread.start()
        self.server_thread = server_thread


    def serverRunning(self,val):
        'setup gui accrodingly to if pyb process is running'
        gtk.threads_enter()
        self.gui_main.set_sensitive(not val)
        gtk.threads_leave()

       
    def alertException(self,e,msg):
        messageDialog(self.window,"%s:%s"%(msg,e))


    
if __name__ == '__main__':
    server_gui = TmGui()
    mainwin = server_gui.window
    mainwin.show_all()
    gtk.threads_init()
    gtk.threads_enter()

    gtk.main()
    gtk.threads_leave()
