__doc__ = '''TMX Module'''

from xml.dom import Node
from transolution.xmlutils import NodeCreator,createNodeAttr,dom_impl,loadXml,toxml
from transolution.xmlutils import EMPTY_NAMESPACE,XML_NAMESPACE

class TMXException(Exception):
    pass
class TMXSyntaxError(TMXException):
    pass

class tmxNote:
    def __init__(self,doc = None):
        self.lang = ''
        self.oencoding = ''
        self.data = ''
        if doc.nodeName == u'note':
            doc.normalize()
            self.lang = doc.getAttributeNS(XML_NAMESPACE,'lang') or doc.getAttributeNS(EMPTY_NAMESPACE,'lang')
            self.oencoding = doc.getAttributeNS(EMPTY_NAMESPACE,'o-encoding')
            for str in doc.childNodes:
                if str.nodeType == Node.TEXT_NODE or str.nodeType == Node.CDATA_SECTION_NODE:
                    self.data += str.data
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected note\n")


class tmxProp:
    def __init__(self,doc = None):
        self.type = ''
        self.lang = ''
        self.oencoding = ''
        self.data = ''
        if doc.nodeName == u'prop':
            doc.normalize()
            self.lang = doc.getAttributeNS(XML_NAMESPACE,'lang') or doc.getAttributeNS(EMPTY_NAMESPACE,'lang')
            self.oencoding = doc.getAttributeNS(EMPTY_NAMESPACE,'o-encoding')
            self.type = doc.getAttributeNS(EMPTY_NAMESPACE,'type')
            for str in doc.childNodes:
                if str.nodeType == Node.TEXT_NODE or str.nodeType == Node.CDATA_SECTION_NODE:
                    self.data += str.data
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected prop\n")
 
class tmxUde:
    def __init__(self,doc = None):
        self.name = ''
        self.base = ''
        self.maps = []
        if doc.nodeName == u'ude':
            pass
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected ude\n")

class tmxMap:
    def __init__(self,doc = None):
        self.unicode = ''
        self.code = ''
        self.ent = ''
        self.subst = ''
        if doc.nodeName == u'map':
            pass
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected map\n")

class tmxHeader:
    def __init__(self,doc = None):
        self.adminlang = ''
        self.changedate = ''
        self.changeid = ''
        self.creationdate = ''
        self.creationid = ''
        self.creationtool = ''
        self.creationtoolversion = ''
        self.datatype = ''
        self.oencoding = ''
        self.otmf = ''
        self.segtype = ''
        self.srclang = ''
        self.tuid = ''
        self.notes = []
        self.props = []
        self.udes = []
        if doc.nodeName == u'header':
            pass
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected header\n")

class tmxTu:
    def __init__(self,doc = None):
        self.changedate = ''
        self.changeid = ''
        self.creationdate = ''
        self.creationid = ''
        self.creationtool = ''
        self.creationtoolversion = ''
        self.datatype = ''
        self.lastusagedate = ''
        self.oencoding = ''
        self.otmf = ''
        self.segtype = ''
        self.srclang = ''
        self.tuid = ''
        self.usagecount = 0
        self.notes = []
        self.props = []
        self.tuvs = []
        if doc.nodeName == u'tu':
            for child in doc.childNodes:
                if child.nodeType != Node.ELEMENT_NODE:
                    continue
                if child.nodeName == u'note':
                    self.notes.append(tmxNote(child))
                elif child.nodeName == u'prop':
                    self.props.append(tmxProp(child))
                elif child.nodeName == u'tuv':
                    self.tuvs.append(tmxTuv(child))
                else:
                    raise TMXSyntaxError("Invalid TMX Syntax, uexpected element in tu\n")
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected tu\n")
    def getKey(self,lang):
        key = []
        lang = lang.upper()
        retkey = []
        for tuv in self.tuvs:
            if tuv.lang.upper() == lang:
                for word in tuv.seg.data.split():
                    key.append(hash(word))
        if not key: raise TMXException('No tuv in tu matches languagecode')
        key.sort()
        retkey.append(key[0])
        for elem in key:
            if retkey[-1] != elem:
                retkey.append(elem)
        return retkey

class tmxTuv:
    def __init__(self,doc = None):
        self.changedate = ''
        self.changeid = ''
        self.creationdate = ''
        self.creationid = ''
        self.creationtool = ''
        self.creationtoolversion = ''
        self.datatype = ''
        self.lang = ''
        self.lastusagedate = ''
        self.oencoding = ''
        self.otmf = ''
        self.segtype = ''
        self.srclang = ''
        self.tuid = ''
        self.usagecount = 0
        self.notes = []
        self.props = []
        self.seg = None
        if doc.nodeName == u'tuv':
            for child in doc.childNodes:
                if child.nodeType != Node.ELEMENT_NODE:
                    continue
                if child.nodeName == u'note':
                    self.notes.append(tmxNote(child))
                elif child.nodeName == u'prop':
                    self.props.append(tmxProp(child))
                elif child.nodeName == u'seg':
                    self.seg = tmxSeg(child)
                else:
                    raise Exception, "Invalid TMX Syntax, uexpected element in tuv\n"

                if doc.hasAttributeNS(XML_NAMESPACE,'lang'):
                    self.lang = doc.getAttributeNS(XML_NAMESPACE,'lang')
                elif doc.hasAttributeNS(EMPTY_NAMESPACE,'lang'):
                    self.lang = doc.getAttributeNS(EMPTY_NAMESPACE,'lang')
                else:
                    self.lang = None
                self.oencoding = doc.getAttributeNS(EMPTY_NAMESPACE,'o-encoding')
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected tuv\n")

class tmxSeg:
    def __init__(self,doc = None):
        self.data = ''
        self.node = doc
        if doc.nodeName == u'seg':
            doc.normalize()
            for str in doc.childNodes:
                if str.nodeType == Node.TEXT_NODE or str.nodeType == Node.CDATA_SECTION_NODE:
                    self.data += str.data
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected seg\n")

class tmxBody:
    def __init__(self,doc = None):
        self.tus = []
        if doc.nodeName == u'body':
            for child in doc.childNodes:
                if child.nodeType != Node.ELEMENT_NODE:
                    continue
                if child.nodeName == u'tu':
                    self.tus.append(tmxTu(child))
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected body\n")

class tmxDoc:
    def __init__(self,doc = None):
        self.version = ''
        self.header = None
        self.body = None
        if doc.nodeName == u'tmx':
            self.version = doc.getAttributeNS(EMPTY_NAMESPACE,u'version')
            for node in doc.childNodes:
                if node.nodeType != Node.ELEMENT_NODE:
                    continue
                if node.nodeName == u'header':
                    self.header = tmxHeader(node)
                elif node.nodeName == u'body':
                    self.body = tmxBody(node)
                else:
                    raise TMXSyntaxError("Invalid TMX Syntax, expected body or header\n")
        else:
            raise TMXSyntaxError("Invalid TMX Syntax, expected TMX\n")


def getTMXFromFile(file):
    dom = loadXml(file)
    doc = dom.documentElement
    tmxdoc = tmxDoc(doc)
    return tmxdoc
