#!/usr/bin/env python

import sys
import codecs
from transolution.path_mod import path
from transolution.xmlutils import createNodeAttr,NodeCreator,createDocument,EMPTY_NAMESPACE,XML_NAMESPACE
from transolution.xliffeditor.document.segments import TuSegment,External
from transolution.xliffeditor.xliffparser_ng import XliffDocument,XliffException

src_lang = u"en"
trg_lang = u"sv-se"

def addTMXHeader(doc,parent,entry):
    header = doc.createElementNS(EMPTY_NAMESPACE,"header")
##    if len(entry.xlatComment) > 0:
##        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
##        str_note = u"".join(entry.xlatComment).strip()
##        note_txt = doc.createTextNode(str_note)
##        note.appendChild(note_txt)
##        header.appendChild(note)
##        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
##        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-pi")
##        str_prop = u"".join(entry.procComment).strip()
##        prop_txt = doc.createTextNode(str_prop)
##        prop.appendChild(prop_txt)
##        header.appendChild(prop)
    parent.appendChild(header)
    
    


def  addTMXTu(doc,parent,entry):
    tu = doc.createElementNS(EMPTY_NAMESPACE,"tu")
##    if len(entry.xlatComment) > 0:
##        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
##        str_note = u"".join(entry.xlatComment).strip()
##        note_txt = doc.createTextNode(str_note)
##        note.appendChild(note_txt)
##        tu.appendChild(note)
##    if len(entry.autoComment) > 0:
##        note = doc.createElementNS(EMPTY_NAMESPACE,"note")
##        str_note = u"".join(entry.autoComment).strip()
##        note_txt = doc.createTextNode(str_note)
##        note.appendChild(note_txt)
##        tu.appendChild(note)
##    if len(entry.refComment) > 0:
##        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
##        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-ref")
##        str_prop = u"".join(entry.refComment).strip()
##        prop_txt = doc.createTextNode(str_prop)
##        prop.appendChild(prop_txt)
##        tu.appendChild(prop)
##    if len(entry.procComment) > 0:
##        prop = doc.createElementNS(EMPTY_NAMESPACE,"prop")
##        prop.setAttributeNS(EMPTY_NAMESPACE,"type","x-gettext-pi")
##        str_prop = u"".join(entry.procComment).strip()
##        prop_txt = doc.createTextNode(str_prop)
##        prop.appendChild(prop_txt)
##        tu.appendChild(prop)
    tuv = doc.createElementNS(EMPTY_NAMESPACE,"tuv")
    tuv.setAttributeNS(XML_NAMESPACE,'xml:lang',src_lang)
    seg = doc.createElementNS(EMPTY_NAMESPACE,"seg")
    for node in entry.tu.getSourceNode().childNodes:
        seg.appendChild(node.cloneNode(True))

    tuv.appendChild(seg)
    tu.appendChild(tuv)
    tuv = doc.createElementNS(EMPTY_NAMESPACE,"tuv")
    tuv.setAttributeNS(XML_NAMESPACE,'xml:lang',trg_lang)
    seg = doc.createElementNS(EMPTY_NAMESPACE,"seg")
    for node in entry.tu.getTargetNode().childNodes:
        seg.appendChild(node.cloneNode(True))

    tuv.appendChild(seg)
    tu.appendChild(tuv)
    
    parent.appendChild(tu)


def parseArgs():
    import sys
    from optparse import OptionParser

    usage = "usage: %prog [options] path out_tmx"

    op = OptionParser(usage=usage)
    op.add_option('-r', '--recursive',
                  action='store_true', dest='recursive',
                  help='Process files recursive.')
    op.add_option('-f', '--fmask',
                  action='store', type='string', dest='fmask',
                  help='File mask to use when running recursive.')


    options, args = op.parse_args(sys.argv[1:])
    try:
        pname = path(args[0])
        outfile = path(args[1])
    except IndexError,e:
        op.error("Invalid number if arguments, use -h for help")

        
    if options.recursive:
        if not options.fmask:
            op.error("must give filemask option to when recurse")
        else:
            fnames = pname.walkfiles(options.fmask)
    else:
        if options.fmask:
            if not pname.isdir():
                op.error("must give a directory path when option fmask is given")
            else:
                fnames = pname.listdir(options.fmask)
        else:
            fnames = [pname,]


    options.fnames = fnames
    options.outfile = outfile
    return options


def main():
    import sys
    import codecs
    import transolution
    global src_lang,trg_lang
    
    options = parseArgs()

    dom = createDocument("tmx",ns=EMPTY_NAMESPACE)
    doc = dom.documentElement
    doc.setAttributeNS(EMPTY_NAMESPACE,"version","1.4")
    body = dom.createElementNS(EMPTY_NAMESPACE,"body")
    doc.appendChild(body)
    print "Parsing Xliff"
    for fname in options.fnames:
        
        print 'Processing:%s -->'%(fname,)
        xliff = XliffDocument(fname)
        xliff.parse()
        for filenode in xliff.files:
            src_lang = filenode.getSourceLang()
            trg_lang = filenode.getTargetLang()
            assert src_lang and trg_lang,'Source or Target language not set.'

##            filenode.parseSkeleton()
            for seg in filenode.generateSegments(use_skeleton=False):
                if seg.__class__ is TuSegment and seg.tu.hasTarget():
                    addTMXTu(dom,body,seg)

    print "Writing TMX"
    transolution.xmlutils.toxml(doc,file(options.outfile,'w'))
    print "Done"
if __name__ == "__main__":
    main()
