from transolution.xliffeditor.xliffparser_ng import XliffDocument,XliffException
from seg_formatters import FileInfoSegFormatter,SkeletonSegFormatter,TuSegFormatter,TuInfoSegFormatter
from document import ModelDoc#,SourceTargetSegFormatter,Search
from document_formatter import DocumentFormatter

def createXliffDocumentModel(xml,use_skeleton=True):
    'factory for ModelDoc that sets up formatters and styles etc.'
    xliff = XliffDocument(xml)

    doc = ModelDoc(xliff)
    doc.parseXliff(use_skeleton=use_skeleton)
   
    return doc

def getDocumentFormatter(tagformatter=None):
    # File info
    sFileInfoText = ("fg_black",'bold')
    sFileInfoTag = ("fg_black",)

    # Setup font styles
    sExternalText = ("fg_gray",)
    sExternalTag = ("fg_red",)


    sUnTranslatedTuSegmentText = ("fg_black",)
    sUntranslatedTag = ("fg_red",)

    sTranslatedTuSegmentText = ("fg_black",)
    sTranslatedTag = ("fg_red",)

    sOpenText = ("fg_black",)
    sOpenTag = ("fg_red",)
    sSub = ("fg_darkcyan",)

    if tagformatter is None:
        tagformatter = FullFormatter()
        tagformatter.setStyles(sTranslatedTuSegmentText,sTranslatedTag,sSub)

    tu_formatter = TuSegFormatter(tagformatter)
    tu_formatter.setStyles(sTranslatedTuSegmentText,sTranslatedTag,sSub)


    eformatter = SkeletonSegFormatter()
    eformatter.setStyles(sExternalText,sExternalTag,sSub)

    #FileFormatter
    fformatter = FileInfoSegFormatter()
    fformatter.setStyles(sFileInfoText,sFileInfoTag,sSub)

    documentformatter = DocumentFormatter()
##    segformatter = SourceTargetSegFormatter()


    # setup TU segment background colors
    bg_colors = {'External':('bg_white',),
                'FileInfo':('bg_lightblue',),
                'TuInfo':('bg_lightgray',),
                'open':('bg_cyan',),
                'approved':('bg_green',),
                'dont_trans':('bg_hotpink',),
                'final':('bg_lightyellow',),
                'translated':('bg_lightgreen',),
                'untranslated':('bg_mistyrose',),
                'needs-translation':('bg_lightblue',),
                'needs-review-translation':('bg_lightblue',),
                'new':('bg_lightyellow',),
                'nostate':('bg_khaki',),
                'unknown':('bg_white',),
                }

    documentformatter.setBgColors(bg_colors)

    # TU infor formatter
    f_tu_info = TuInfoSegFormatter()
    f_tu_info.sText = ('fg_darkgray','bold')
    f_tu_info.sId = ('fg_darkgray','bold')
    f_tu_info.bg_color = bg_colors['TuInfo']

    documentformatter.setSubFormatters(tu_formatter,eformatter,fformatter,f_tu_info)

    return documentformatter
