__doc__ = '''Provides functionallity to search & replace text in a document model'''

from transolution.observer import Observable,Observer
from transolution.xliffeditor.xliffsegment import Model
from transolution.xliffeditor.formatters import Formatter,MinimalFormatter,FullFormatter
from segments import External,TuSegment
from tu import TuException

class SearchException(Exception):pass

class FoundObject(Observable):
    'Object stores find/replace information, implements Observable to notify observers on replace'
    def __init__(self,seg,model,segtype,pos,search_str):
        Observable.__init__(self)
        self.seg = seg
        self.model = model
        self.segtype=segtype
        self.pos = pos
        self.start,self.end = self.pos
        self.search_str = search_str
        self.replace_str = None

    def replace(self,replace_str):
        self.replace_str = replace_str
        self.model.deleteTextWithUndo(self.start,self.end)
        self.model.insertTextWithUndo(self.start,replace_str)
        self.notifyObservers()
        # set flag that data in segment has changed
        self.seg.dirty = True


class Search(object):
    'search object, searches the segments in a document for a phrase'
    def __init__(self,doc,searchterm,search_source=False,search_target=False,case=False,model=None):
        if not searchterm:
            raise SearchException('search term cant be empty')
        if not (search_source or search_target):
            raise SearchException('Must search source and/or target segments')
        self.doc = doc
        self.searchterm = searchterm
        self.case = case
        self.search_source, self.search_target = search_source, search_target
        self.foundobj = None
        # setup xliff source/target node wrapper (model)
        if model is None:
            raise SearchException("Error: No xliff model supplied")
##            self.model = Model(None,FullFormatter(),protect_tags=True)
        self.model = model
        
        self.seg_searcher = _SegModelSearch()

    def doSearch(self,offset=None,reverse=False):
        '''returns a FoundObject generator.
           offset is a segment object to start the search from
        '''
        model = self.model
        
        segs = self.doc.segs[:]
        if reverse: 
            segs.reverse() # search backwards(works inplace so make sure you work on a copy of the list)
        if offset:
            segs = segs[segs.index(offset):]
        internal_segs = [seg for seg in segs if seg.__class__ == TuSegment]
        for seg in internal_segs:
            if self.search_source:
                model.setRootNode(seg.tu.getSourceNode())

                for res in self.seg_searcher.search(model,self.searchterm,reverse,self.case):
                    self.foundobj = FoundObject(seg,model,'source',res,self.searchterm)
                    self.seg_searcher.subscribeToObservable(self.foundobj,"cbFound")

                    yield self.foundobj 
            if self.search_target:
                try: node = seg.tu.getTargetNode()
                except TuException,e: pass
                else:
                    model.setRootNode(node)

                    for res in self.seg_searcher.search(model,self.searchterm,reverse,self.case):
                        self.foundobj = FoundObject(seg,model,'target',res,self.searchterm)
                        self.seg_searcher.subscribeToObservable(self.foundobj,"cbFound")

                        yield self.foundobj 
                
class _SegModelSearch(Observer):
    'Searches a xliff segment model'
    def __init__(self):
        Observer.__init__(self)
        self.dirty = False
        
    def cbFound(self, subject,event,msg):
        'called when a foundobject has done a replace'
        self.replace_offset = len(subject.replace_str) - len(subject.search_str)
        if not self.reverse: self.i += self.replace_offset
        
    def _initSearch(self,model,reverse,case):
        self.case = case
        self.reverse = reverse
        self.data = unicode(model)
        if not self.case:
            self.data,self.searchterm = self.data.lower(),self.searchterm.lower()
        self.dirty = False
        
    def search(self,model,searchterm,reverse,case):
        'search a node, yields the text span index (start,end) model is a xliff segment'
        #TODO: does not work if node has been edited by replace operation in between calls as the data value is not updated
        print "start seg Search"
        self.searchterm = searchterm
        self._initSearch(model,reverse,case)
        if self.reverse: self.i = len(self.data)
        else:   self.i = 0
        
        while True:
            self._initSearch(model,reverse,case)
            try:
                if reverse: self.i = self.data.rindex(self.searchterm,0,self.i)
                else: self.i = self.data.index(self.searchterm,self.i)
            except ValueError,e: break
            else:
                start,end = self.i,self.i + len(self.searchterm)
                if not reverse: self.i = end
                yield start,end
