__doc__ = 'Formatters for document model segments (external/internal)'
from segments import External,TuSegment,FileInfo
from transolution.xmlutils import getText

bg_colors = None
def createStyledText(styles,text):
    return [ (styles,text), ]


class SegmentFormatter(object):
    def setStyles(self,sText,sTag,sSub):
        self.sText,self.sTag,self.sSub = sText,sTag,sSub

class TuInfoSegFormatter(SegmentFormatter):
    'Shows the info for a FileNodeWrapper'
    # TODO: Should file info use the xml file node instead of file_node_wrapper object?
    def __init__(self):
        global bg_colors
        SegmentFormatter.__init__(self)

    def format(self,seg):
        seg.styledtext = createStyledText(self.sText, '\n' + str(seg.tu_nr) + '\n')
        seg.data = ''
        for o in seg.styledtext:
            seg.data += o[1]
        seg.bg_color = self.bg_color                

class FileInfoSegFormatter(SegmentFormatter):
    'Shows the info for a FileNodeWrapper'
    # TODO: Should file info use the xml file node instead of file_node_wrapper object?
    def format(self,seg):
        global bg_colors
        # TODO: move be_colors to this module instead
        seg.bg_color = bg_colors['FileInfo']
        styledtext = self._format(seg.getNode())

        ret = []
        a = ret.append
        for o in styledtext:
            a(o[1])

        seg.data,seg.styledtext = unicode(''.join(ret)),styledtext
        seg.dirty = True # flag tells that Document UI needs to update seg

    def _format(self,file_node_wrapper):
        styledtext = []
        f = file_node_wrapper
        sText = self.sText
        sTag = self.sTag
        a = styledtext.append
        a((sText,'Original: '))
        a((sTag,unicode(f.getOriginal())))
        a((sText,'\nData Type: '))
        a((sTag,unicode(f.getDatatype()) ))
        a((sText,'\nSlang: '))
        a((sTag,unicode(f.getSourceLang())))
        a((sText,'\nTlang: '))
        a((sTag,unicode(f.getTargetLang())))
        a((sText,'\n\n'))
        return styledtext

import re
re_ws = re.compile(r'^\s*$',re.DOTALL)
class SkeletonSegFormatter(SegmentFormatter):
    'Formatter for External content'
    def __init__(self,mode='Full',separator='<>'):
        self.modes = {'Full':self.fullFormat,'Minimal':self.minimalFormat}
        self.separator = separator
        self.mode = mode
        self.setFormatterMode(mode)
        SegmentFormatter.__init__(self)

    def setFormatterMode(self,mode):
        self.mode = mode
        self.format = self.modes[self.mode]

    def getFormatterMode(self):
        return self.mode
        
    def _buildSeparator(self,node):
        'see if nodedata has newlines, if so add a newline to the separator'
        text = getText(node.childNodes)
        if re_ws.search(text):
            # only whitespaces in skeleton content
            return ' ' + '\n'* min(text.count('\n'),2)
        else:
            return self.separator + '\n'* min(text.count('\n'),2)
        
    def minimalFormat(self,seg):
        global bg_colors
        node = seg.getNode()
        seg.data, seg.styledtext = self._buildSeparator(node),createStyledText(self.sText,self._buildSeparator(node))
        seg.bg_color = bg_colors['External']

    def fullFormat(self,seg):
        global bg_colors
        seg.data = getText(seg.getNode().childNodes)
        seg.styledtext = createStyledText(self.sText,seg.data)
        seg.bg_color = bg_colors['External']


class TuSegFormatter(SegmentFormatter):
    def __init__(self,subformatter=None):
        self.subformatter = subformatter
        self.show_source = False
        self.show_target = True
        self.seg_tag_start = '{>'
        self.seg_tag_sep = '<X>'
        self.seg_tag_end = '<}'
        self.tag_style = ('fg_purple','bold',)
        SegmentFormatter.__init__(self)

    def format(self,seg):
        global bg_colors
        if self.show_target and seg.tu.hasTarget():
            if self.show_source:
                styledtext = []
                s_data,s_styledtext = self._subformat(seg.tu.getSourceNode())
                seg.dirty = True
            
                t_data,t_styledtext = self._subformat(seg.tu.getTargetNode())

                seg.data = '%s%s%s%s%s'%(self.seg_tag_start, s_data, self.seg_tag_sep, t_data, self.seg_tag_end)

                styledtext += createStyledText(self.tag_style,self.seg_tag_start)
                styledtext += s_styledtext
                styledtext += createStyledText(self.tag_style,self.seg_tag_sep)
                styledtext += t_styledtext
                styledtext += createStyledText(self.tag_style,self.seg_tag_end)
                seg.styledtext = styledtext
            else:
                seg.data,seg.styledtext = self._subformat(seg.tu.getTargetNode())

        else:
            seg.data,seg.styledtext = self._subformat(seg.tu.getSourceNode())

        if seg.tu.open:
            status = 'open'
        else:
            #Get the common status of tu
            status = seg.tu.getStatus()
        seg.bg_color = bg_colors[status]

    def _subformat(self,node):
        if node is None: node = self.getNode()
        styledtext = []
        ret = []
        subformatter = self.subformatter
        for child in node.childNodes:
            styledtext.extend(subformatter.getFormatted(child))

        a = ret.append
        for s,t in styledtext:
            a(t)

        data,styledtext = unicode(''.join(ret)),styledtext
        return data,styledtext
