from transolution.observer import Observable,Observer
from tu import TranslationUnit,TuException,TU_CHANGE_EVENT,TU_UI_UPDATE_EVENT

class SELECT_SEG_EVENT: pass

bg_colors = {}


class Segment(Observable):
    'Base object for External and TuSegment segments'
    formatter = None
    def __init__(self,node):
        Observable.__init__(self)
        self._node = node
        self.dirty = True # flag that tells that ui needs to update
        self.oldlen = 0
        self.styledtext = None
        self.data = ''
        self.bg_color = None

        
    def getNode(self):
        'set the node to format'
        raise NotImplementedError('must be implemented in subclasses')

    def format(self,node=None):
        self.oldlen = len(self.data)
        if not self.dirty: return
        self.__class__.formatter.format(self)
        self.dirty = True # flag tells that Document UI needs to update seg

    def __str__(self):
        return unicode(self.data)
    def __len__(self):
        return len(unicode(self))

class External(Segment):
    'External segment. content not localizable'
    def __init__(self,node):
        Segment.__init__(self,node)

    def getNode(self):
        return self._node

class FileInfo(Segment):
    'File Info segment. Used to marks the beginning of file'
    def __init__(self,file_node_wrapper):
        Segment.__init__(self,file_node_wrapper)

    def getNode(self):
        return self._node


class TuInfo(Segment):
    'Tu Info segment. Used to marks the beginning of file'
    def __init__(self,id,tu_nr=None):
        Segment.__init__(self,None)
        self.tu_nr = tu_nr
        self.id = id
        
    def getNode(self):
        return self._node


class TuSegment(Segment,Observer):
    'TuSegment segment. Has Translation unit'
    show_both = True
    def __init__(self,node):
        Segment.__init__(self,node)
        Observer.__init__(self)
        self.tu = TranslationUnit(self._node)
        # observe TU for changes
        self.subscribeToObservable(self.tu, "cbTuChanged",(TU_CHANGE_EVENT,TU_UI_UPDATE_EVENT))


    def cbTuChanged(self, observable,event,subject):
        'observer interface called by TUs notify'
        print "TU changed"
        self.dirty = True
        # Update segment to
        self.notifyObservers(event,subject)
        
    def getTU(self):
        return self.tu
    
    def getNode(self):
        'get the node to represent segment in docview'
        if not self.tu.hasTarget():
            return self.tu.getSourceNode()
        else:
            tnode = self.tu.getTargetNode()

            # Handle empty nodes so we show source instead
            if tnode.hasChildNodes():
                return tnode
            else:
                return self.tu.getSourceNode()

