__doc__ = '''This module has a tree for efficient lookup of segments and it's position in a buffer'''

class Node:
    def __init__(self,parent, key=None):
        self.parent = parent
        self.childs = []
        self.key = key
        self.append = self.childs.append

class LeafNode:
    childs = None
    def __init__(self,parent, key, seg):
        self.parent = parent
        self.key = key
        self.seg = seg
        seg.treenode = self

DEPTH=4
FANOUT = 5
class IndexTree:
    def __init__(self):
        # initializes the root member
        self.root = Node(None)
    
    def insert(self,node,seg,dept=0):
        if node.parent and (len(node.childs) > FANOUT):
            # create new branch
            n = Node(node.parent)
            node.parent.append(n)
            self.insert(n,seg,dept-1)
        elif dept < DEPTH:
            if node.childs:
                n = node.childs[-1]
            else:
                n = Node(node)
                node.append(n)
            # recurse down
            self.insert(n,seg,dept+1)
        else:
            node.append(LeafNode(node,len(seg),seg))


    def buildTree(self, segments):
        print "Building Tree"
        if self.root:
            self.unlink()
            del self.root
        # inserts a new data
        root = Node(None)
        for seg in segments:
            self.insert(root,seg)
        self.sumBranch(root)

        self.root = root
        print "Done"

    def sumBranch(self,node):
        'goes down the tree and summerize the childs keys and sets the sum as nodes key'
        sum = 0
        if node.childs:
            for child in node.childs:
                sum += self.sumBranch(child)
        else:
            return node.key
        node.key = sum
        return sum

    def updateSeg(self,seg):
        'Update the tree if seg length has changed'
        node = seg.treenode
        new = len(seg)
        diff = new - node.key
        if diff:
            self._update(node,diff)

    def _update(self,node,diff):
        if not node: return
        node.key +=diff
        self._update(node.parent,diff)

    def lookup(self, i):
        'return segment at character index'
        return self._lookup(i,self.root)

    def _lookup(self,i,node,last=0):
        count = last
        if node.childs is None: return node
        for child in node.childs:
            last = count
            count += child.key
            if count > i:
                return self._lookup(i,child,last)
        raise IndexError('text index out of range')
        

    def getBounds(self, seg):
        'return start and end text index of segment'
        # return segment at character index
        node = seg.treenode
        count = self._getBounds(node)
        return count,count+node.key

    def _getBounds(self, node,count=0):
        if not node.parent: return count
        for node in node.parent.childs[:node.parent.childs.index(node)]:
            count += node.key
        return self._getBounds(node.parent,count)


    def printTree(self,node=None,indent=0):
        if not node: node = self.root
        if node.childs:
            print (' '*indent) + str(node.key)
            for i,node in enumerate(node.childs):
                self.printTree(node,indent+1)
        else:
            print (' '*indent) + str(node.key) + str(node.seg)

    def unlink(self):
        'unlink all nodes in the tree to free objects'
        if self.root is not None:
            self._unlink(self.root)
        self.root = None
        
    def _unlink(self,node):
        del node.parent
        if node.__class__ is LeafNode:
            # leafnode
            del node.seg.treenode
            del node.seg
        else:
            for c in node.childs:
                self._unlink(c)
            del node.childs
        del node
                
    def __del__(self):
        try:        
            self.unlink()
        except AttributeError,e:
            pass

def testGetBounds():
    print "Test Getbounds",
    interval = 100
    for i,seg in enumerate(segments):
        s,e = tree.getBounds(seg)
        assert segstr[s:e] == seg.val,'Invalid getBounds'
##        if i%interval == 0:
##            print i
    print "OK"
        
def testLookup():
    print "Test char index lookup",
    oldnode = None
    for i in range(len(segstr)):
        node = tree.lookup(i)
        if oldnode != node:
            s,e = tree.getBounds(node.seg)
        oldnode = node
        assert segstr[i] == node.seg.val[i-s],'invalid lookup'
    print "OK"

from random import randint
if __name__ == "__main__":
    class TestSeg:
        def __init__(self,val):
            self.val = val
        def __len__(self):
            return len(self.val)
        def __str__(self):
            return self.val

    # create the binary tree
    tree = IndexTree()
    segs = ['Test','of', 'the','index', 'lookup','tree','that','is','way','cool']*313
    print "Total test segments:",len(segs)
    segstr = ''.join(segs)
    segments = [TestSeg(seg) for seg in segs]
    
    tree.buildTree(segments)
    testGetBounds()
    testLookup()
    print "Random Updates"
    # do some random updates
    for i in range(100):
        i = randint(0,len(segs))
        segs[i] = 'UPDATED'*randint(1,4)
        segments[i].val = segs[i]
        tree.updateSeg(segments[i])
    segstr = ''.join(segs)
    testGetBounds()
    testLookup()
##    tree.printTree()
    