##tu.state = 'translated'
##tu.approved = 'yes'
##
##tu.hasTarget()
'''
Evil trans use:


TU:
 *approved = yes/no: Master state.

Target:
 states:
  *new
  *translated
  #*signed-off
  *final
  
 state-qualifier
  *exact-match
  *fuzzy: tag diff
  *tm-suggestion: exact match percent kept in attr evil:match

 Evil Xliffs own namespace:
  *evil:match=0-100
  *evil:repetition=yes/no

    <trans-unit id="1" translate="yes" xml:space="default">
        <source>This is a test string for EvilXliff.</source>
        <target state="new" evil:match='0' evil:repetition='no'>Detta ar  en text strang av ElakXliff</target>t>
    </trans-unit>

TU State flow:
 On open:
    pass

 On close:
    evil:match = tm-match
    evil:repetition= yes/no
    target_state = 'translated'
    tu.approved = 'no'
    state-qualifier = 'exact-match'|'tm-suggestion'

 On cancel:
    pass

 On delete:
    #state = 'needs-translation'
    tu.approved = 'no'
    #delete evil:match

 On approve:
    state='final'
    tu.approved = 'yes



'''
# The name space where TM match and repetition are kept
evilNS = 'http://www.evilxliff.org/xliff/tm'

bgdict = {
    'open':None,
    'approved':None,
    'dont_trans':None,
    'untranslated':None,
    'final':None,
    'needs-translation':None,
    'new':None,
    'nostate':None,
    }


from transolution.xmlutils import XmlError,getNodeAttr,setNodeAttr,setCreateNodeAttr,removeNodeAttr,getNodes,getElementsByTagName
from wrappers import ContextGroupWrapper,NoteWrapper,PropGroupWrapper,AltTransWrapper

# FIX THIS
class TuStatusWrapperBase(object):
    # --------------------
    # id attribute
    def _getId(self):
        'Get the ID of the TU'
        try:
            return getNodeAttr(self.tu_node,'id')
        except KeyError,e:
            return None

    def _setId(self,val):
        'Sets the ID attr of the tu'
        setCreateNodeAttr(self.tu_node,'id',val)
        
    def _delId(self):
        'delete the ID attr from the tu'
        removeNodeAttr(self.tu_node,'id')

    id = property(_getId,_setId, _delId, "TU ID attribute.")

    # --------------------
    # Approved attr actions
    def _getApproved(self):
        'Get the approved attr of the tu'
        try:
            return getNodeAttr(self.tu_node,'approved')
        except KeyError,e:
            return 'no' # default val

    def _setApproved(self,val):
        'Sets the approved attr of the tu'
        if val in ('yes',True):
            setCreateNodeAttr(self.tu_node,'approved','yes')
        elif val in ('no',False):
            # only set it if it's already there
            try: setNodeAttr(self.tu_node,'approved','no')
            except KeyError,e:
                pass
        else:
            raise ValueError('Illegal value, must be yes or no')
        

    def _delApproved(self):
        'delete the approved attr from the tu'
        removeNodeAttr(self.tu_node,'approved')

    approved = property(_getApproved, _setApproved, _delApproved, "TU approved atribute.")

    # --------------------
    # translate attr actions
    def _getTranslate(self):
        'Get the translate attr of the tu'
        try:
            return getNodeAttr(self.tu_node,'translate')
        except KeyError,e:
            return 'yes' # default val

    def _setTranslate(self,val):
        'Sets the translate attr of the tu'
        if val in ('yes',True):
            setCreateNodeAttr(self.tu_node,'translate','yes')
        elif val in ('no',False):
            # only set it if it's already there
            try: setNodeAttr(self.tu_node,'translate','no')
            except KeyError,e:
                pass
        else:
            raise ValueError('Illegal value, must be yes or no')
        
    def _delTranslate(self):
        'delete the translate attr from the tu'
        removeNodeAttr(self.tu_node,'translate')

    translate = property(_getTranslate,_setTranslate, _delTranslate, "TU translate atribute.")

    # --------------------
    # target status attr actions
    def _getState(self):
        'Get the state-qualifier attr of the target node'
        try:
            return getNodeAttr(self.target_node,'state')
        except (KeyError,AttributeError),e:
            return None

    def _setState(self,val):
        'Sets the state attr of the target node'
##        if val not in states:
##            raise ValueError('state value not valid. Must be one of: %s'%(states,))
        setCreateNodeAttr(self.target_node,'state',val)
        
    def _delState(self):
        'delete the state attr from the target node'
        removeNodeAttr(self.target_node,'state')

    state = property(_getState, _setState, _delState, "Target state atribute.")


    # --------------------
    # target status attr actions
    def _getStateQualifier(self):
        'Get the state-qualifier attr of the target node'
        try:
            return getNodeAttr(self.target_node,'state-qualifier')
        except KeyError,e:
            return None

    def _setStateQualifier(self,val):
        'Sets the state-qualifier attr of the target node'
##        if val not in state_qualifiers:
##            raise ValueError('"%s" state value not valid. Must be one of: %s'%(val,state_qualifiers,))
        setCreateNodeAttr(self.target_node,'state-qualifier',val)
        
    def _delStateQualifier(self):
        'delete the state-qualifier attr from the target node'
        removeNodeAttr(self.target_node,'state-qualifier')

    state_qualifier = property(_getStateQualifier, _setStateQualifier, _delStateQualifier, "Target state-qualifier attribute.")
    #---------------

    # --------------------
    # target TM match  attr
    def _getTmMatch(self):
        try:
            return int(getNodeAttr(self.target_node,'match',evilNS))
        except KeyError,e:
            return None

    def _setTmMatch(self,val):
        val = int(val)
        assert val > 0 and val <= 100, 'Not a valid match percent'
        createNodeAttr(self.target_node,'match',val,evilNS)

    def _delTmMatch(self):
        removeNodeAttr(self.target_node,'match',evilNS)
    tm_match = property(_getTmMatch, _setTmMatch, _delTmMatch, "Target TM match attribute.")

    #---------------

    # --------------------
    # target TM match  attr
    def _getRepetition(self):
        try:
            return getNodeAttr(self.target_node,'repetition',evilNS)
        except KeyError,e:
            return None

    def _setRepetition(self,val):
        if val in ('yes',True):
            setCreateNodeAttr(self.target_node,'repetition','yes',evilNS)
        elif val in ('no',False):
            # only set it if it's already there
            try: setNodeAttr(self.target_node,'repetition','no',evilNS)
            except KeyError,e:
                pass
        else:
            raise ValueError('Illegal value, must be yes or no')

    def _delRepetition(self):
        removeNodeAttr(self.target_node,'repetition',evilNS)
    repetition = property(_getRepetition, _setRepetition, _delRepetition, "Target repetition match attribute flag.")

    #---------------

class TuStatusWrapper(TuStatusWrapperBase):

    def getContextGroups(self):
        for node in self.tu_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            if node.tagName == 'context-group':
                yield ContextGroupWrapper(node)

    def getPropGroups(self):
        for node in self.tu_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            if node.tagName == 'prop-group':
                yield PropGroupWrapper(node)

    def getAltTrans(self):
        'return generator for alt-trans wrappers'
        tu = self.tu_node
        for node in tu.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            elif node.tagName == 'alt-trans':
                yield AltTransWrapper(node)


##    # Status changes
##    def OnOpen(self):
##        self.open = True
##        self.tm_match = 0 # dummy for test
##        
##    def OnClose(self):
##        self.open = False
##        self.target_state('translated')
##        if self.tm_match == 100:
##            self.state_qualifier = 'exact-match'
##        elif self.tm_match > 0:
##            self.state_qualifier = 'tm-suggestion'
##        else:
##            del self.state_qualifier # remove attr
##
##        self.approved = 'no'
##
##    def OnDelete(self):
##        self.approved = 'no'
##        del self.tm_match
##
