from transolution.xmlutils import XmlError,getNodeAttr,setNodeAttr,setCreateNodeAttr,removeNodeAttr,getNodes,getElementsByTagName

class AltTransWrapper(object):
    'Wraps a AltTrans segment'
    def __init__(self,root_node):
        if root_node.tagName != 'alt-trans':
            raise ValueError('Node not a alt-trans')
        self.root_node = root_node
        self._parse()

    def getContextGroups(self):
        for node in self.root_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            if node.tagName == 'context-group':
                yield ContextGroupWrapper(node)

    def _parse(self):
        'parse tu node to get source & target '
        self.source_node = None
        self.target_nodes = []

        try:self.source_node = getNodes(self.root_node,'source','?')[0]
        except IndexError,e: pass
        self.target_nodes = getNodes(self.root_node,'target','+')

    
    # --------------------
    # match_quality attribute
    def _getMQ(self):
        'Get the match-quality of the alttrans'
        try:
            return getNodeAttr(self.root_node,'match-quality')
        except KeyError,e:
            return None

    def _setMQ(self,val):
        'Sets the match-quality attr of the tu'
        setCreateNodeAttr(self.root_node,'match-quality',val)
        
    def _delMQ(self):
        'delete the match-quality attr from the tu'
        removeNodeAttr(self.root_node,'match-quality')

    match_quality = property(_getMQ,_setMQ, _delMQ, "match_quality")

    def __str__(self):
        return "Match:%s"%(self.match_quality,)


class ContextGroupWrapper(object):
    'Wraps a AltTrans segment'
    def __init__(self,root_node):
        if root_node.tagName != 'context-group':
            raise ValueError('Node not a context-group')
        self.root_node = root_node
##        self._parse()

    def getContextNodes(self):
        root_node = self.root_node
        for node in root_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            if node.tagName == 'context':
                yield ContextWrapper(node)

    # --------------------
    # attributes
    def _getName(self):
        try:
            return getNodeAttr(self.root_node,'name')
        except KeyError,e:
            # must have a name attribute
            raise

    def _setName(self,val):
        setCreateNodeAttr(self.root_node,'name',val)
        
    def _delName(self):
        removeNodeAttr(self.root_node,'name')

    name = property(_getName,_setName, _delName, "name")
    # --------------------
    def _getPurpose(self):
        try:
            return getNodeAttr(self.root_node,'purpose')
        except KeyError,e:
            return None

    def _setPurpose(self,val):
        setCreateNodeAttr(self.root_node,'purpose',val)
        
    def _delPurpose(self):
        removeNodeAttr(self.root_node,'purpose')
    purpose = property(_getPurpose,_setPurpose, _delPurpose, "purpose")

    def __str__(self):
        return "Context group:%s purpose:%s"%(self.name,self.purpose)


class ContextWrapper(object):
    'Wraps a AltTrans segment'
    def __init__(self,root_node):
        if root_node.tagName != 'context':
            raise ValueError('Node not a context node')
        self.root_node = root_node
    # --------------------
    # attributes
    def _getContextType(self):
        try:
            return getNodeAttr(self.root_node,'context-type')
        except KeyError,e:
            # must have a name attribute
            raise

    def _setContextType(self,val):
        setCreateNodeAttr(self.root_node,'context-type',val)
        
    def _delContextType(self):
        removeNodeAttr(self.root_node,'context-type')

    context_type = property(_getContextType,_setContextType, _delContextType, "context-type")

    def getValue(self):
        rc = []
        for node in self.root_node.childNodes:
            if node.nodeType in (node.TEXT_NODE,node.CDATA_SECTION_NODE):
                rc.append(node.data)

        return ''.join(rc)

    # --------------------
    def __str__(self):
        return "Context type:%s\n%s"%(self.context_type,self.getValue())

#-----------------------------------------------------------------
class PropGroupWrapper(object):
    'Wraps a prop-group segment'
    def __init__(self,root_node):
        if root_node.tagName != 'prop-group':
            raise ValueError('Node not a prop-group')
        self.root_node = root_node
##        self._parse()

    def getPropNodes(self):
        root_node = self.root_node
        for node in root_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue
            if node.tagName == 'prop':
                yield PropWrapper(node)

    # --------------------
    # attributes
    def _getName(self):
        try:
            return getNodeAttr(self.root_node,'name')
        except KeyError,e:
            return None

    def _setName(self,val):
        setCreateNodeAttr(self.root_node,'name',val)
        
    def _delName(self):
        removeNodeAttr(self.root_node,'name')

    name = property(_getName,_setName, _delName, "name")
    # --------------------

    def __str__(self):
        return "Prop group:%s "%(self.name)


class PropWrapper(object):
    'Wraps a Prop segment'
    def __init__(self,root_node):
        if root_node.tagName != 'prop':
            raise ValueError('Node not a prop node')
        self.root_node = root_node
    # --------------------
    # attributes
    def _getPropType(self):
        try:
            return getNodeAttr(self.root_node,'prop-type')
        except KeyError,e:
            return None

    def _setPropType(self,val):
        setCreateNodeAttr(self.root_node,'prop-type',val)
        
    def _delPropType(self):
        removeNodeAttr(self.root_node,'prop-type')

    prop_type = property(_getPropType,_setPropType, _delPropType, "prop-type")

    def getValue(self):
        rc = []
        for node in self.root_node.childNodes:
            if node.nodeType in (node.TEXT_NODE,node.CDATA_SECTION_NODE):
                rc.append(node.data)

        return ''.join(rc)

    # --------------------
    def __str__(self):
        return "Prop type:%s\n%s"%(self.prop_type,self.getValue())



#-----------------------------------------------------------------




class NoteWrapper(object):
    'Wraps a Note segment'
    def __init__(self,root_node):
        if root_node.tagName != 'note':
            raise ValueError('Node not a note node')
        self.root_node = root_node
    # --------------------
    # attributes
    def _getFrom(self):
        try:
            return getNodeAttr(self.root_node,'from')
        except KeyError,e:
            return None

    def _setFrom(self,val):
        setCreateNodeAttr(self.root_node,'from',val)
        
    def _delFrom(self):
        removeNodeAttr(self.root_node,'from')

    _from = property(_getFrom,_setFrom, _delFrom, "from")

    #--------
    def _getPriority(self):
        try:
            return getNodeAttr(self.root_node,'priority')
        except KeyError,e:
            return None

    def _setPriority(self,val):
        if 0 < int(val) < 11:
            setCreateNodeAttr(self.root_node,'priority',val)
        else:
            raise ValueError("priority value must be between 1 and 10")
        
    def _delPriority(self):
        removeNodeAttr(self.root_node,'priority')

    priority = property(_getPriority,_setPriority, _delPriority, "priority")

    #--------
    def _getAnnotates(self):
        try:
            return getNodeAttr(self.root_node,'annotates')
        except KeyError,e:
            return None

    def _setAnnotates(self,val):
        if val not in ('source', 'target', 'general'):
            raise ValueError("annotates value must be 'source', 'target' or 'general'")
        setCreateNodeAttr(self.root_node,'annotates',val)
        
    def _delAnnotates(self):
        removeNodeAttr(self.root_node,'annotates')

    annotates = property(_getAnnotates,_setAnnotates, _delAnnotates, "annotates")

    def getValue(self):
        rc = []
        for node in self.root_node.childNodes:
            if node.nodeType in (node.TEXT_NODE,node.CDATA_SECTION_NODE):
                rc.append(node.data)

        return ''.join(rc)

    def setValue(self,val):
        for node in self.root_node.childNodes:
            self.root_node.removeChild(node)
        self.root_node.appendChild(node.ownerDocument.createTextNode(val))

    def delete(self):
        'remove the note node'
        parent = self.root_node.parentNode
        parent.removeChild(self.root_node)


    # --------------------
    def __str__(self):
        return "from:%s\npriority:%s\nannotates:%s\nValue:%s"%(self._from,self.priority,self.annotates,self.getValue())

##NoteWrapper.from = NoteWrapper._from
##NoteWrapper.from = property(NoteWrapper._getFrom,NoteWrapper._setFrom, NoteWrapper._delFrom, "from")
