class TagMinimalBase(object):
    style = 'sTag'
    text = None
    empty = False
    def node2text(self,node):
        try:id = node.attributes[(None,'id')].value
        except KeyError,e: id = ''
        return self.text%(id,)

class Tag_minimal_x(TagMinimalBase):
    text = u'\xAB%s\xBB'
    empty = True
class Tag_minimal_bx(TagMinimalBase):
    text = u'|%s\xBB'
    empty = True
class Tag_minimal_ex(TagMinimalBase):
    text = u'\xAB%s|'
    empty = True
class Tag_minimal_bpt(TagMinimalBase):
    text = '<%s>'
class Tag_minimal_ept(TagMinimalBase):
    text = '</%s>'
class Tag_minimal_it(TagMinimalBase):
    text = '|%s|'
class Tag_minimal_ph(TagMinimalBase):
    text = u'\xAB%s\xBB'
class Tag_minimal_sub(TagMinimalBase):
    style = 'sSub'
    text = u'SUB:%s'

class Tag_x(Tag_minimal_x): pass
class Tag_bx(Tag_minimal_bx): pass
class Tag_ex(Tag_minimal_ex): pass

class Formatter(object):
    'Formatter interface'
    tags = {'x':Tag_x(),'bx':Tag_bx(),'ex':Tag_ex()}

    def __init__(self):
        pass

    def setStyles(self,sText,sTag,sSub):
        self.sText,self.sTag,self.sSub = sText,sTag,sSub

    def node2text(self,node):
        raise NotImplementedError('must be implemented in subclasses')

    def getFormatted(self,node):
        '''
        formats a @nodes text
        returns (((style1,..),text1),((style2,..),text2),...)
        '''
        raise NotImplementedError('must be implemented in subclasses')


class FullFormatter(Formatter):
    'Shows full tags'
    def node2text(self,node):
        if node.nodeType == node.TEXT_NODE:
            r = node.data
        elif node.nodeType == node.ELEMENT_NODE:
            parent = node
            # if is an empty XLIFF inline tag dont recurse into it.
            if self.tags.has_key(parent.tagName) and self.tags[parent.tagName].empty:
                r = self.tags[parent.tagName].node2text(parent)
            else:
                substr = []
                for node in parent.childNodes:
                    substr.append(self.node2text(node))
                r = ''.join(substr)
        else:
            raise ValueError('node type not supported')
        return r

    def getFormatted(self,node):
        if node.nodeType == node.TEXT_NODE:
            r = node.data
            s = self.sText
            
        elif node.nodeType == node.ELEMENT_NODE:
            parent = node
            # if is an empty XLIFF inline tag dont recurse into it.
            if self.tags.has_key(parent.tagName) and self.tags[parent.tagName].empty:
                tag = self.tags[parent.tagName]
                r = tag.node2text(parent)
                s = getattr(self,tag.style)
            else:
                substr = []
                for node in parent.childNodes:
                    substr.append(self.node2text(node))
                r = ''.join(substr)
                # FIX ME: sub segments
                if parent.tagName in ('mrk','g','sub'):
                    s = self.sSub
                elif parent.tagName == 'transolution:tu':
                    s = self.sSub
                else:
                    s = self.sTag
        return (s,r),

class FullFormatterTagId(FullFormatter):
    'Shows full tags and append ID'
    def node2text(self,node):
        r = FullFormatter.node2text(self,node)
        if node.nodeType == node.ELEMENT_NODE:
            #Append tag id
            try:id = node.attributes[(None,'id')].value
            except KeyError,e: id = ''
            else:
                r += str(id)

        return r

    def getFormatted(self,node):
        styletext = FullFormatter.getFormatted(self,node)
        #Append tag id
        if node.nodeType == node.ELEMENT_NODE:
            try:id = node.attributes[(None,'id')].value
            except KeyError,e: id = ''
            else:
                return styletext[0],(("superscript","fg_darkcyan"),str(id))
        return styletext



  
class MinimalFormatter(Formatter):
    tags = {'x':Tag_minimal_x(),
            'bx':Tag_minimal_bx(),
            'ex':Tag_minimal_ex(),
            'bpt':Tag_minimal_bpt(),
            'ept':Tag_minimal_ept(),
            'it':Tag_minimal_it(),
            'ph':Tag_minimal_ph(),
            'sub':Tag_minimal_sub(),
##            'g':Tag_minimal_g(),
            }

    'Show only id of tag'
    def node2text(self,node):
        parent = node.parentNode
        if node.nodeType == node.TEXT_NODE:
            r = node.data
        elif node.nodeType == node.ELEMENT_NODE:
            if self.tags.has_key(node.tagName):
                r = self.tags[node.tagName].node2text(node)
            elif node.tagName in ('mrk','g'):
                #Make it return the whole content to the transolution tu
                r = ' '.join([self.node2text(child) for child in node.childNodes])
            elif node.tagName == 'transolution:tu':
                #Make it return the whole content to the transolution tu
                r = ' '.join([self.node2text(child) for child in node.childNodes])
            else:
                raise ValueError('tag not supported:%s'%(node,))
        return r

    def getFormatted(self,node):
        r = self.node2text(node)
        if node.nodeType == node.TEXT_NODE:
            s = self.sText
        elif node.nodeType == node.ELEMENT_NODE:
            # FIX ME:
            if self.tags.has_key(node.tagName):
                sname = self.tags[node.tagName].style
                s = getattr(self,sname)
            elif node.tagName in ('mrk','g'):
                s = self.sSub
            elif node.tagName == 'transolution:tu':
                s = self.sSub
            else:
                s = self.sTag
#            s = self.sTag
        else:
            raise ValueError('tag not supported:%s'%(node,))
        return (s,r),
