import gtk
import pango
from general import EditorException
from transolution.observer import Observer,Observable,Event
from gtk_basetextbuf import BaseTextBuffer
from document.segments import External,TuSegment,FileInfo,SELECT_SEG_EVENT
from document.tu import TU_UI_UPDATE_EVENT,TU_CHANGE_EVENT


class UI_UPDATE_EVENT(Event): pass

class DocumentBuffer(BaseTextBuffer,Observer):
    'make events work with seg model'
    def __init__(self,*args,**kwargs):
        BaseTextBuffer.__init__(self,*args,**kwargs)
        Observer.__init__(self)

        self.old_pos = 0 # keeps track of last cursor pos
        self.interactive = True # flag tells if text changes was interctive
        
    def setModel(self,model):
        self.model = model
        # tell us to observer model for gui changes
        self.subscribeToObservable(self.model,'cbUpdateUI',events=(TU_UI_UPDATE_EVENT,UI_UPDATE_EVENT))#,SELECT_SEG_EVENT))

    def cbUpdateUI(self,doc,event,msg):
        '''
        observe/notify interface. 
        Called by doc model to update GUI.
        '''
        print "DocumentBuffer update"
##        from traceback import print_stack
##        print print_stack()
##        max = float(doc.getSegCount())
        org = self.interactive
        self.interactive = False # disable interactive event checks
        i = self.getInsertionPoint()
        # freeze widget updates
        self.freeze_notify()
##        wxBeginBusyCursor()
        segiter = self.get_iter_at_offset(self.getInsertionPoint())

        #save the current segment so we can change insertion point to the same segment after update
        selected_seg = doc.getSegByTextIndex(self.getInsertionPoint())[0]
        iter = self.get_iter_at_offset
        for o in doc.getSegUIUpdates():
            iter_start,iter_end = iter(o.start),iter(o.end)
            self.delete(iter_start,iter_end)

            for x in o.styledtext:
                s,t = x
                # BUG?
                # this should work but it seems like it picks up tag styles from text nearby so we have to to this the hard way
                #self.insert_with_tags_by_name(iter_start, t, *s)

                # The hard way invloves deleting all style tags before applying new ones.
                startpos = iter_start.get_offset()
                self.insert(iter_start, t )
                iter_startpos = iter(startpos)
                self.remove_all_tags(iter_startpos,iter_start)
                # apply styles to the inserted text range
                for tag in s:
                    self.apply_tag_by_name(tag,iter_startpos,iter_start)

            # apply bg for whole segment
            for tag in o.bg_color:
                self.apply_tag_by_name(tag,iter(o.start),iter_start)

##        # set insertion point to selected seg
##        self.setInsertionPoint(selected_seg.start)
        
        self.thaw_notify()


        self.setInsertionPoint(doc.tree.getBounds(selected_seg)[0])

        self.interactive = org # restore orginal interactive state

##        wxEndBusyCursor()

class DocumentEdit(gtk.TextView,Observable):
    def __init__(self,*args,**kwargs):
        gtk.TextView.__init__(self,*args,**kwargs)
        Observable.__init__(self)
        self.connect("key_press_event", self.keyPressed)

        s = self
        self.kmap={65361:s.k_left,65362:s.k_up,65363:s.k_right,65364:s.k_down,65365:s.k_pgup,65366:s.k_pgdown}
        self.current_seg = None
        self.constraint = None
        self.interactive = True

        # connect the signal that is emitted when the cursor pos changes
##        self.buf = self.get_buffer()
##        self.buf.connect_after("mark-set", self.markSet)
        self.connect("button_press_event", self.buttonPressEvent)

    def setModel(self,doc_model):
        self.buf = self.get_buffer()
        self.buf.setModel(doc_model)
        return False

    def buttonPressEvent(self,widget,event):
        if event.type == gtk.gdk._2BUTTON_PRESS:
            seg,start,end = self.buf.model.getSegByTextIndex(self.buf.getInsertionPoint())
            if seg.__class__ is TuSegment:
                self._update(seg)
            return True
##        self.emit_stop_by_name("button_press_event")
        return False
    

    def keyPressed(self,widget, event, data=None ):
        if not self.interactive: return False
        self.buf = self.get_buffer()
##        print event
        try:
            self.kmap[event.keyval]()
        except KeyError,e:
            print e
            return False
        return True
    
    def k_left(self):
        print "left"
        self.prevSeg()
    def k_right(self):
        print "right"
        self.nextSeg()
    def k_up(self):
        self.prevSeg()
    def k_down(self):
        self.nextSeg()

    def k_pgup(self):
        print "pgup"
        raise KeyError()
##        self._update()
    def k_pgdown(self):
        print "pgdown"
        raise KeyError()
##        self._update()

    def setConstraint(self,constraint):
        self.constraint = constraint

    def nextSeg(self,):
        seg,start,end = self.buf.model.getSegByTextIndex(self.buf.getInsertionPoint())
        seg = self.buf.model.nextSegmentByConstraint(seg,self.constraint)
        self._update(seg)

    def prevSeg(self,constraint=None):
        seg,start,end = self.buf.model.getSegByTextIndex(self.buf.getInsertionPoint())
        seg = self.buf.model.previousSegmentByConstraint(seg,self.constraint)
        self._update(seg)

    def _update(self,seg):
        # unset interactive flag
        self.interactive = False

        buf = self.buf
        iter = buf.get_iter_at_offset
        self.current_seg = seg

        start_mark = buf.get_mark('select_start')
        end_mark = buf.get_mark('select_end')
        if start_mark is None:
            start_mark = buf.create_mark('select_start',iter(0))
        if end_mark is None:
            end_mark = buf.create_mark('select_end',iter(0))

        start_iter = buf.get_iter_at_mark(start_mark)
        end_iter = buf.get_iter_at_mark(end_mark)

        buf.remove_tag_by_name('bg_selected',start_iter,end_iter)

        # get the text span of segment
        s,e = self.buf.model.getTextSpanOfSeg(seg)
        iter_start,iter_end = iter(s),iter(e)
        
        buf.move_mark(start_mark,iter_start)
        buf.move_mark(end_mark,iter_end)

        buf.apply_tag_by_name('bg_selected',iter_start,iter_end)
        buf.place_cursor(iter_start)

        self.showInsertPoint()
##        self.buf.emit_stop_by_name("mark-set")
        # notify obervers that we have changed
        self.notifyObservers(event=SELECT_SEG_EVENT, msg=seg)

        
        # set interactive flag
        self.interactive = True

    def showInsertPoint(self):
        self.scroll_mark_onscreen(self.buf.get_insert())

    def Enable(self,enabled=True):
        self.set_sensitive(enabled)


    def IsEnabled(self):
        return docpanel.document_control.get_sensitive()
