import sys
import os

import pygtk
##pygtk.require("2.0")

import gtk
##import gobject

from gtk_documenteditor import DocumentEdit,DocumentBuffer,UI_UPDATE_EVENT
from gtk_tagedit import TagEdit,TagBuffer

from gtk_utils import askOk,askDialog,entryDialog,messageDialog

from transolution.observer import Observer,Observable
from xliffsegment import XliffError,Model #TODO: Model name is to generic

from transolution.xmlutils import getNodeAttr,setNodeAttr
from document import Search,DOCUMENT_DIRTY_EVENT

from formatters import FullFormatterTagId,FullFormatter,MinimalFormatter

from document.segments import External,TuSegment,FileInfo,SELECT_SEG_EVENT

from document.tu import TuException,TU_CHANGE_EVENT,TU_UI_UPDATE_EVENT

from subseg import AtRootSegException,SubSeg

from general import EditorException

from settings import settings

class OpenSubSegException(EditorException): pass
class CloseSubSegException(EditorException): pass
class CloseTUSegmentException(EditorException): pass
class OpenTUSegmentException(EditorException): pass
class CloseCancelTUSegmentException(EditorException):pass


def getScrolledWindow():
    'ScrolledWindow factory'
    sw = gtk.ScrolledWindow()
    sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
    sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    return sw


class DocumentViewPanelGUI(gtk.VBox):
    'DocumentView GUI'
    def __init__(self, parent=None,main_app=None,tag_table=None):
        self.main_app = main_app
        # Create the main widget
        gtk.VBox.__init__(self)

        # create Main pane
        main_pane = gtk.VPaned()
        main_pane.set_border_width(2)
        self.add(main_pane)

        # create Source/target pane
        edit_pane = gtk.VPaned()
        edit_pane.set_border_width(0)
        main_pane.add1(edit_pane)

        self.tag_table = tag_table

        # create source widget
        self.source_control = TagEdit(TagBuffer(tag_table))
        self.source_control.set_wrap_mode(gtk.WRAP_WORD)
       
        sw = getScrolledWindow()

        edit_pane.add1(sw)
        sw.add(self.source_control)

        # create target widget
        self.target_control = TagEdit(TagBuffer(tag_table))
        self.target_control.set_wrap_mode(gtk.WRAP_WORD)

        sw = getScrolledWindow()

        edit_pane.add2(sw)
        sw.add(self.target_control)

        vbox = gtk.VBox()

        #-------------DIFF VIEW -------------------
        sw = getScrolledWindow()
        self.diff_widget = sw
        # create source widget
        self.diff_control = gtk.TextView(gtk.TextBuffer(tag_table))
        self.diff_control.set_wrap_mode(gtk.WRAP_WORD)
        sw.add(self.diff_control)
        vbox.pack_start(sw,expand=False)

        #--------------------------------

        # create Tu info/docview pane
        doc_pane= gtk.VPaned()
        doc_pane.set_border_width(2)
        main_pane.add2(doc_pane)
##        main_pane.add2(vbox)

        # add tu info panel
        from gtk_tustatus import TuStatusPanel
        self.tu_info_panel = TuStatusPanel()
        vbox.pack_start(self.tu_info_panel,expand=True)
        doc_pane.add1(vbox)


        # create document control widget
        self.document_control = DocumentEdit(DocumentBuffer(table=tag_table))
        self.document_control.set_wrap_mode(gtk.WRAP_WORD)
        self.document_control.set_editable(False)

        # Observe document text widget for segment changes
        self.subscribeToObservable(self.document_control,'cbSelectedSegmentChanged',events=(SELECT_SEG_EVENT,))

        
        self.documentbuffer = self.document_control.get_buffer()

        sw = getScrolledWindow()
        sw.add(self.document_control)

##        vbox.pack_start(sw,expand=True)
        doc_pane.add2(sw)


        self.win = None
        self.show_all()

    def setDocumentFont(self,font_desc):
        'set the base font to use in document view text widget'
        self.document_control.modify_font(font_desc)

    def setEditorFont(self,font_desc):
        'set the base font to use in tag editor text widgets'
        self.source_control.modify_font(font_desc)
        self.target_control.modify_font(font_desc)


class DocumentViewPanelBase(DocumentViewPanelGUI,Observer):
    def __init__(self, *args,**kwargs):
        DocumentViewPanelGUI.__init__(self,*args,**kwargs)
        Observer.__init__(self)
##        self._closeSegUISetup()
        # TODO: convert to gtk
        self.sub_seg = None
        self.open_seg = None

        self.doc_model = None
        self.source_seg_model = None
        self.target_seg_model = None
        self.search_object = None

    def initGui(self):
        # set inactive color for text views
        color = gtk.gdk.color_parse("lightgray")
        for w in self.document_control,self.target_control,self.source_control:
            w.modify_base(gtk.STATE_INSENSITIVE,color)

        self._closeSegUISetup()
        self.tu_info_panel.initGui()
        
    def _openSegUISetup(self):
        self.document_control.Enable(False)
        self.source_control.Enable(True)
        self.target_control.Enable(True)
        self.target_control.grab_focus()
        parent = self.main_app
        for w in parent.widgets_to_disable_when_tu_open:
            w.set_sensitive(False)
        for w in parent.widgets_to_disable_when_tu_closed:
            w.set_sensitive(True)



    def _closeSegUISetup(self):
        self.document_control.Enable(True)
        self.source_control.Enable(False)
        self.target_control.Enable(False)
        self.document_control.grab_focus()

        self.diff_widget.hide_all()
        parent = self.main_app
        for w in parent.widgets_to_disable_when_tu_open:
            w.set_sensitive(True)
        for w in parent.widgets_to_disable_when_tu_closed:
            w.set_sensitive(False)


##        self.diff_widget.hide_all()


    def setupModels(self,doc_model,source_seg_model,target_seg_model):
        'set and connect models and viewers'
        if self.doc_model:
            self.unsubscribeToObservable(self.doc_model)
        self.doc_model = doc_model
        self.subscribeToObservable(self.doc_model,'cbDocumentDirty',DOCUMENT_DIRTY_EVENT)
        
        self.source_seg_model = source_seg_model
        self.target_seg_model = target_seg_model

        # ---- doc_model ----
        # bind document_control to doc_model
        self.document_control.setModel(self.doc_model)

        # ---- source/target models ----
        # Lock source segmodel for edit
        self.source_seg_model.locked = True
        # set and observe models
        self.source_control.get_buffer().setModel(self.source_seg_model)
        self.target_control.get_buffer().setModel(self.target_seg_model)

    def cbDocumentDirty(self,observable, event, seg):
        self.doc_model.changed = True
        self.main_app._updateGui()

    def cbSelectedSegmentChanged(self,observable, event, seg):
        'called by document_control when selected segment changes'
        self.source_seg_model.setRootNode(seg.tu.getSourceNode())
        self.source_seg_model.notifyObservers()
        try:
            target_node = seg.tu.getTargetNode()
        except TuException,e:
            self.target_seg_model.setRootNode(None)
            print e
        else:
            self.target_seg_model.setRootNode(target_node)
        self.target_seg_model.notifyObservers()
        # update TU status panel
        self.tu_info_panel.setTu(seg.tu)

        # Force full gui update of the document view if tu_info expanders changed visability
        # as it somtimes leaves a gray line in the document control otherwise.
        if self.tu_info_panel.ui_changed:
            print "FULL update"
            self.document_control.queue_draw()
            self.tu_info_panel.ui_changed = False

##        self.document_control.showInsertPoint()

        
    def onTuConstraintChanged(self,constraint):
        self.document_control.setConstraint(constraint)
        
    def onTuNext(self):
        if not(self.open_seg is None):
            raise EditorException('Segment Open')
        self.document_control.nextSeg()
        self.document_control.grab_focus()

    def onTuPrev(self):
        if not(self.open_seg is None):
            raise EditorException('Segment Open')
        self.document_control.prevSeg()
        self.document_control.grab_focus()

    def onTuOpen(self,*args):
        print "open TU"
        self.openTransUnit(self.document_control.current_seg)
        
    def openTransUnit(self,seg):
        if seg.__class__ != TuSegment:
            raise EditorException('Can only open TuSegment segs')

        self.open_seg = seg

        self.tu_info_panel.setTu(self.open_seg.tu)

        # openTU for edit, it notiyfies all observers it self
        snode,tnode = self.open_seg.tu.openForEdit()

        #TODO: do this so that sub seg sets both source/target control
        if self.sub_seg is None:
            self.sub_seg = SubSeg(self.open_seg.tu,self.source_control,self.target_control)
        else:
            # try to open a subseg(tag or mrk)
            self.sub_seg = self.sub_seg.open()
            return

        self._openSegUISetup()

    def onTuCommitClose(self,*args):
        print "Commit & Close"
        self._segClose(commit=True,common_state='translated')

    def onTuCommitApproveClose(self,*args):
        print "Commit, Close and approve"
        self._segClose(commit=True,approve=True)

    def onTuCancelClose(self,*args):
        print "Cancel & Close"
        self._segClose(commit=False)

    def _segClose(self,commit=False,approve=None,state=None,common_state=None):
        if self.open_seg is None:
            raise CloseTUSegmentException('No Segment open')

        from transolution.xmlutils import getText
        # Update the seg to reflect the current node content
        try:
            node = self.sub_seg.tu.getTargetNode()
            if commit:
                self.updateTM(self.sub_seg.tu)
            self.sub_seg = self.sub_seg.close(commit)

            # set caret to subsegs end
            self.target_control.get_buffer().setInsertionPoint( self.target_seg_model.getTextSpanOfNode(node)[1])

        except AtRootSegException,e:
            print e
            if commit and not len(getText(self.open_seg.tu.getTargetNode().childNodes)):
                raise CloseTUSegmentException('Cant commit when target empty')

            # commit, approve?
            self.open_seg.tu.closeForEdit(commit,approve,state=state,common_state=common_state)
            # Update doc ui to jump to end of closed segment
            self.document_control.get_buffer().setInsertionPoint(self.doc_model.getTextSpanOfSeg(self.open_seg)[1])
            self.document_control.showInsertPoint()

##            if commit:
##                self.updateTM(self.sub_seg.tu)

            self.sub_seg.updateUI()
            
            self.sub_seg = None
            self.open_seg = None
            self.tu_info_panel.setTu(None)

            self._closeSegUISetup()

    def onTuCopySource(self,*args):
        print "copy source"
        source_node = self.source_seg_model.rootnode
        self._replaceTargetNodesWithUndo(source_node.childNodes)

    def onTuApprove(self,*args):
        print "approve"
        # TODO: dont work if we are in a subseg
        tu = self.document_control.current_seg.tu
        tu.approved = 'yes'
        tu.notifyObservers(TU_CHANGE_EVENT)
        tu.notifyObservers(TU_UI_UPDATE_EVENT)


    def onAddNote(self,*args):
        print "Add Note"
        self.tu_info_panel.tu.addNote()
        # update gui
        self.tu_info_panel.tu.notifyObservers(TU_CHANGE_EVENT)
        self.tu_info_panel.tu.notifyObservers(TU_UI_UPDATE_EVENT)

##        self._segClose(commit=False)


    def onTagInsertAll(self,*args):
        print "copy all tags"
        tu = self.open_seg.tu
        snode = self.source_seg_model.rootnode
        tnode = self.target_seg_model.rootnode
        tmodel = self.target_seg_model
        i = 0
        for node in snode.childNodes:
            if node.nodeType is node.ELEMENT_NODE:
                tmodel.pasteNodesAtTextIndexWithUndo(i,[node,],False)
                node = tmodel.getNodeByTextIndex(i)[0]
                i = tmodel.getTextSpanOfNode(node)[1]
        self.sub_seg.updateUI()

    def onInsertTag(self,id):
        print "insert tag"
        tu = self.open_seg.tu
        smodel = self.source_seg_model
        tmodel = self.target_seg_model
        new_id = None
        old_tags = None
        # Check if tags already present in target
        if list(tmodel.getTagNodeByID(id)):
            # ask what to do
            INSERT_NEW,MOVE,CANCEL = 1,2,gtk.RESPONSE_CANCEL
            buttons = (("Move Existing", MOVE),
                       ("Insert new copy",INSERT_NEW),
                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
            
            res = askDialog(self.win,
                            'Tag already exists in target',
                            'Target already have tag(s) choose action.',
                            buttons)
            if res == INSERT_NEW:
                new_id = max(tmodel.getHighestId(),smodel.getHighestId())+1
            elif res == MOVE:
                # get the old tag(s) to remove after insertion
                old_tags = list(tmodel.getTagNodeByID(id))
            else:
                # Cancel
                return

        # get source tags to insert
        tags =  list(smodel.getTagNodeByID(id))
        if not tags:
            # No corresponding tags in source, try to find tag in the target
            tags = list(tmodel.getTagNodeByID(id))
            if not tags:
                raise EditorException('no tags with id:"%s" found'%(id,))

        # should tags be inserted with new id?
        if not(new_id is None):
            tags = [n.cloneNode(True) for n in tags]
            for tag in tags:
                setNodeAttr(tag,"id",str(new_id))

        if len(tags) == 1:
            ph = tags[0]
            start,end = self.target_control.get_buffer().getSelection()
            tmodel.pasteNodesAtTextIndexWithUndo(start,[ph,])
        elif len(tags) == 2:
            bpt,ept = tags
            if not((bpt.tagName == 'bpt' and ept.tagName == 'ept') or
               (bpt.tagName == 'bx' and ept.tagName == 'ex')):
                raise EditorException('Illegal paired tags')

            start,end = self.target_control.get_buffer().getSelection()
            tmodel.pasteNodesAtTextIndexWithUndo(start,[bpt,],interactive=False)
            
            node = tmodel.getNodeByTextIndex(start)[0]
            s,e = tmodel.getTextSpanOfNode(node)
            end += e-s # compensate for inserted tag
            tmodel.pasteNodesAtTextIndexWithUndo(end,[ept,],interactive=False)
        else:
            raise EditorException('Illegal number of tags with id found:%s'%(id,))


        if old_tags:
            # remove the old tags
            for node in old_tags:
                s,e = tmodel.getTextSpanOfNode(node)
                tmodel.deleteTextWithUndo(s,e,interactive=False,force=True)

##        self.target_control.showInsertPoint()
        self.sub_seg.updateUI()
        self.target_control.get_buffer().setInsertionPoint(start)

    def onTuCopyAltTrans(self,i,sub_target_index=0):
        '@i is the alttrans index number to copy'
        self._assertMainTuIsOpen()
        try:
            if i < 0: raise IndexError('negative alttrans index not supported')
            alttrans = list(self.open_seg.tu.getAltTrans())[i]
        except IndexError,e:
            raise EditorException('alttrans index ut of range:%i'%(i,))

        try: targetnode = alttrans.target_nodes[sub_target_index]
        except IndexError,e:
            raise EditorException('alttrans sub target index ut of range:%i'%(sub_target_index,))
        #insert the alt trans with undo
        self._replaceTargetNodesWithUndo(targetnode.childNodes)

    def _replaceTargetNodesWithUndo(self,nodelist):
        'inserts the childnodes of node into target_seg_model'
        self.target_seg_model.clearWithUndo(interactive=True)
        self.target_seg_model.pasteNodesAtTextIndexWithUndo(0,nodelist)
        self.sub_seg.updateUI()

    def onTuRemoveTarget(self,*args):
        print "Delete"
        self._assertMainTuIsOpen()
        self.open_seg.tu.removeAndCloseForEdit()
        self.sub_seg.updateUI()
        self.open_seg = None
        # empty open seg list
        self.sub_seg = None

        self._closeSegUISetup()
        
    def onCloseOpenNextFuzzy(self,approve=False):
        print "CloseOpenNextFuzzy"
        self._assertMainTuIsOpen()
        fromseg = self.open_seg
        if approve:
            self.onTuCommitApproveClose()
        else:
            self.onTuCommitClose()

        try:
            seg = self.doc_model.nextUntranslatedSegment(fromseg)
        except IndexError,ex:
            print ex
            buf = self.document_control.get_buffer()
            buf.place_cursor(buf.get_end_iter())
            self.document_control.showInsertPoint()
        else:
            self.openTransUnit(seg)
            # Update doc ui to show open segment
            start,end = self.doc_model.getTextSpanOfSeg(seg)
            self.document_control.get_buffer().setInsertionPoint(start)
            self.document_control.showInsertPoint()


    def _assertMainTuIsOpen(self):
        if self.open_seg is None:
            raise EditorException('No Segment open')
        elif self.sub_seg and self.sub_seg.parent:
            raise EditorException('Operation not permitted in a sub seg')

    def close(self):
        if self.open_seg:
            raise EditorException('Cant close when theres a open TU')
        else:
            # Do some cleanup to free objects
            self.doc_model.delete()
            del self.doc_model
            self.destroy()


import tm


class DocumentViewPanelTM(DocumentViewPanelBase):
    'Expands DocumentViewx with TM operations'
    def __init__(self, *args, **kwds):
        DocumentViewPanelBase.__init__(self, *args, **kwds)
        self.diff_control.set_editable(False)

    def updateTM(self,tu):
##        from transolution.xmlutils import getText
        print "Update TM"
        if tm.tm:
##            print getText(tu.getSourceNode().childNodes)
            tm.tm.addTU(tu.getSourceNode(),tu.getTargetNode())

    def _tmLookup(self):
        m = None
        if not tm.tm:
            raise EditorException('No TM connection')
            
        m = list(tm.tm.lookup(self.sub_seg.tu.getSourceNode()))[0]
        return m
    
    def onTuCopyFromTM(self):
        try: m = self._tmLookup()
        except tm.TmNoMatchFoundException,e:
            print e
            return

        #Insert the tm lookup result
        self._replaceTargetNodesWithUndo(m.target.childNodes)

        self.sub_seg.updateUI()


        tu = self.open_seg.tu
        if m.quality < 1.0:
            tu.state_qualifier = 'fuzzy-match'
            tu.state = 'needs-adaptation'
        else:
            tu.state_qualifier = 'exact-match'
            tu.state = 'needs-review-adaptation'
        
        tu.notifyObservers(TU_CHANGE_EVENT)

    def openTransUnit(self,*args,**kwargs):
        ret = DocumentViewPanelBase.openTransUnit(self,*args,**kwargs)
        if not settings.tm.automatic_lookup.lower() == 'true':
            return ret

        try: m = self._tmLookup()
        except tm.TmNoMatchFoundException,e:
            print e
            return


        self.showDiff(m)
        return ret

    def showDiff(self,m):
        '''
        show the differens visually in diff widget.
        @m is a tm lookup match object
        '''
        # Show the widget
        self.diff_widget.show_all()
        #gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT, gtk.STATE_SELECTED , gtk.STATE_INSENSITIVE
        text_buffer = self.diff_control.get_buffer()

        # clear text
        text_buffer.delete(*text_buffer.get_bounds())
        iter = text_buffer.get_iter_at_offset(0)

        if m.quality == 1.0:
            bg_color = 'bg_lightgreen'
        if m.quality < 1.0 :
            bg_color = 'bg_lightyellow' # 'bg_lightgray'
        # show match quality
        text_buffer.insert_with_tags_by_name(iter, 'Match quality:%0.3f\n'%(float(m.quality),), bg_color,'fg_black')

        # Show difference in diff textview
        from difflib import ndiff
        stylemap = {' ':['fg_black',],'+':['fg_blue','underline'],'-':['fg_red','strikethrough']}
        for item in m.getNDiff():
            s,t = item[0],item[2]
            text_buffer.insert_with_tags_by_name(iter, t, *stylemap[s])

        
        text_buffer.insert_with_tags_by_name(iter, '\n', 'fg_darkgray')
        text_buffer.insert_with_tags_by_name(iter, m.ttext, 'fg_darkgray')



class DocumentViewPanel(DocumentViewPanelTM):
    'Expands DocumentViewx with search & replace operations'
    def __init__(self, *args, **kwds):
        DocumentViewPanelTM.__init__(self, *args, **kwds)
        self.findgen = None

    def _initSearch(self,s,reset_offset=False):
        model = Model(None,self.target_seg_model.formatter.__class__(),protect_tags=True)
        'init new search object'
        self.search_object = Search(self.doc_model,s['search_phrase'],
                                    search_source=s['search_source'],
                                    search_target=s['search_target'],
                                    case=s['case'],model = model)

        if reset_offset:    offset = None
        elif self.open_seg: offset = self.open_seg
        else:               offset = self.doc_model.getSegByTextIndex(self.document_control.get_buffer().getInsertionPoint())[0]
        self.findgen = self.search_object.doSearch(reverse=s['reverse'],offset=offset)
        self.old_settings = s

    def onSearch(self,s,search_dlg):
        'callback for search dialog'
        print "OnSearch"
        self.search_dlg = search_dlg
        # create new search generator?
        if not self.findgen or self.old_settings != s:
            self._initSearch(s)

        try:
            # get next found instance
            f =self.findgen.next()
        except StopIteration,e:
            # Search ended
            self.findgen = None
            # should we wrap around and start again?
            if not self._askWrapSearch():
                return

            # Close the open seg
            self._segCloseForce(search_dlg)

            # wrap around search
            if s['reverse']:
                self.document_control.get_buffer().setInsertionPointEnd()
            else:
                self.document_control.get_buffer().setInsertionPoint(0)
            self.onSearch(s,search_dlg)
        else:
            # update GUI
            self._showFound(f)
        search_dlg.grab_focus()

    def _showFound(self,f):
        'update gui to show found term. f: found object'
        # should we change segment?
        if f.seg != self.open_seg:
            # Update GUI to Show found phrase
            # Close the open seg
            self._segCloseForce(self.search_dlg)
            self.openTransUnit(f.seg)
        # update document widget to show open segment
        start,end = self.doc_model.getTextSpanOfSeg(f.seg)
        self.document_control.get_buffer().setInsertionPoint(start)
        # update TU status info in document view window
        self.document_control.notifyObservers(msg=f.seg)

        
        if f.segtype =='source':
            #self.source_control.grab_focus()
            self.source_control.get_buffer().setSelection(f.start,f.end)
        elif f.segtype =='target':
            #self.target_control.grab_focus()
            self.target_control.get_buffer().setSelection(f.start,f.end)

        self.document_control.showInsertPoint()


    def _askWrapSearch(self):
        'Ask for wrap around search'
        res = askOk(self.search_dlg,'End reached', 'End reached, wrap around and continue?')
        return res

    def _askCommit(self,parent_dialog=None):
        '''Open dialog to Ask for commit 
           returns Boolean
           raises CloseCancelTUSegmentException if cancel
        '''
        res = askOk(parent_dialog,'Commit?','Current segment changed, do you want to commit?')
        if not res:
            raise CloseCancelTUSegmentException()
        return res

    def _segCloseForce(self,parent_dialog=None):
        '''Close the open seg, remove it if empty ask if it has changed'''
        try:
            # ask for commit if target segment has changed
            if self.target_seg_model.undobuffer:
                commit = self._askCommit(parent_dialog=parent_dialog)
            else:
                commit = False
            self._segClose(commit=commit)
        except CloseCancelTUSegmentException,e:
            print e
            self.onTuCancelClose()
        except EditorException,ex:
            print ex
            try:self.onTuRemoveTarget(None)
            except EditorException,ex: print ex

    def onReplace(self,s,search_dlg):
        'callback for replace dialog'
        print "Replace"

        if s['replace_all']:
            replaced = self.replaceAll(s,search_dlg)
            messageDialog(search_dlg,'replaced:%d'%(replaced,))

            return

        if not self.search_object:
            self.onSearch(s,search_dlg)
            return

        # TODO:DONE! Possible problem: found object has a separate model instead of self.target_seg_model

        # Replace and update UI
        self.search_object.foundobj.replace(s['replace_phrase'])

        self.source_seg_model.notifyObservers()
        self.target_seg_model.notifyObservers()
        # save changes to segment
        try:
            self._segClose(commit=True)
        except CloseTUSegmentException,ex: print ex
        # find next
        self.onSearch(s,search_dlg)
        search_dlg.grab_focus()

    def replaceAll(self,s,search_dlg):
        'replace all'
        try:
            if self.target_seg_model.undobuffer:
                commit = self._askCommit(parent_dialog=search_dlg)
            else:
                commit = False
            self._segClose(commit=commit)
        except CloseTUSegmentException,ex: print ex

        print "Create new search object"
        self._initSearch(s,reset_offset=True)
        replaced = 0
        for f in self.findgen:
            try:
                # Replace
                f.replace(s['replace_phrase'])
                replaced += 1
            except XliffError,e:
                print e


        #update UI
        self.source_seg_model.notifyObservers()
        self.target_seg_model.notifyObservers()

        self.doc_model.notifyObservers(UI_UPDATE_EVENT)
        if replaced: self.doc_model.notifyObservers(DOCUMENT_DIRTY_EVENT)
        
        search_dlg.grab_focus()
        return  replaced

    def onChangeStatus(self,s,dlg):
        internal_segs = [seg for seg in self.doc_model.segs if seg.__class__ == TuSegment]
        from_state = s['from_state']
        to_state = s['to_state']
        from_state_q = s['from_state_q']
        to_state_q = s['to_state_q']

        approvemap = {None:None, True:'yes', False:'no'}
        from_approved = approvemap[s['from_approved']]
        to_approved = approvemap[s['to_approved']]
        changed = False
        changes = 0
        for seg in internal_segs:
            tu = seg.tu
            if not tu.hasTarget():
                continue
            # check constraints
            if not (from_approved is None):
                if from_approved != tu.approved:
                    continue
            if not (from_state is None):
                if from_state != tu.state and (from_state or tu.state):
                    continue
            if not (from_state_q is None):
                if from_state_q != tu.state_qualifier and (from_state_q or tu.state_qualifier):
                    continue
            # do the changes
            if not (to_state is None) and seg.tu.state != to_state:
                seg.tu.state = to_state
                changed = True
            if not (to_state_q is None) and seg.tu.state_qualifier != to_state_q:
                seg.tu.state_qualifier = to_state_q
                changed = True
            if not (to_approved is None) and tu.approved != to_approved:
                tu.approved = to_approved
                changed = True
            if changed:
                changes += 1
                changed = False
                seg.dirty = True
        self.doc_model.notifyObservers(UI_UPDATE_EVENT)
        if changes:
            print "CHANGED!"
            self.doc_model.notifyObservers(DOCUMENT_DIRTY_EVENT)
        return changes
