#!/usr/bin/python
#  Transolution Trans XLIFF Editor
#  Copyright (C) 2005 Fredrik Corneliusson <fredrik.corneliusson at gmail.com>
#
#  URL: http://transolution.python-hosting.com
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
__version__ = '0.4 Beta 5'

import sys
import os
import pygtk
#tell pyGTK, if possible, that we want GTKv2
try:
    pygtk.require("2.0")
except:
    pass

import gtk
import gtk.glade

from transolution.observer import Observer,Observable
from gtk_utils import askOk,askDialog,messageDialog,aboutDialog,cb_get_active_text,entryDialog,create_tag_table,fileChooserDialog,dropDownDialog
from general import EditorException

from gtk_search_replace_dialog import SearchReplaceDialog
from gtk_status_change_dialog import StatusChangeDialog
from gtk_documentviewpanel import DocumentViewPanel,DOCUMENT_DIRTY_EVENT
from document.creators import createXliffDocumentModel,getDocumentFormatter
from undoredo import UndoException,RedoException

from xliffsegment import Model #TODO: Model name is to generic
from settings import settings
from gtk_documenteditor import UI_UPDATE_EVENT

# I18n support
import gettext
domain='transolution'

localePath=gettext.bindtextdomain(domain)
gettext.install(domain, localePath, unicode=1)
gtk.glade.bindtextdomain(domain, localePath)

# setup prev next seg constraints
from document.constraints import unTranslated,unApproved
constraint_dict = {0:(None,'None'),1:(unTranslated,_('Untranslated')),2:(unApproved,_('Unapproved'))}

# set the tu state dialect
import document.tu.state_maps
document.tu.state_maps.state_dialect = 'standard'
##document.tu.state_maps.state_dialect = 'suntrans'

def getEditorFormatter(FormatterClass):
    'create xliff source/target node formatter'
    # setup styles
    sText = ('fg_black',)
    sTag = ('fg_red','bold',)
    sSub = ("bg_lightyellow",)

    formatter = FormatterClass()
    formatter.setStyles(sText,sTag,sSub)
    return formatter


import tm
from transolution import getLangCodes
import transolution.xliffeditor
from transolution.path_mod import path


class TmSettings(object):
    'store tm connection settings'
    def __init__(self,address):
        self.addressString(address)
    def addressString(self,s):
        'parse a addresstring (extract port)'
        parts = s.split(':')
        self.address = ':'.join(parts[:-1])
        self.port = int(parts[-1])

    def __str__(self):
        return '%s:%d'%(self.address,self.port)
    
class MainWindowBase(Observer):
    def __init__(self, parent=None):
        Observer.__init__(self)
        self.interactive = True # flag to block non interavtive changes being handled

        windowname="MainWindow"
        # Create the toplevel window
        self.wTree=gtk.glade.XML (transolution.xliffeditor.gladefile,windowname,domain)
        self.window = self.wTree.get_widget(windowname)
        self.notebook = self.wTree.get_widget("notebook")

        self.notebook.remove_page(0)

        # connect
        self.wTree.signal_autoconnect(self)
        # The default folder to use in fileopen
        self.last_open_folder = None
        self.docpane = None
        # Setup Document Formatters
        from formatters import FullFormatterTagId,FullFormatter,MinimalFormatter

        f = getEditorFormatter
        self.editor_formatters = {}
        self.editor_formatters['Minimal'] = f(MinimalFormatter)
        self.editor_formatters['Medium'] = f(FullFormatter)
        self.editor_formatters['Full'] = f(FullFormatterTagId)
        self.doc_sub_formatters = self.editor_formatters

        # copy the windgets we need access to
        g = self.wTree.get_widget
        self.tb_tagprotection = g("tb_tagprotection")
        self.menu_tagprotection = g("menu_tagprotection")
        self.cb_tu_constraint = g("tu_constraint")

        self.tu_closed_toolbar = g("tu_closed_toolbar")

        self.show_source_target_radio_bt = {'Source':g("mb_show_source"),
                                            'Target':g("mb_show_target"),
                                            'Both':g("mb_show_both")}

        self.doc_formatter_radio_bt = {'Full':g("mb_formatter_full"),
                                       'Medium':g("mb_formatter_medium"),
                                       'Minimal':g("mb_formatter_minimal")}

        self.editor_formatter_radio_bt = {'Full':g("mb_editor_formatter_full"),
                                          'Medium':g("mb_editor_formatter_medium"),
                                          'Minimal':g("mb_editor_formatter_minimal")}

        self.mb_tm_active = g("tm_active")
        self.cb_tm_automatic_lookup = g("cb_tm_automatic_lookup")
        self.tb_save = g("tb_save")
        self.menu_save = g("menu_save")


        self.menu_skeleton = g("menu_skeleton")
        self.skeleton_radio_bt = {'Full':g("mb_skeleton_full"),
                                  'Minimal':g("mb_skeleton_minimal"),
                                  'None':g("mb_skeleton_none")}

        disable_when_tu_closed = ["tu_open_toolbar","tags_menu",
                                  "commit_close1","cancel_close1",
                                  "remove_target_close1","commit_open_next1",
                                  "copy_source1","copy_from_tm1","copy_alttrans1",
                                  ]
        self.widgets_to_disable_when_tu_closed = [g(w) for w in disable_when_tu_closed]
        self.widgets_to_disable_when_tu_open = [g(w) for w in ("tu_nav_toolbar","next_tu_menuitem","previous_tu_menuitem")] #self.tu_closed_toolbar]

        self.widgets_to_disable_if_no_file_open = [g(w) for w in ('toolbar_file','edit_menu','view_menu','tu_menu','tags_menu','document_operations')]

        # setup combobox TU prev/next constraint
        for i,value in constraint_dict.items():
            self.cb_tu_constraint.insert_text(i,value[1])
        self.cb_tu_constraint.set_active(0)

        # create a TextTagTable
        self.tag_table = create_tag_table()        

        # Default fonts
        self.document_font_desc = None
        self.editor_font_desc = None

        # load default fonts from settings
        import pango
        if settings.fonts.document_font:
            self.document_font_desc = pango.FontDescription(settings.fonts.document_font)
        if settings.fonts.editor_font:
            self.editor_font_desc = pango.FontDescription(settings.fonts.editor_font)
        
        self._updateGui()

    # Document Formatter change handlers
    def onDocumentFormatterFull(self,widget):
        self._setDocFormatter('Full')
    def onDocumentFormatterMedium(self,widget):
        self._setDocFormatter('Medium')
    def onDocumentFormatterMinimal(self,widget):
        self._setDocFormatter('Minimal')

    def _setDocFormatter(self,name):
        'sets the document sub formatter'
        if not self.interactive: return False
        sub_formatter = self.doc_sub_formatters[name]
        doc_formatter = self.docpane.doc_model.getFormatter()
        doc_formatter.setTagFormatter(sub_formatter)
        from document.segments import TuSegment
        self.docpane.doc_model.setDirty(TuSegment)
        self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
##        self.docpane.document_control.showInsertPoint()

    # Editor Formatter change handlers
    def onEditorFormatterFull(self,widget):
        self._setEditorFormatter('Full')
    def onEditorFormatterMedium(self,widget):
        self._setEditorFormatter('Medium')
    def onEditorFormatterMinimal(self,widget):
        self._setEditorFormatter('Minimal')

    def _setEditorFormatter(self,name):
        'sets the editor formatter for the current docpane'
        if not self.interactive: return False
        f = self.editor_formatters[name]
        if not(f is self.docpane.target_seg_model.getFormatter()):
            print "Formatter:",name

            self.docpane.source_seg_model.setFormatter(f)
            self.docpane.source_seg_model.notifyObservers()
            self.docpane.target_seg_model.setFormatter(f)
            self.docpane.target_seg_model.notifyObservers()
##            self.docpane.target_seg_model.showInsertPoint()

    def onSkeletonFull(self,cb):
        if not self.interactive: return False
        if cb.get_active():
            self._setSkeletonFormatterMode('Full')
        return True

    def onSkeletonMinimal(self,cb):
        if not self.interactive: return False
        if cb.get_active():
            self._setSkeletonFormatterMode('Minimal')
        return True
    
    def _setSkeletonFormatterMode(self,mode):
        doc_formatter = self.docpane.doc_model.getFormatter()
        doc_formatter.setSkeletonFormatterMode(mode)
        if self.docpane.doc_model.use_skeleton:
            from document.segments import External
            self.docpane.doc_model.setDirty(External)
            self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
        else:
            # no skeleton used before so we need to reparse using skeleton
            self.docpane.document_control.buf.clear()
            self.docpane.doc_model.parseXliff(use_skeleton=True)
            self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
            return True

    def onSkeletonNone(self,cb):
        if not self.interactive: return False
        if not cb.get_active(): return True

        print "Use Skeleton:", False
        self.docpane.document_control.buf.clear()
        print "Buffer Cleared"
        
        self.docpane.doc_model.parseXliff(use_skeleton=False)
        self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
        return True

    def onShowSource(self,cb):
        if not self.interactive: return False
        self._setShowSourceTarget(True,False)
    def onShowTarget(self,cb):
        if not self.interactive: return False
        self._setShowSourceTarget(False,True)
    def onShowBoth(self,cb):
        if not self.interactive: return False
        self._setShowSourceTarget(True,True)

    def _setShowSourceTarget(self,source,target):
        f = self.docpane.doc_model.getFormatter()
        f.showSource(source)
        f.showTarget(target)
        from document.segments import TuSegment
        self.docpane.doc_model.setDirty(TuSegment)
        self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
        return True

    def onCbTargetActivate(self,cb):
        if not self.interactive: return False
        f = self.docpane.doc_model.getFormatter()
        f.showTarget(cb.get_active())
        from document.segments import TuSegment
        self.docpane.doc_model.setDirty(TuSegment)
        self.docpane.doc_model.notifyObservers(UI_UPDATE_EVENT)
        return True
        

    def findFocus(self):
        'return the focused widget in mainwindow(just like wx)'
        return self.window.get_focus()

    def _checkTargetLang(self,fname,xliff):
        '''Initialize target language of @xliff if there is none
           @returns True if language setting was changed
        '''
        if len(xliff.files):
            f = xliff.files[0]
            if not f.getTargetLang():
                msg = _('Source language=%s\n'%(f.getSourceLang()))
                msg += _('No target language set for file.\n')
                msg += _('Please enter target language to use.')

                try:
                    drop_down_values = ['%s (%s)'%(lcid,country) for lang,country,lcid in list(getLangCodes())]
                    drop_down_values.sort()
                except IOError,e:
                    drop_down_values = None

                target_lang = entryDialog(self.window,_('Set target language for "%s"'%(path(fname).basename()) ),msg, _('Target language'),
                                          default_response=gtk.RESPONSE_OK,drop_down_values=drop_down_values)
                if target_lang:
                    # use the first part of the result string as it can have language description e.g. "AF-ZA (South Africa)"
                    target_lang = target_lang.split()[0]
                    # set the target languge the same for all files in the XLIFF (can this be a problem?)
                    for f in xliff.files:
                        f.setTargetLang(target_lang)
                    return True
        return False

    def openFile(self,xml):
        'Open the xml file and add the Document to the notepad'
        doc = createXliffDocumentModel(xml, use_skeleton=True )
        doc.setFormatter(getDocumentFormatter(self.doc_sub_formatters['Medium']))
        changed = self._checkTargetLang(xml,doc.xliff)
        

        # Create the DocumentViewPanel
        notebook = self.notebook
        docpanel = DocumentViewPanel(notebook,main_app=self, tag_table=self.tag_table)

        # Set default fonts
        if self.document_font_desc: docpanel.setDocumentFont(self.document_font_desc)
        if self.editor_font_desc: docpanel.setEditorFont(self.editor_font_desc)

        # create models and bind them to GUI
        formatter = self.editor_formatters['Full']
        docpanel.setupModels(doc,Model(None,formatter,protect_tags=True),Model(None,formatter,protect_tags=True))

        # update document view
        doc.notifyObservers(UI_UPDATE_EVENT)

        # setup mode
        docpanel.initGui()

        # Add a page to the notebook
        notebook.append_page(docpanel, gtk.Label(os.path.basename(xml)))
        # Set selection to last page
        notebook.set_current_page(notebook.get_n_pages()-1)
        self._updateGui()

        if changed:
            # tell observers that we changed the document
            docpanel.doc_model.notifyObservers(DOCUMENT_DIRTY_EVENT)

    def _updateGui(self):
        'update GUI to reflect the current document'
        print "_updateGui"
        self.interactive = False

        # check if no file is open
        if self.notebook.get_n_pages() ==0:
            # disable file conditional widgets
            for w in self.widgets_to_disable_if_no_file_open:
                w.set_sensitive(False)
            self.interactive = True
            return

        # enable file conditional widgets
        for w in self.widgets_to_disable_if_no_file_open:
            w.set_sensitive(True)

        doc_model = self.docpane.doc_model
        
        # update doc formatter Gui
        current = doc_model.getFormatter().getTagFormatter()

        for name,formatter in self.doc_sub_formatters.items():
            self.doc_formatter_radio_bt[name].set_active(current is formatter)

        # update editor formatter Gui
        current = self.docpane.source_seg_model.getFormatter()
        for name,formatter in self.editor_formatters.items():
            self.editor_formatter_radio_bt[name].set_active(current is formatter)


        # update source/target check boxes
        f = doc_model.getFormatter()
        self.show_source_target_radio_bt['Both'].set_active(f.getShowSource() and f.getShowTarget())
        self.show_source_target_radio_bt['Source'].set_active(f.getShowSource() and not f.getShowTarget())
        self.show_source_target_radio_bt['Target'].set_active(not f.getShowSource() and f.getShowTarget())

        # update use skeleton checkbox
        if doc_model.has_skeleton:
            self.menu_skeleton.set_sensitive(True)
        else:
            self.menu_skeleton.set_sensitive(False)

        for radio in self.skeleton_radio_bt.values(): radio.set_active(False)
        if doc_model.has_skeleton and doc_model.use_skeleton:
            mode = doc_model.getFormatter().getSkeletonFormatterMode()
            self.skeleton_radio_bt[mode].set_active(True)
        else:
            self.skeleton_radio_bt['None'].set_active(True)

        # automatic tm lookup
        self.cb_tm_automatic_lookup.set_active(settings.tm.automatic_lookup.lower() == 'true')
        # tag protection
        model = self.docpane.target_control.getModel()
        self.tb_tagprotection.set_active(model.protect_tags)
        self.menu_tagprotection.set_active(model.protect_tags)
##        model.protect_tags = self.tb_tagprotection.get_active()
        self.onTuConstraintChanged()

        # save icon
        self.tb_save.set_sensitive(doc_model.changed)
        self.menu_save.set_sensitive(doc_model.changed)
        
        self.interactive = True

    def onNotebookChangePage(self,widget, dummy, pagenum):
        if not self.interactive: return False
        print "page change!"
        self.docpane = self.notebook.get_nth_page(pagenum)

        print "Activating doc models formatter"
        self.docpane.doc_model.getFormatter().activate()

        print "Current Docpane",self.docpane
##        # setup the TU toolbars
##        for w in self.widgets_to_disable_when_tu_open:
##            w.set_sensitive(self.docpane.open_seg is None)
##        for w in self.widgets_to_disable_when_tu_closed:
##            w.set_sensitive(self.docpane.open_seg is not None)
##
        self._updateGui()

    def onDocumentFont(self,*args):
        font_desc = self.selectFont(self.docpane.document_control)
        if font_desc:
            settings.fonts.document_font = font_desc.to_string()
            self.document_font_desc = font_desc
            # update all notbook pages
            for pagenum in range(self.notebook.get_n_pages()):
                o = self.notebook.get_nth_page(pagenum)
                o.setDocumentFont(font_desc)

    def onEditorFont(self,*args):
        font_desc = self.selectFont(self.docpane.source_control)
        if font_desc:
            settings.fonts.editor_font = font_desc.to_string()
            self.editor_font_desc = font_desc
            # update all notbook pages
            for pagenum in range(self.notebook.get_n_pages()):
                o = self.notebook.get_nth_page(pagenum)
                o.setEditorFont(font_desc)
        
    def selectFont(self, widget):
        import pango
        font_dialog = gtk.FontSelectionDialog("Font Selection Dialog")
        # get the current font
        fontname = widget.get_pango_context().get_font_description().to_string()
        font_dialog.set_font_name(fontname)
        response = font_dialog.run()
        font_dialog.destroy()
        if response != gtk.RESPONSE_OK:
            return None
        
        # get and set new font
        font = font_dialog.get_font_name()
        font_desc = pango.FontDescription(font)
        return font_desc        
##        print "LARGESCALE",pango.SCALE_LARGE
##        self.tag_table.foreach(lambda a,b: a.set_property('scale',pango.SCALE_LARGE),None)


class MainWindowBase1(MainWindowBase):
    'Add Drag/Drop fileopen support MainWindowBase'
    def __init__(self,*args,**kwargs):
        MainWindowBase.__init__(self,*args,**kwargs)

        # enable Drag/Drop
        self.window.drag_dest_set(gtk.gdk.ACTION_DEFAULT, [], 0)
        self.window.connect('drag_drop', self.target_drag_drop)
        self.window.connect('drag_data_received', self.target_drag_data_received)

    def target_drag_drop(self, widget, context, x, y, time):
        if context.targets:
            widget.drag_get_data(context, context.targets[0], time)
            return True
        return False

    def target_drag_data_received(self, widget, context, x, y, selection, targetType,time):
        'Called when things are dropped on the main window'
        if selection.format == 8:
            from urllib import url2pathname
            # try and open all file URL's dropped
            for fname in selection.data.split(os.linesep):
                fname = fname.replace('file:///','')
                fname = path(url2pathname(fname))
                if fname.isfile():
                    self.openFile(fname)
            context.finish(True, False, time)
        else:
            context.finish(False, False, time)


class MainWindowBase2(MainWindowBase1):
    'Add Search & replace dialog and callbacks to MainWindowBase'
    def __init__(self,*args,**kwargs):
        MainWindowBase1.__init__(self,*args,**kwargs)
        self.search_replace_dlg = SearchReplaceDialog(self.window)
        self.search_replace_dlg.hide()
        self.search_replace_dlg.setSearchListner(self.searchListner) # Add callback for search dialog search event
        self.search_replace_dlg.setReplaceListner(self.replaceListner) # Add callback for replace dialog search event

        self.batch_change_status_dialog = StatusChangeDialog(self.window)
        self.batch_change_status_dialog.hide()
        self.batch_change_status_dialog.setChangeListner(self.changeListner) # Add callback for search dialog search event


    def searchListner(self,s,dialog):
        self.docpane.onSearch(s,dialog)
    def replaceListner(self,s,dialog):
        self.docpane.onReplace(s,dialog)

    def changeListner(self,s,dialog):
        return self.docpane.onChangeStatus(s,dialog)

class MainWindow(MainWindowBase2):
    'Implement GUI event handlers for GUI'
    def __init__(self,*args,**kwargs):
        MainWindowBase2.__init__(self,*args,**kwargs)
        address = settings.tm.address
        if not address:
            from tm.tm_client import default_address,default_port
            address = '%s:%d'%(default_address,default_port)
        self.tm_settings = TmSettings(address)

        from webdict import WebDict
        self.webdict = WebDict()
        for site in settings.webdict.sites:
            if site.name == settings.webdict.current:
                self.webdict.setUrl(site.url,site.name,encoding=site.encoding)

    def onQuit(self,*args):
        if self.notebook.get_n_pages() > 0:
            changes = False
            for i in range(self.notebook.get_n_pages()):
                docpane = self.notebook.get_nth_page(i)
                if docpane.doc_model.changed: changes = True
            if changes:
                if not askOk(self.window,_('Quit'),_('There are unsaved changes.\nAre you sure you want to quit?')):
                    return True
        # save settings
        settings.save()
        gtk.main_quit()
        return False

    def onFileClose(self,*args):
        'Close the selected notbook page'
        n = self.notebook.get_current_page()
        o = self.notebook.get_nth_page(n)
        if o.doc_model.changed:
            # ask what to do
            SAVE,DISCARD,CANCEL = 1,2,gtk.RESPONSE_CANCEL
            buttons = ((gtk.STOCK_YES, gtk.RESPONSE_YES),
                       (gtk.STOCK_NO, gtk.RESPONSE_NO),
                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
            
            res = askDialog(self.window,
                            _('Close Document'),
                            _('The document has changed\nDo you want to save?'),
                            buttons)
            if res == gtk.RESPONSE_YES:
                self.onFileSave()
            elif res == gtk.RESPONSE_NO:
                pass
            else: # Cancel
                return True

        self.notebook.remove_page(n)
        # Close (destroy) will remove the notebook page also so do this after we call remove_page.
        # or we will remove an extra page.
        o.close()
        o.destroy()
        self._updateGui()
        return True
        
    def onFileOpen(self,*args):
        print "OPEN!"
        filefilters = (("Xliff files (.xlf;.xlz;.xliff)",("*.xlf","*.xlz","*.xliff",)),
                       ("All files",("*",),))
        fnames = fileChooserDialog(filefilters=filefilters,folder=self.last_open_folder)
        if fnames:
            self.last_open_folder = os.path.split(fnames[0])[0]
            for fname in fnames:
                self.openFile(fname)

    def onFileSave(self,*args):
        print "Save!"
        self.docpane.doc_model.xliff.save()
        self.docpane.doc_model.changed = False
        self._updateGui()
        
    def onFileSaveAs(self,*args):
        print "Save As!"
        docpane = self.docpane
        dialog = gtk.FileChooserDialog(_("Save As"),
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        orgfname = os.path.abspath(docpane.doc_model.xliff.fname)

        # set default path and filename
        fpath,fname = os.path.split(orgfname)
        dialog.set_current_folder(fpath)
        dialog.set_current_name(fname)

        # set the file type to use
        if docpane.doc_model.xliff.xlz: ftype = "xlz"
        else: ftype = "xlf"

        filter = gtk.FileFilter()
        filter.set_name(ftype)
        filter.add_pattern("*.%s"%(ftype,))
        dialog.add_filter(filter)

        response = dialog.run()
        fname = None
        if response == gtk.RESPONSE_OK:
            fname = dialog.get_filename()
            self.last_open_folder = os.path.split(fname)[0]
            # fix extension
            ext = "." + ftype
            if os.path.splitext(fname)[1] != ext: fname += ext
            # save the file
            docpane.doc_model.xliff.saveAs(fname)
            # upate notebook lable
            self.notebook.set_tab_label_text(self.docpane,os.path.basename(fname))
            docpane.doc_model.changed = False
            self._updateGui()

        dialog.destroy()

    def onOpenSearchDialog(self,*args):
        self.search_replace_dlg.enableReplace(False)
        self.search_replace_dlg.show()
    def onOpenReplaceDialog(self,*args):
        self.search_replace_dlg.enableReplace(True)
        self.search_replace_dlg.show()
        
    def onOpenBatchChangeStatusDialog(self,*args):
        self.batch_change_status_dialog.show()

    def onEditUndo(self,*args):
        try: self.findFocus().undo()
        except AttributeError,e: print e
        except UndoException,e:
            pass
    def onEditRedo(self,*args):
        try: self.findFocus().redo()
        except AttributeError,e: print e
        except RedoException,e:
            pass


    def onEditCopy(self,*args):
        self.findFocus().emit('copy_clipboard')
    def onEditCut(self,*args):
        self.findFocus().emit('cut_clipboard')
    def onEditPaste(self,*args):
        self.findFocus().emit('paste_clipboard')

    # These methods work like a proxy for the docpane handlers
    # as I did not find a nice way to unbind signals done by "signal_autoconnect"
    def onTuConstraintChanged(self,*args):
        print "TU constraint Changed!"
        if self.docpane:
            constraint = constraint_dict[self.cb_tu_constraint.get_active()][0]
            try:self.docpane.onTuConstraintChanged(constraint)
            except EditorException,e:
                messageDialog(self.window,'%s'%(e,))
        
    def onTuNext(self,*args):
        try:self.docpane.onTuNext()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuPrev(self,*args):
        try:self.docpane.onTuPrev()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onTuOpen(self,*args):
        try:
            self.docpane.onTuOpen()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuCommitClose(self,*args):
        try:self.docpane.onTuCommitClose()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuCancelClose(self,*args):
        try:self.docpane.onTuCancelClose()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuRemoveTarget(self,*args):
        try:self.docpane.onTuRemoveTarget()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuApproveOpenNextFuzzy(self,*args):
        try:self.docpane.onCloseOpenNextFuzzy(approve=True)
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuCommitOpenNextFuzzy(self,*args):
        try:self.docpane.onCloseOpenNextFuzzy()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))
    def onTuApprove(self,*args):
        try:self.docpane.onTuApprove()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onTuCopySource(self,*args):
        try:self.docpane.onTuCopySource()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onTagInsertAll(self,*args):
        try:self.docpane.onTagInsertAll()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onAddNote(self,*args):
        try:self.docpane.onAddNote()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onTagInsert(self,*args):
        res= entryDialog(self.window,_('Insert tag'),_('Set tag number to insert'),_('tag id'))
        if not(res is None):
            self._insertTag(res)

    def onInsertTag1(self,*args): self._insertTag(1)
    def onInsertTag2(self,*args): self._insertTag(2)
    def onInsertTag3(self,*args): self._insertTag(3)
    def onInsertTag4(self,*args): self._insertTag(4)
    def onInsertTag5(self,*args): self._insertTag(5)
    def onInsertTag6(self,*args): self._insertTag(6)
    def onInsertTag7(self,*args): self._insertTag(7)
    def onInsertTag8(self,*args): self._insertTag(8)
    def onInsertTag9(self,*args): self._insertTag(9)
    def _insertTag(self,id):
        id = str(id)
        try:self.docpane.onInsertTag(id)
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    def onTuCopyAltTrans(self,*args):
        res= entryDialog(self.window,_('Insert Alt-trans'),_('Set alttrans index number to insert'),_('alttrans id'))
        if not(res is None):
            try:self.docpane.onTuCopyAltTrans(int(res)-1)
            except EditorException,e:
                messageDialog(self.window,'%s'%(e,))

    def onTuCopyFromTM(self,*args):
        try:self.docpane.onTuCopyFromTM()
        except EditorException,e:
            messageDialog(self.window,'%s'%(e,))

    # TM handlers
    def onTmActive(self,widget):
        if widget.get_active():
            print "TM Active!"
            try: self.openTm()
            except EditorException,e: pass
            else:messageDialog(self.window,_("TM Connected:\n")+tm.tm.version())
        else:
            tm.tm = None
            print "NO TM Active!"

    def onTmAutomaticLookup(self,widget):
        if widget.get_active():
            settings.tm.automatic_lookup = str(True)
            print "Automatic TM Lookup!"
        else:
            settings.tm.automatic_lookup = str(False)
            print "Automatic TM lookup disabled"

    def onTmSetup(self,widget):
        print "TM Setup!"
        address = str(self.tm_settings)
        res= entryDialog(self.window,_('TM Connection'),_('Select TM address'),_('address'),default_val=address)
        if res is None:
            return
        settings.tm.address = res
        self.tm_settings.addressString(res)
        if self.mb_tm_active.get_active():
            try: self.openTm()
            except EditorException,e: pass
            else:messageDialog(self.window,_("TM Connected:\n")+tm.tm.version())

    def openTm(self):
        from tm import TMClient
        from socket import error as sockerr
        try:
            tm.tm = TMClient(self.tm_settings.address,self.tm_settings.port)
            tm.tm.version()
        except (sockerr,IOError),e:
            tm.tm = None
            messageDialog(self.window, '%s'%(e,))
            self.mb_tm_active.set_active(False)
            raise EditorException('Error connecting to TM')

    def onCbShowTuStatus(self,widget):
        if widget.get_active():
            self.docpane.tu_info_panel.show()
        else:
            self.docpane.tu_info_panel.hide()


    def onToggleProtectTags(self,w,*args):
        print "onToggle"
        model = self.docpane.target_control.getModel()
        model.protect_tags = w.get_active()
        print "TagProtection:%s"%(('off','on')[model.protect_tags],)
        model.notifyObservers()
        self._updateGui()

    def onWebLookupConfig(self,*args):
        print "WebLookup Config"
        try:
            drop_down_values = [site.name for site in settings.webdict.sites]
            drop_down_values.sort()
        except IOError,e:
            drop_down_values = None
        msg = _('Select the web dictionary to use.')
        result_webdict = dropDownDialog(self.window,_('Set web url to use for web lookups' ),
                                        msg, _('Web post url'),
                                        drop_down_values,
                                        default_val=settings.webdict.current,
                                        default_response=gtk.RESPONSE_OK)
        if result_webdict:
            settings.webdict.current = result_webdict
            # Get the site that matches
            site, = [site for site in settings.webdict.sites if site.name == settings.webdict.current]
            self.webdict.setUrl(site.url,site.name,encoding=site.encoding)


    def onWebLookup(self,*args):
        print "WebLookup"
        from gtk_clipboard import clipboard
        text = clipboard.getText()
        if text:
            self.webdict.lookup(text)
            print "  ",text

        
    def onHelp(self,*args):
        'Open help in a web browser'
        import urllib
        local_help = path('./doc/XliffEditor.htm').abspath()
        web_help = 'http://transolution.python-hosting.com/wiki/doc/XliffEditor?format=html'
        # open the local help file if it exists
        if local_help.isfile(): help = 'file:' + urllib.pathname2url(local_help)
        else: help = web_help
        import webbrowser
        webbrowser.open(help)
        return True

    def onAbout(self,*args):
        name = "Transolution XLIFF Editor"
        version = __version__
        license = 'GPL'
        homepage = 'http://transolution.python-hosting.com'
        developers ='Fredrik Corneliusson (fredrik.corneliusson at gmail.com)'
        artists = 'Tobias Einarsson'
        tmdevelopers = 'Fredrik Estreen'
        translators = _('translator-credits') # request translator-credits from gettext
        
        logo_file = os.path.join(transolution.xliffeditor.imagedir,'transolution_blank.png')
        try:
            # remove this to use the GTK 2.6 About dialog if avaible.
            # raise AttributeError()
            # d = gtk.AboutDialog("About",parent=self.window)
            d = gtk.AboutDialog()
        except AttributeError,e:
            # GTK < 2.6
            msg = '<big>%(name)s</big> \nVersion: <b>%(version)s</b>\n'
            msg += 'XLIFF Editor &#169; 2005 <b>%(developers)s</b>\n'
            msg += 'TM Engine &#169; 2005 <b>%(tmdevelopers)s</b>\n'
            msg += 'License <b>%(license)s</b>\n'
            msg += 'Logo by: <b>%(artists)s</b>\n'
            msg += 'Transolution homepage: <b>%(homepage)s</b>\n'
            image = gtk.Image()
            image.set_from_file(logo_file)
            aboutDialog(self.window,"About",msg%locals(),image)
##            messageDialog(self.window,msg)
        else:
            # GTK >= 2.6
            d.set_name(name)
            from license import license_long
            d.set_license(license_long)
            d.set_version(version)
            d.set_authors([developers])
            d.set_artists([artists])
            d.set_translator_credits(translators)
            d.set_logo(gtk.gdk.pixbuf_new_from_file(logo_file))

            def openHomePage(widget,url,url2):
                'callback for gtk.about_dialog_set_url_hook'
                import webbrowser
                webbrowser.open_new(url)

            gtk.about_dialog_set_url_hook(openHomePage,homepage)
            d.set_website(homepage)
            
            d.run()
            d.destroy()
        return True



def main():
    import sys,os

    mainw = MainWindow()
    try:
        xmls = sys.argv[1:]
    except IndexError,e:
        xmls = []

    for xml in xmls:        
        mainw.openFile(xml)

    gtk.main()

if __name__ == '__main__':
    main()
