import pygtk
#tell pyGTK, if possible, that we want GTKv2
##pygtk.require("2.0")

import gtk
import gtk.glade
from gtk_utils import askOk,messageDialog,CustomHandler
from general import EditorException
import transolution.xliffeditor

class NoteValueException(EditorException):pass

#   columns
(
  COLUMN_PRIORITY,
  COLUMN_ANNOTATES,
  COLUMN_FROM,
  COLUMN_VALUE,
  COLUMN_NOTEOBJ
) = range(5)

class NoteEdit(CustomHandler):
    'provies a gtk widget to view/edit a TU:s notes'
    def __init__(self, parent=None):
        CustomHandler.__init__(self)
        self.tu = None
        self.parent = parent
        # Load Glade GUI
        windowname="notes_panel"

        # Create the toplevel window
        self.wTree=gtk.glade.XML (transolution.xliffeditor.gladefile,windowname)
        self.notes_control = self.wTree.get_widget(windowname)

        w = self.wTree.get_widget
        # get the widgets we need access to
        self.note_list = w("note_list")

        # connect
        self.wTree.signal_autoconnect(self)
        self.notes_control.show_all()

#----------- Create note List (gtkListStore) ------------------
    def getNoteList(self,str1, str2, int1, int2):
        'custom widget provider'
        # create list store
        import gobject
        g = gobject
        liststore = gtk.ListStore(g.TYPE_INT,
                                  g.TYPE_STRING,
                                  g.TYPE_STRING,
                                  g.TYPE_STRING,
                                  g.TYPE_PYOBJECT)
        self.model = liststore
        view = gtk.TreeView(liststore)
        self.__add_columns(view)
        view.set_enable_search(True)
        return view

    def fillNoteList(self,model):
        model.clear()
        for note in self.tu.notes:
            # add note
            if note.priority is None: priority = 1
            else: priority = int(note.priority)
            if note._from is None: _from = ''
            else: _from = note._from
            if note.annotates is None: annotates = 'general'
            else: annotates = note.annotates
            if note.getValue() is None: value = ''
            else: value = note.getValue()
            model.append([priority,annotates,_from,value,note])

    def __add_columns(self, treeview):
        model = treeview.get_model()

        # Add Colums
        for name,COL in (('Priority',COLUMN_PRIORITY),('Annotates',COLUMN_ANNOTATES),('From',COLUMN_FROM),('Note Value',COLUMN_VALUE)):
            renderer = gtk.CellRendererText()
            renderer.connect("edited", self.onCellEdited, model)
            renderer.set_data("column", COL)
            renderer.set_property('editable', True)
            column = gtk.TreeViewColumn(name, renderer,text=COL) # why does edit=0 work?
            treeview.append_column(column)
            column.set_sort_column_id(COL)

    def alertException(self,e,msg):
        messageDialog(None,"%s:%s"%(msg,e),gtk.MESSAGE_ERROR)
        print e,msg

#-----------------------------

    def setTu(self,tu):
        self.tu = tu
        self.fillNoteList(self.model)

        
    def onAddNote(self,*args):
        'Add a new note'
        self.tu.addNote()
        self.fillNoteList(self.model)
        return True

    def onRemoveNote(self,*args):
        'Remove a note'
        #TODO: implement delete note
        selection = self.note_list.get_selection()
        model, iter = selection.get_selected()
        if iter:
            note = model.get_value(iter, COLUMN_NOTEOBJ)
            note.delete()

            # update tu and note list
            self.tu._parse()
            self.fillNoteList(self.model)
        self.tu.notifyObservers()
        return True


    def onCellEdited(self, cell, path_string, new_text, model):
        iter = model.get_iter_from_string(path_string)
        # path is the same as row index before sort
        path = model.get_path(iter)[0]
        column = cell.get_data("column")
        note = model.get_value(iter, COLUMN_NOTEOBJ)
        try:

            if column == COLUMN_PRIORITY:
                print "Priority Edited!"
                if int(new_text) not in range(1,11):
                    raise NoteValueException('Priority must be between 1 and 10 (default 1)')
                note.priority = str(new_text)
                model.set(iter, column, int(note.priority))
            elif column == COLUMN_FROM:
                print "From Edited!"
                note._from = new_text
                model.set(iter, column, note._from)
            elif column == COLUMN_ANNOTATES:
                print "annotates Edited!"
                if new_text not in ('general','source','target'):
                    raise NoteValueException("annotates value must be 'general','source' or 'target'(default 'general')")
                note.annotates = new_text
                model.set(iter, column, note.annotates)
            elif column == COLUMN_VALUE:
                print "Value Edited!"
                note.setValue(new_text)
                model.set(iter, column, note.getValue())
        except NoteValueException,e:
            self.alertException(e,'Error setting note value')


