import gtk
import gtk.glade
from gtk_utils import askOk,messageDialog

from general import EditorException
from gtk_documentviewpanel import CloseCancelTUSegmentException
import transolution

class SearchReplaceDialog(object):
    def __init__(self, parent=None):
        self.parent = parent
        # Load Glade GUI
        windowname="ReplaceDialog"

        # Create the toplevel window
        self.wTree=gtk.glade.XML (transolution.xliffeditor.gladefile,windowname)
        self.window = self.wTree.get_widget("ReplaceDialog")
        # Stay on top of parent
        self.window.set_transient_for(self.parent)

        w = self.wTree.get_widget
        # get the widgets we need access to
        self.cb_source = w("cb_source")
        self.cb_target = w("cb_target")
        self.cb_case = w("cb_case")
        self.cb_reverse = w("cb_reverse")
        self.search_entry = w("search_entry")
        self.replace_entry = w("replace_entry")

        self.bt_replace = w("bt_replace")
        self.bt_replaceall = w("bt_replaceall")
        self.bt_search = w("bt_search")

        # connect
        self.wTree.signal_autoconnect(self)

        self.listner = None
        self.search_settings = {}
        self._update()

    def enableReplace(self,enable):
        'setup GUI for search or replace'
        print "Replace",enable
##        self.replace_entry.set_sensitive(enable)
        widgets = [ #self.label_replace,
                    self.bt_replace,
                    self.bt_replaceall,
                    self.replace_entry
                    ]
        # show/hide replace operations
        for w in widgets:
            w.set_sensitive(enable)
        self.cb_source.set_sensitive(not enable)
        # disable source search & replace
        if enable:
            self.cb_source.set_active(False)
        
    def setSearchListner(self,listner):
        self.search_listner = listner

    def setReplaceListner(self,listner):
        self.replace_listner = listner

    def focusSearchText(self):
        self.text_ctrl_search_phrase.grab_focus()
        
    def _update(self):
        'update search settings dict'
        d = {}
        d['case'] = self.cb_case.get_active()
        d['reverse'] = self.cb_reverse.get_active()
        d['search_source'] = self.cb_source.get_active()
        d['search_target'] = self.cb_target.get_active()
        d['search_phrase'] = self.search_entry.get_text()
        d['replace_phrase'] = self.replace_entry.get_text()
        self.search_settings = d

    def alertException(self,e,msg):
        messageDialog(self.window,"%s:%s"%(msg,e),gtk.MESSAGE_ERROR)
        print e,msg
        
    def onSearch(self,e):
        print "Search"
        self._update()
        try:
            self.search_listner(self.search_settings,self.window)
        except Exception,ex:
            self.alertException(ex,'Search Error')
##        self.text_ctrl_search_phrase.addToHistory(self.search_settings['search_phrase'])

    def onReplace(self,e):
        print "Replace"
        self._update()
        self.search_settings['replace_all'] = False
        try:
            self.replace_listner(self.search_settings,self.window)
        except Exception,e:
            self.alertException(e,'Replace Error')

##        self.text_ctrl_search_phrase.addToHistory(self.search_settings['search_phrase'])
##        self.text_ctrl_replace_phrase.addToHistory(self.search_settings['replace_phrase'])
            

    def onReplaceAll(self,e):
        print "Replace All"
        if askOk(self.window,'Replace All',"This operation can not be reverted.\nAre you sure?"):
            self._update()
            self.search_settings['replace_all'] = True
            try:
                self.replace_listner(self.search_settings,self.window)
            except Exception,e:
                self.alertException(e,'Replace Error')

##            self.text_ctrl_search_phrase.addToHistory(self.search_settings['search_phrase'])
##            self.text_ctrl_replace_phrase.addToHistory(self.search_settings['replace_phrase'])

    def hide(self):
        self.window.hide()
    def show(self):
        self.window.show_all()

        
    def onCancel(self,*args):
        print "Cancel"
        self.parent.present()
        self.hide()
        return True

    
if __name__ == '__main__':
    def printlistner(settings,dialog):
        print settings

    search_dlg = SearchReplaceDialog()
    search_dlg.setSearchListner(printlistner)
    gtk.main()
