import gtk
import gtk.glade
from gtk_utils import askOk,messageDialog,CustomHandler

from general import EditorException
from gtk_documentviewpanel import CloseCancelTUSegmentException
import transolution

class StatusChangeDialog(CustomHandler):
    def __init__(self, parent=None):
        CustomHandler.__init__(self)
        self.parent = parent
        # Load Glade GUI
        windowname="ChangeTuStatus"

        # Create the toplevel window
        self.wTree=gtk.glade.XML(transolution.xliffeditor.gladefile,windowname)
        self.window = self.wTree.get_widget(windowname)
        # Stay on top of parent
        self.window.set_transient_for(self.parent)

        w = self.wTree.get_widget
        # get the widgets we need access to
        self.from_state = w("from_state")
        self.to_state = w("to_state")
        self.from_state_q = w("from_state_q")
        self.to_state_q = w("to_state_q")
        self.from_approved = w("from_approved")
        self.to_approved = w("to_approved")

        self.use_from_state = w("use_from_state")
        self.use_to_state = w("use_to_state")
        self.use_from_state_q = w("use_from_state_q")
        self.use_to_state_q = w("use_to_state_q")

        self.use_from_approved = w("use_from_approved")
        self.use_to_approved = w("use_to_approved")
        import document
        # setup combobox state TU constraint
        for v in document.tu.state_maps.getTuStates():
            self.from_state.append_text(v)
            self.to_state.append_text(v)
        for v in document.tu.state_maps.getTuStatesQualifiers():
            self.from_state_q.append_text(v)
            self.to_state_q.append_text(v)

        # connect
        self.wTree.signal_autoconnect(self)

        self.listner = None
        self.change_settings = {}
        self._update()

#------- custom widget creator functions ----------------------
    def getComboBox(self,str1, str2, int1, int2):
        'custom widget provider'
        comboboxentry = gtk.combo_box_entry_new_text()
        return comboboxentry
#-----------------------------

    def onButtonToggled(self,*args):
        self.guiUpdate()

    def guiUpdate(self):
        s = self
        conditions = {
            s.from_state:   s.use_from_state,
            s.to_state:     s.use_to_state,
            s.from_state_q: s.use_from_state_q,
            s.to_state_q:   s.use_to_state_q,
            s.from_approved:s.use_from_approved,
            s.to_approved:  s.use_to_approved
         }
        for w,c in conditions.items():
            w.set_sensitive(c.get_active())
        
        
    def setChangeListner(self,listner):
        self.change_listner = listner

    def _update(self):
        self.guiUpdate()
        'update search settings dict'
        d = {'from_state':None,'to_state':None,'from_state_q':None,'to_state_q':None,'to_approved':None,'from_approved':None}
        if self.use_from_state.get_active():
            d['from_state'] = self.from_state.child.get_text()
        if self.use_to_state.get_active():
            d['to_state'] = self.to_state.child.get_text()
        if self.use_from_state_q.get_active():
            d['from_state_q'] = self.from_state_q.child.get_text()
        if self.use_to_state_q.get_active():
            d['to_state_q'] = self.to_state_q.child.get_text()
        if self.use_to_approved.get_active():
            d['to_approved'] = self.to_approved.get_active()
        if self.use_from_approved.get_active():
            d['from_approved'] = self.from_approved.get_active()
        self.change_settings = d

    def alertException(self,e,msg):
        messageDialog(self.window,"%s:%s"%(msg,e),gtk.MESSAGE_ERROR)
        print e,msg
        
    def onChange(self,e):
        if askOk(self.window,'Batch change status',"This operation can not be reverted.\nAre you sure?"):
            self._update()
            changes = self.change_listner(self.change_settings,self.window)
            messageDialog(self.window,'Translation units changed:%d'%(changes,))
        

    def hide(self):
        self.window.hide()
    def show(self):
        self.window.show_all()

        
    def onCancel(self,*args):
        self.parent.present()
        self.hide()
        return True

    
if __name__ == '__main__':
    def printlistner(settings,dialog):
        print settings

    search_dlg = StatusChangeDialog()
    search_dlg.setChangeListner(printlistner)
    gtk.main()
