import gtk

from transolution.observer import Observer
from xliffsegment import XliffError
from gtk_basetextbuf import BaseTextBuffer
from gtk_clipboard import clipboard
from types import ListType,TupleType

class TagBuffer(BaseTextBuffer,Observer):
    'make events work with seg model'
    def __init__(self,*args,**kwargs):
        BaseTextBuffer.__init__(self,*args,**kwargs)
        Observer.__init__(self)

        self.connect("insert-text", self.insert_text)
        self.connect("delete-range", self.delete_range)
        self.connect("mark-set", self.mark_set)

        self.old_pos = 0 # keeps track of last cursor pos
        self.interactive = True # flag tells if text changes was interctive
        self.model = None
        
    def mark_set(self,textbuffer,iter,textmark):
        'handel cursor and selection changes so it treat tags as atoms'
        if not self.interactive: return
        if self.model is None:return
        org = self.interactive
        self.interactive = False
        
        old_pos = self.old_pos
        pos = self.getInsertionPoint()
        if pos != old_pos: 
            try:
                node,subpos = self.model.getNodeByTextIndex(pos)
            except IndexError,e:
                node,subpos = self.model.getLast()
            except XliffError,e:
                print "Error",e
                self.interactive = org
                return
                
            if self.model.isTag(node) and subpos:
                # Jump to start or end of locked node
                tagstart,tagend = self.model.getTextSpanOfNode(node)
                if old_pos > pos: pos = tagstart
                else:    pos = tagend

                if self.get_selection_bounds():
                    # there is a selection so only move the textmark
                    self.move_mark(textmark,self.get_iter_at_offset(pos))
                else:
                    # there was no selection so dont create one.
                    self.place_cursor(self.get_iter_at_offset(pos))

            self.old_pos = pos

        self.interactive = org

    def setModel(self,model):
        self.model = model
        # tell us to observer model for gui changes
        self.subscribeToObservable(self.model, "xupdate")

    def delete_range(self,textbuf,start,end,interactive=True):
        if not self.interactive: return

        s,e = start.get_offset(),end.get_offset()
        print "buf del!"
        try:
            self.model.deleteTextWithUndo(s,e,interactive=interactive)
        except (XliffError),e:
            print "Error:",e
            # Stop event from continue
            self.emit_stop_by_name("delete-range")
            return True
        return False

    def insert_text(self,textbuf, position, new_text, text_length,interactive=True):
##        print "Insert Text"
        if not self.interactive: return
        #insert-text (Gtk2::TextBuffer, Gtk2::TextIter, string, integer)
        i = self.getInsertionPoint()
        try:
            self.model.insertTextWithUndo(i,unicode(new_text),interactive=interactive)
        except (XliffError,ValueError),e:
            # Stop event from continue
            self.emit_stop_by_name("insert-text")
            return True
        return False
##            self.emit_stop_by_name("insert-text")


##        else:
##            # update observers (views)
##            self.model.notifyObservers()
##            # move insertion point to reflect the inserted char
##            tw.setInsertionPoint(tw.getInsertionPoint()+1)


    def xupdate(self, seg, event,msg):
        'Update TextCtrl with formatted segment data'
        print "update"
        org = self.interactive
        self.interactive = False # disable interactive event checks
        i = self.getInsertionPoint()
        self.clear()
        iter = self.get_iter_at_offset(0)
        if seg.getRootNode():
            for item in seg.getFormatted():
                for s,t in item:
                    self.insert_with_tags_by_name(iter, t, *s)

        self.setInsertionPoint(i)

        self.interactive = org # restore orginal interactive state


class TagEdit(gtk.TextView):
    def __init__(self,*args,**kwargs):
        gtk.TextView.__init__(self,*args,**kwargs)
        # Disable drop events
        self.drag_dest_unset()

        self.connect("copy-clipboard", self.copy_clipboard)
        self.connect("cut-clipboard", self.cut_clipboard)
        self.connect("paste-clipboard", self.paste_clipboard)

    def Enable(self,enabled=True):
        self.set_sensitive(enabled)

    def getModel(self):
        return self.get_buffer().model

    def undo(self,*args):
        print "undo"
        model = self.getModel()
        model.undo()
        model.notifyObservers()

    def redo(self,*args):
        print "redo"
        model = self.getModel()
        model.redo()
        model.notifyObservers()

    def cut_clipboard(self,*args):
        print "Cut"
        buf = self.get_buffer()
        start,end = buf.getSelection()
        text = buf.getText(start, end)
        nodelist =  buf.model.cutNodesByTextIndexWithUndo(start,end)
        clipboard.setClipBoardNodeList(nodelist,text)
        buf.model.notifyObservers()
        buf.setInsertionPoint(start)
        return True

    def copy_clipboard(self,*args):
        print "Copy"
        buf = self.get_buffer()
        start,end = buf.getSelection()
        text = buf.getText(start,end)
        nodelist =  buf.model.copyNodesByTextIndex(start,end)
        # TODO: implement gtk clipboard
        clipboard.setClipBoardNodeList(nodelist,text)
        return True

    def paste_clipboard(self,*args):
        print "Paste"
        buf = self.get_buffer()
        # Get from system clipboard
        text,nodelist = clipboard.getClipBoardNodeList(buf.model.getDocument())
        if nodelist:
            start,end = buf.getSelection()
            try:buf.model.deleteTextWithUndo(start,end,interactive=False)
            except XliffError,e:
                print "Error:",e
                # Stop event from continue
                self.emit_stop_by_name("paste-clipboard")
                return True

            buf.model.pasteNodesAtTextIndexWithUndo(start,nodelist)
            buf.model.notifyObservers()
            buf.setInsertionPoint(start+len(text))
        else: print "Nothing to paste"
        # stop signal from propegating to textbuf
        self.emit_stop_by_name("paste-clipboard")
        return True
