import pygtk
#tell pyGTK, if possible, that we want GTKv2
##pygtk.require("2.0")

import gtk
import gtk.glade
from gtk_utils import askOk,messageDialog,create_textbuffer_tags,CustomHandler
from transolution.observer import Observer,Observable
from tm.utils import getTextWithTags
from document.tu.state_maps import getTuStates,getTuStatesQualifiers
import transolution
from document.tu import TU_CHANGE_EVENT,TU_UI_UPDATE_EVENT

# get the relevant states
tu_states = getTuStates()
state_qualifiers = getTuStatesQualifiers()

##state_variant = 'suntrans'

class TuStatusPanel(gtk.VBox,Observer,CustomHandler):
    def __init__(self, parent=None):
        gtk.VBox.__init__(self)
        Observer.__init__(self)
        CustomHandler.__init__(self)
        self.tu = None
      
        # Load Glade GUI
        rootname="tu_info_panel"

        self.interactive = True # flag tells if text changes was interctive

        # Create the toplevel window
        self.wTree=gtk.glade.XML (transolution.xliffeditor.gladefile,rootname)
        self.tu_info_panel = self.wTree.get_widget(rootname)
        # add tu_info_panel
        self.add(self.tu_info_panel)

        w = self.wTree.get_widget
        # get the widgets we need access to
        self.tu_id = w("tu_id")
        self.tu_info = w("tu_info")
        self.tu_state = w("tu_state")
        self.tu_state_qualifiers = w("tu_state_qualifiers")
        self.cb_approved = w("cb_approved")
        self.cb_translate = w("cb_translate")
        self.context_expander = w("context_expander")
        self.context_control = w("context_control")

        self.alttrans_expander = w("alttrans_expander")
        self.alttrans_control = w("alttrans_control")

        self.notes_expander = w("notes_expander")
        self.notes_control = w("notes_control")

        self.prop_expander = w("prop_expander")
        self.prop_control = w("prop_control")

        # add states to tu states combobox
        for v in tu_states:
            self.tu_state.append_text(v)
        for v in state_qualifiers:
            self.tu_state_qualifiers.append_text(v)

        # Flag indecates if tu_control child widgets have changed visability
        self.ui_changed = False

        # Add text styles to context view buffer and lock text
        for textw in self.context_control,self.alttrans_control,self.prop_control:
            create_textbuffer_tags(textw.get_buffer())
            textw.set_editable(False)
        # connect
        self.wTree.signal_autoconnect(self)
        self.show_all()

#------- custom widget creator functions ----------------------
    def getTuStateCombo(self,str1, str2, int1, int2):
        'custom widget provider'
        comboboxentry = gtk.combo_box_entry_new_text()
        comboboxentry.child.connect('changed', self.onTuStateChange)
        return comboboxentry

    def getTuStateQualifiersCombo(self,str1, str2, int1, int2):
        'custom widget provider'
        comboboxentry = gtk.combo_box_entry_new_text()
        comboboxentry.child.connect('changed', self.onTuStateQualifiersChange)
        return comboboxentry

    def getNotesPanel(self,str1, str2, int1, int2):
        'custom widget provider'
        from gtk_noteedit import NoteEdit
        self.note_edit = NoteEdit()
        notes_panel = self.note_edit.notes_control#,parent=self.main_app.window)
        return notes_panel
#-----------------------------
        
    def initGui(self):
        self.context_expander.hide_all()
        self.alttrans_expander.hide_all()
        self.notes_expander.hide_all()
        self.prop_expander.hide_all()
        self.setTu(None)
        
    def cbTuChanged(self,tu,event,msg):
        'TU Observer callback'
        assert self.tu is tu,'Recived notification from wrong TU'
        self.update()

    def update(self):
        'update the windgets with the tu status'
        tu = self.tu
        org = self.interactive
        self.interactive = False # disable interactive event checks

        self.set_sensitive(True)
        #Show the status of tu
        self.tu_id.set_text(tu.id)
        self.tu_info.set_text(tu.getStatus())
        if tu.state is None:
            self.tu_state.child.set_text('')
        else:
            self.tu_state.child.set_text('%s'%(tu.state,))
        self.cb_approved.set_active(tu.approved=='yes')
        self.cb_translate.set_active(tu.translate=='yes')

        try:
            val = tu.state_qualifier
        except AttributeError,e:
            val = None

        if val is not None:
            self.tu_state_qualifiers.child.set_text('%s'%(val,))
        else:
            self.tu_state_qualifiers.child.set_text('')


        self.note_edit.setTu(tu)

        # show context groups
        self.showContextGroups(tu)
        # show alt-trans
        self.showAltTrans(tu)
        # Show prop groups
        self.showPropGroups(tu)
        # show notes
        self.showNotes(tu)

        self.interactive = org # restore orginal interactive state

    def setTu(self,tu):
        'update TU status info panel'
        if self.tu:
            self.unsubscribeToObservable(self.tu)

        # disable us if tu is none
        if tu is None:
            self.set_sensitive(False)
            return
        if tu.target_node is None:
            self.tu_state.set_sensitive(False)
            self.tu_state_qualifiers.set_sensitive(False)
        else:
            self.tu_state_qualifiers.set_sensitive(True)
            self.tu_state.set_sensitive(True)

        self.subscribeToObservable(tu, "cbTuChanged")
        self.tu = tu
        self.update()

    def onAlttransExpanderActivate(self,*args):
        self.update()
        return True
        
    def onContextExpanderActivate(self,*args):
        self.update()
        return True

    def onCbTranslateToggled(self,*args):
        if not self.interactive: return
        print args
        return True

    def onCbApprovedToggled(self,widget):
        if not self.interactive: return
        if self.tu:
            self.tu.approved = widget.get_active()
            self.tu.notifyObservers(TU_CHANGE_EVENT)
            self.tu.notifyObservers(TU_UI_UPDATE_EVENT)

        return True

    def onTuInfoChanged(self,*args):
        if not self.interactive: return
        print args

    def onTuStateChange(self,entry):
        if not self.interactive: return
        state = entry.get_text()
        if state:
            self.tu.state = state
        else:
            del self.tu.state
        self.tu.notifyObservers(TU_CHANGE_EVENT)
        self.tu.notifyObservers(TU_UI_UPDATE_EVENT)

        return True

    def onTuStateQualifiersChange(self,entry):
        if not self.interactive: return
        if entry.get_text():
            self.tu.state_qualifier = entry.get_text()
        else: del self.tu.state_qualifier
        self.tu.notifyObservers(TU_CHANGE_EVENT)
        self.tu.notifyObservers(TU_UI_UPDATE_EVENT)

        return True
        
    def alertException(self,e,msg):
        messageDialog(self.window,"Error","%s:%s"%(msg,e))
        print e,msg

    def showContextGroups(self,tu):
        '''
        show the context groups visually in context_widget.
        '''
        text_buffer = self.context_control.get_buffer()
        context_groups = list(tu.getContextGroups())
        if not context_groups:
            self.ui_changed = self.context_expander.get_property('visible')
            if self.ui_changed:
                self.context_expander.hide_all()
            return
        else:
            self.ui_changed = not self.context_expander.get_property('visible')
            if self.ui_changed:
                # Show the widget
                self.context_expander.show_all()

        # clear text
        text_buffer.delete(*text_buffer.get_bounds())
        iter = text_buffer.get_iter_at_offset(0)
        indent = "       "
        for i,context_group in enumerate(context_groups):
            text_buffer.insert_with_tags_by_name(iter, unicode(context_group)+'\n', 'fg_darkgray')
            for context in context_group.getContextNodes():
                text_buffer.insert_with_tags_by_name(iter, indent + "Type: " + unicode(context.context_type)+'\n', 'fg_darkgray')
                text_buffer.insert_with_tags_by_name(iter, indent*2 + context.getValue()+'\n', 'fg_black')

    def showPropGroups(self,tu):
        '''
        show the prop groups visually in context_widget.
        '''
        text_buffer = self.prop_control.get_buffer()
        expander = self.prop_expander
        prop_groups = list(tu.getPropGroups())
        if not prop_groups:
            self.ui_changed = expander.get_property('visible')
            if self.ui_changed:
                expander.hide_all()
            return
        else:
            self.ui_changed = not expander.get_property('visible')
            if self.ui_changed:
                # Show the widget
                expander.show_all()

        # clear text
        text_buffer.delete(*text_buffer.get_bounds())
        iter = text_buffer.get_iter_at_offset(0)
        indent = "       "
        for i,group in enumerate(prop_groups):
            text_buffer.insert_with_tags_by_name(iter, unicode(group)+'\n', 'fg_darkgray')
            for prop in group.getPropNodes():
                text_buffer.insert_with_tags_by_name(iter, indent + "Type: " + unicode(prop.prop_type)+'\n', 'fg_darkgray')
                text_buffer.insert_with_tags_by_name(iter, indent*2 + prop.getValue()+'\n', 'fg_black')
                
    def showNotes(self,tu):
        '''
        show the notes visually in note_widget.
        '''
        notes = tu.notes
        if not notes:
            self.ui_changed = self.notes_expander.get_property('visible')
            if self.ui_changed:
                self.notes_expander.hide_all()
            return
        else:
            self.ui_changed = not self.notes_expander.get_property('visible')
            if self.ui_changed:
                # Show the widget
                self.notes_expander.show_all()

    def showAltTrans(self,tu):
        '''
        show the alt-trans visually.
        '''
        text_buffer = self.alttrans_control.get_buffer()
        alttransilations = list(tu.getAltTrans())
        if not alttransilations:
            self.ui_changed = self.alttrans_expander.get_property('visible')
            if self.ui_changed:
                self.alttrans_expander.hide_all()
            return
        else:
            self.ui_changed = not self.alttrans_expander.get_property('visible')
            if self.ui_changed:
                # Show the widget
                self.alttrans_expander.show_all()

        # clear text
        text_buffer.delete(*text_buffer.get_bounds())
        iter = text_buffer.get_iter_at_offset(0)
        for i,alttrans in enumerate(alttransilations):
            text_buffer.insert_with_tags_by_name(iter, '%d '%(i+1), 'fg_blue','bold')
            text_buffer.insert_with_tags_by_name(iter, 'Match:%s\n'%(alttrans.match_quality), 'fg_darkgray')

            # show source if it's present
            if alttrans.source_node:
                text_buffer.insert_with_tags_by_name(iter, getTextWithTags(alttrans.source_node.childNodes)+'\n', 'fg_black')

            # Show the targets
            # TODO: make it possible to choose alt-trans subtarget
            for target_index,target in enumerate(alttrans.target_nodes):
                # show sub target index
                text_buffer.insert_with_tags_by_name(iter, '%i.%i'%(i,target_index) +'\n', 'fg_gray')
                # show target
                text_buffer.insert_with_tags_by_name(iter, getTextWithTags(target.childNodes)+'\n', 'fg_black')

            # show context groups
            for context_group in alttrans.getContextGroups():
                for context in context_group.getContextNodes():
                    text_buffer.insert_with_tags_by_name(iter, "     " + unicode(context.context_type) + '=', 'fg_darkgray')
                    text_buffer.insert_with_tags_by_name(iter, context.getValue()+'\n', 'fg_black')

            
        

    
if __name__ == '__main__':
    tu_status_panel = TuStatusPanel()
    mainwin = gtk.Window()
    mainwin.add(tu_status_panel)
    mainwin.show_all()
    gtk.main()
