import gtk
import pango

class CustomHandler(object):
    'adds automatic custom widget dispatch'
    def __init__(self):
        # Create the set custom widget handler
        gtk.glade.set_custom_handler(self.custom_handler)

    def custom_handler(self,glade, function_name, widget_name,
                        str1, str2, int1, int2):
        'custom widget proxy'
        if hasattr(self, function_name):
            handler = getattr(self, function_name)
            return handler(str1, str2, int1, int2)

def createTag(name,**kwargs):
    'convenience method to create a TextTag and setting properties.'
    tag = gtk.TextTag(name=name)
    for key,val in kwargs.items():
        tag.set_property(key,val)
    return tag

def create_tag_table():
    'create a TextTagTable with common tags'
    table = gtk.TextTagTable()
    a = table.add
    
    a(createTag("strikethrough", strikethrough=True))
    a(createTag("underline",underline=pango.UNDERLINE_SINGLE))
    a(createTag("superscript",rise=5 * pango.SCALE, size=8 * pango.SCALE)) #  8 points
    a(createTag("fg_black", foreground="black"))
    a(createTag("fg_gray", foreground="gray"))
    a(createTag("fg_darkgray", foreground="darkgray"))
    a(createTag("fg_red", foreground="red"))
    a(createTag("fg_blue", foreground="blue"))
    a(createTag("fg_green", foreground="green"))
    a(createTag("fg_darkcyan", foreground="darkcyan"))
    a(createTag("fg_purple", foreground="purple"))

    a(createTag("bg_white", background="white"))
    a(createTag("bg_cyan", background="cyan"))
    a(createTag("bg_green", background="green"))
    a(createTag("bg_blue", background="blue"))
    a(createTag("bg_lightgray", background="lightgray"))
    a(createTag("bg_lightgreen", background="lightgreen"))
    a(createTag("bg_hotpink", background="hotpink"))
    a(createTag("bg_yellow", background="yellow"))
    a(createTag("bg_lightyellow", background="lightyellow"))
    a(createTag("bg_mistyrose", background="mistyrose"))
    a(createTag("bg_lightblue", background="lightblue"))
    a(createTag("bg_khaki", background="khaki"))
    # selected needs to be added after other tags to get higer priority
    a(createTag("bg_selected", background="yellow"))

    a(createTag("bold", weight=pango.WEIGHT_BOLD))

    a(createTag("italic", style=pango.STYLE_ITALIC))

    a(createTag("big_gap_before_line", pixels_above_lines=30))

    a(createTag("big_gap_after_line", pixels_below_lines=30))
    a(createTag("wide_margins",
                left_margin=10, right_margin=10))

    return table

def create_textbuffer_tags(text_buffer):
    'setup styles for document edit widget'
    text_buffer.create_tag("strikethrough", strikethrough=True)
    text_buffer.create_tag("underline",underline=pango.UNDERLINE_SINGLE)
    text_buffer.create_tag("superscript",rise=5 * pango.SCALE, size=8 * pango.SCALE) #  8 points
    text_buffer.create_tag("fg_black", foreground="black")
    text_buffer.create_tag("fg_gray", foreground="gray")
    text_buffer.create_tag("fg_darkgray", foreground="darkgray")
    text_buffer.create_tag("fg_red", foreground="red")
    text_buffer.create_tag("fg_blue", foreground="blue")
    text_buffer.create_tag("fg_green", foreground="green")
    text_buffer.create_tag("fg_darkcyan", foreground="darkcyan")
    text_buffer.create_tag("fg_purple", foreground="purple")

    text_buffer.create_tag("bg_white", background="white")
    text_buffer.create_tag("bg_cyan", background="cyan")
    text_buffer.create_tag("bg_green", background="green")
    text_buffer.create_tag("bg_blue", background="blue")
    text_buffer.create_tag("bg_lightgray", background="lightgray")
    text_buffer.create_tag("bg_lightgreen", background="lightgreen")
    text_buffer.create_tag("bg_hotpink", background="hotpink")
    text_buffer.create_tag("bg_yellow", background="yellow")
    text_buffer.create_tag("bg_lightyellow", background="lightyellow")
    text_buffer.create_tag("bg_mistyrose", background="mistyrose")
    text_buffer.create_tag("bg_lightblue", background="lightblue")
    text_buffer.create_tag("bg_khaki", background="khaki")
    # selected needs to be added after other tags to get higer priority
    text_buffer.create_tag("bg_selected", background="yellow")
##    text_buffer.create_tag("bg_selected", underline=pango.UNDERLINE_SINGLE,left_margin=10, right_margin=10)
##    text_buffer.create_tag("bg_selected", weight=pango.WEIGHT_BOLD)

    text_buffer.create_tag("bold", weight=pango.WEIGHT_BOLD)

    text_buffer.create_tag("italic", style=pango.STYLE_ITALIC)

    text_buffer.create_tag("big_gap_before_line", pixels_above_lines=30)

    text_buffer.create_tag("big_gap_after_line", pixels_below_lines=30)
    text_buffer.create_tag("wide_margins",
                left_margin=10, right_margin=10)

def fileChooserDialog(title='Open...', filefilters= ( ("All files",('*',) )),folder=None ):
    dialog = gtk.FileChooserDialog(title,
                                   None,
                                   gtk.FILE_CHOOSER_ACTION_OPEN,
                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    dialog.set_default_response(gtk.RESPONSE_OK)

    # Add location popup info
    dialog.set_extra_widget(gtk.Label(_("To enter location manually press <CTRL>+L")))

    if folder:
        dialog.set_current_folder(folder)
    dialog.set_select_multiple(True)

    for ff in filefilters:
        filter_name,patterns = ff
        filter = gtk.FileFilter()
        filter.set_name(filter_name)
        for pattern in patterns:
            filter.add_pattern(pattern)
        dialog.add_filter(filter)

    response = dialog.run()
    fnames = None
    if response == gtk.RESPONSE_OK:
        fnames = dialog.get_filenames()
    elif response == gtk.RESPONSE_CANCEL:
        print 'Closed, no files selected'
    dialog.destroy()
    return fnames


def askOk(parent,title,msg):
    'Create a OK/Cancle dialog and return True if ok was pressed'
    dialog = gtk.Dialog(title, parent, gtk.DIALOG_MODAL,
            (gtk.STOCK_OK, gtk.RESPONSE_OK,
             gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

    hbox = gtk.HBox(False, 8)
    hbox.set_border_width(8)
    dialog.vbox.pack_start(hbox, False, False, 0)

    stock = gtk.image_new_from_stock(
            gtk.STOCK_DIALOG_QUESTION,
            gtk.ICON_SIZE_DIALOG)
    hbox.pack_start(stock, False, False, 0)

    label = gtk.Label(msg)
    hbox.add(label)
    dialog.show_all()

    response = dialog.run()

    dialog.destroy()
    return response == gtk.RESPONSE_OK

def askDialog(parent,title,msg,buttons):
    'Create a dialog witht the @buttons button id of the pressed button'
    dialog = gtk.Dialog(title, parent, gtk.DIALOG_MODAL)

    for text,id in buttons:
        dialog.add_button(text,id)
    hbox = gtk.HBox(False, 8)
    hbox.set_border_width(8)
    dialog.vbox.pack_start(hbox, False, False, 0)

    stock = gtk.image_new_from_stock(
            gtk.STOCK_DIALOG_QUESTION,
            gtk.ICON_SIZE_DIALOG)
    hbox.pack_start(stock, False, False, 0)

    label = gtk.Label(msg)
    hbox.add(label)
    dialog.show_all()

    response = dialog.run()

    dialog.destroy()
    return response

def entryDialog(parent,title,msg,lable,default_val=None,default_response=gtk.RESPONSE_OK,drop_down_values = None):
    '''
    Create a input dialog.
    @drop_down_values = list of text values to fill a combobox dropdown with.
    @Returns = string entered or None if Cancel was pressed.
    '''

    dialog = gtk.Dialog(title, parent, 0,
            (gtk.STOCK_OK, gtk.RESPONSE_OK,
             gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

    dialog.set_default_response(default_response)

    hbox = gtk.HBox(False, 8)
    hbox.set_border_width(8)
    dialog.vbox.pack_start(hbox, False, False, 0)

    stock = gtk.image_new_from_stock(
            gtk.STOCK_DIALOG_QUESTION,
            gtk.ICON_SIZE_DIALOG)
    hbox.pack_start(stock, False, False, 0)
    vbox = gtk.VBox(False, 8)
    hbox.pack_start(vbox, True, True, 0)
    vbox.pack_start(gtk.Label(msg), True, True, 0)


    table = gtk.Table(2, 1)
    table.set_row_spacings(4)
    table.set_col_spacings(4)
    vbox.pack_start(table, True, True, 0)

    label = gtk.Label(lable)
    label.set_use_underline(True)
    table.attach(label, 0, 1, 0, 1)

    if drop_down_values is None:
        # Create entry
        local_entry1 = gtk.Entry()
        table.attach(local_entry1, 1, 2, 0, 1)
    else:
        # Create a combobox entry
        combobox = gtk.combo_box_entry_new_text()
        for v in drop_down_values:
            combobox.append_text(v)
        local_entry1 = combobox.child
        table.attach(combobox, 1, 2, 0, 1)

    local_entry1.set_activates_default(True)
##    local_entry1 = gtk.Entry()
    label.set_mnemonic_widget(local_entry1)
    if default_val:
        local_entry1.set_text(default_val)

    
    dialog.show_all()

    response = dialog.run()

    if response == gtk.RESPONSE_OK:
        res = local_entry1.get_text()
    else:
        res = None
    dialog.destroy()

    return res

def dropDownDialog(parent,title,msg,lable,drop_down_values,default_val=None,default_response=gtk.RESPONSE_OK,):
    '''
    Create a dropdown dialog.
    @drop_down_values = list of text values to fill a combobox dropdown with.
    @Returns = string entered or None if Cancel was pressed.
    '''

    dialog = gtk.Dialog(title, parent, 0,
            (gtk.STOCK_OK, gtk.RESPONSE_OK,
             gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

    dialog.set_default_response(default_response)

    hbox = gtk.HBox(False, 8)
    hbox.set_border_width(8)
    dialog.vbox.pack_start(hbox, False, False, 0)

    stock = gtk.image_new_from_stock(
            gtk.STOCK_DIALOG_QUESTION,
            gtk.ICON_SIZE_DIALOG)
    hbox.pack_start(stock, False, False, 0)
    vbox = gtk.VBox(False, 8)
    hbox.pack_start(vbox, True, True, 0)
    vbox.pack_start(gtk.Label(msg), True, True, 0)


    table = gtk.Table(2, 1)
    table.set_row_spacings(4)
    table.set_col_spacings(4)
    vbox.pack_start(table, True, True, 0)

    label = gtk.Label(lable)
    label.set_use_underline(True)
    table.attach(label, 0, 1, 0, 1)

    # Create a combobox entry
    combobox = gtk.combo_box_new_text()
    for v in drop_down_values:
        combobox.append_text(v)
    table.attach(combobox, 1, 2, 0, 1)

    if default_val:
        for row in combobox.get_model():
            if row[0] == default_val:
                combobox.set_active(row.path[0])
                break
    
    dialog.show_all()

    response = dialog.run()

    if response == gtk.RESPONSE_OK:
        model = combobox.get_model()
        active = combobox.get_active()
        if active < 0:
            return None
        res = model[active][0]
    else:
        res = None
    dialog.destroy()

    return res

def messageDialog(parent,msg,dlg_type=gtk.MESSAGE_INFO):
    dialog = gtk.MessageDialog(parent,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            dlg_type, gtk.BUTTONS_OK,
            msg)
    dialog.run()
    dialog.destroy()

def aboutDialog(parent,title,msg,image):
    '''
    Show About Dialog
    @title: text
    @msg:   tagged text
    @image: gtk.Image object
    '''
    dialog = gtk.Dialog(title, parent, gtk.DIALOG_MODAL)

    dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)

    dialog.vbox.pack_start(image, False, False, 0)

    label = gtk.Label()
    label.set_markup(msg)
    label.set_selectable(True)
    dialog.vbox.pack_start(label, False, False, 0)

    dialog.show_all()

    response = dialog.run()

    dialog.destroy()
    return response


def cb_get_active_text(combobox):
    model = combobox.get_model()
    active = combobox.get_active()
    if active < 0:
        return None
    return model[active][0]