import os
import re
import sys
#---------------------------------------------------------
# The following is taken from Bittorent 4.1.1 __init__.py

# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.0 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# a cross-platform way to get user's home directory
def get_config_dir():
    shellvars = ['${APPDATA}', '${HOME}', '${USERPROFILE}']
    dir_root = get_dir_root(shellvars)
    if dir_root is None:
        reg_dir = get_registry_dir('AppData')
        if reg_dir is not None:
            dir_root = reg_dir
    return dir_root

def get_home_dir():
    shellvars = ['${HOME}', '${USERPROFILE}']
    dir_root = get_dir_root(shellvars)
    return dir_root

def get_dir_root(shellvars):
    def check_sysvars(x):
        y = os.path.expandvars(x)
        if y != x and os.path.isdir(y):
            return y
        return None

    dir_root = None
    for d in shellvars:
        dir_root = check_sysvars(d)
        if dir_root is not None:
            break
    else:
        dir_root = os.path.expanduser('~')
        if dir_root == '~' or not os.path.isdir(dir_root):
            dir_root = None
    return dir_root


def get_registry_dir(value):
    reg_dir = None 

    find_pat = re.compile('%([A-Z_]+)%')
    repl_pat = '${\\1}'
    
    if os.name == 'nt':
        #from win32com.shell import shell, shellcon
        #desktop = shell.SHGetPathFromIDList(shell.SHGetSpecialFolderLocation(0, shellcon.CSIDL_DESKTOPDIRECTORY))
        import _winreg as wreg
        try: 
            key = wreg.OpenKey(wreg.HKEY_CURRENT_USER,
                               r'Software\Microsoft\Windows\CurrentVersion\Explorer\User Shell Folders')
            d = wreg.QueryValueEx(key, value)
            reg_dir, a_random_number = os.path.expandvars(d)
            reg_dir = find_pat.sub(repl_pat, reg_dir)
            reg_dir = os.path.expandvars(reg_dir)
            reg_dir = reg_dir.encode('mbcs')
        except Exception, e:
            pass

        if reg_dir is not None and os.access(reg_dir, os.R_OK|os.W_OK):
            pass
        else:
            reg_dir = None
    return reg_dir


from transolution.path_mod import path

class ConfigError(Exception):pass
def getConfigFile():
    dir_root = get_config_dir()
    if dir_root is None and os.name == 'nt':
        tmp_dir_root = os.path.split(sys.executable)[0]
        if os.access(tmp_dir_root, os.R_OK|os.W_OK):
            dir_root = tmp_dir_root

    if dir_root is None:
        raise ConfigError('Cant get configuration root dir')

    configfile = os.path.join(dir_root, '.xliffeditor.xml')
    return path(configfile)

import sys
from transolution.EaseXML import *

webdict_list = {}
webdict_list['leo_en-de_url'] = 'http://dict.leo.org/?searchLoc=-1&searchLocRelinked=-1&lp=ende&lp=ende&lang=en&searchLoc=1&searchLocRelinked=1&search=%s'
webdict_list['leo_de-en_url'] = 'http://dict.leo.org/?searchLoc=1&searchLocRelinked=1&lp=ende&lp=ende&lang=en&searchLoc=-1&searchLocRelinked=1&search=%s'
webdict_list['google_en-de'] = 'http://translate.google.com/translate_t?text=%s&ie=UTF-8&oe=UTF-8&langpair=en%%7Cde&hl=en&ie=UTF-8&oe=UTF-8'
webdict_list['google_de-en'] = 'http://translate.google.com/translate_t?text=%s&ie=UTF-8&oe=UTF-8&langpair=de%%7Cen&hl=en&ie=UTF-8&oe=UTF-8'


class tm(XMLObject):
    _orderNodes = [ 'address', 'automatic_lookup' ]
##    _stripStrings = False
    address = TextNode(default='http://localhost:6315')
    automatic_lookup = TextNode(default="False")

class fonts(XMLObject):
    document_font= TextNode(default="10")
    editor_font= TextNode(default="10")

class site(XMLObject):
    name = StringAttribute()
    encoding = StringAttribute(default='utf-8')
    url = TextNode()
    
class webdict(XMLObject):
    current = StringAttribute(optional=True)
    sites = ListNode('site')

class config(XMLObject):
    _entities = [ ('&xml;','eXtensible Markup Language')]
    version = StringAttribute(optional=True)
    tm = ItemNode('tm')
    fonts = ItemNode('fonts')
    webdict = ItemNode('webdict')
    
class xliffeditor(config):
    def __init__(self,configfile=None):
        config.__init__(self)
        self._file = configfile

    def save(self):
        self._file.write_text(self.toXml())
        
configfile = getConfigFile()
settings = None
# If there is no config file create one
if not configfile.isfile():
    settings = xliffeditor(configfile)
    settings.tm = tm()
    settings.fonts = fonts()
    settings.webdict = webdict()
    print "creating configfile"
    for name,url in webdict_list.items():
        settings.webdict.sites.append(site(name=name,url=url))
    # set current to first site
    settings.webdict.current = settings.webdict.sites[0].name
    settings.save()
else:
    settings = xliffeditor.fromXml(configfile.bytes())
    settings._file = configfile
