from general import EditorException
from transolution.xmlutils import XmlError,getNodeAttr,setNodeAttr,setCreateNodeAttr,removeNodeAttr,getNodes,getElementsByTagName
from xliffsegment import XliffError
from document.tu import TuException

class AtRootSegException(EditorException): pass
class OpenSubSegException(EditorException): pass
class CloseSubSegException(EditorException): pass

class SubTu(object):
    'TU interface for sub segs things like tags.'
    def __init__(self,snode,tnode):
        self.source_node = snode
        self.target_node = tnode
    def getSourceNode(self):
        return self.source_node
    def getTargetNode(self):
        return self.target_node

#---DocumentView---#000000#FFFFFF----------------------------------------------------
empty_tags = ('x','bx','ex')

class SubSeg(object):
    def __init__(self,tu,source_control,target_control,parent=None):
        self.parent = parent
        self.tu = tu
        self.source_control = source_control
        self.target_control = target_control
        self.sub_seg = None
        self.updateUI()
        
    def open(self):
        snode,tnode = self.__checkSubSeg()
        subtu = SubTu(snode,tnode)
        self.sub_seg = SubSeg(subtu,self.source_control,self.target_control,parent=self)
        return self.sub_seg

    def updateUI(self,insertionNode=None):
        source_seg_model = self.source_control.get_buffer().model
        target_seg_model = self.target_control.get_buffer().model
        # Set the source-text widget model to the current node
        source_seg_model.setRootNode(self.tu.getSourceNode())
        source_seg_model.notifyObservers()
        # Set the target-text widget model to the current target node
        try:
            target_seg_model.setRootNode(self.tu.getTargetNode())
        except TuException,e:
            print e
            target_seg_model.setRootNode(None)
        target_seg_model.notifyObservers()
        if insertionNode:
            # set caret to subsegs end
            self.target_control.get_buffer().setInsertionPoint( target_seg_model.getTextSpanOfNode(insertionNode)[1])
        else:
            # Set insertion point to start
            self.target_control.get_buffer().setInsertionPoint(0)


    def __checkSubSeg(self):
        tw = self.target_control.get_buffer()
        target_seg_model = tw.model
        source_seg_model = self.source_control.get_buffer().model
        pos = tw.getInsertionPoint()
        
        try:
            tnode = target_seg_model.getNodeByTextIndex(pos)[0]
        except IndexError,e:
            tnode = target_seg_model.getLast()[0]

        if target_seg_model.isTag(tnode):
            if tnode.tagName in empty_tags:
                raise OpenSubSegException('Cant open empty XLIFF placeholder tags (x,bx,ex).')
            print "SUBTAG!"
            try:
                id = getNodeAttr(tnode,'id')
            except KeyError,e:
                snode = self.tu.getSourceNode()
            else:
                print "Subseg ID:",id
                try:
                    snode = source_seg_model.getTagNodeByNameAndID(tnode.tagName,id)
                except XliffError,e:
                    snode = self.tu.getSourceNode()
                    print e
        else:
            raise OpenSubSegException('Cant open subseg, no tag present at selected position.')
        return snode,tnode

    def close(self,commit,approve=False):
        from transolution.xmlutils import getText
        if not self.parent:
            raise AtRootSegException('At root')

        if commit and not len(getText(self.tu.getTargetNode().childNodes)):
            raise CloseSubSegException('Cant commit when target empty')
        if not commit:
            raise CloseSubSegException('Must commit on subsegs, use undo instead of cancel')

        self.parent.sub_seg = None
        self.parent.updateUI(self.tu.getTargetNode())

        return self.parent
