from utils import shallowNodeCopy,copyTags,getPlainText,getTextWithTags,TMException
from difflib import ndiff

class Match(object):
    def __init__(self,lookup,source,target,quality=None):
        '''
        @lookup - source looked for
        @source - tm source found
        @target - tm target but with lookup tags copied into place
        @quality - the match quality
        '''
        self.lookup = lookup
        self.source = source
        self.target = target
        self.quality = quality

        # create text representation
        self.ltext = getTextWithTags(self.lookup.childNodes)
        self.stext = getTextWithTags(self.source.childNodes)
        self.ttext = getTextWithTags(self.target.childNodes)

    def getNDiff(self):
        'create text to do diff on and return ndiff ourput'
        return ndiff(self.stext,self.ltext)       


from transolution.xmlutils import toxml,parseString

default_port = 6315
default_address = 'http://localhost'
import xmlrpclib
class TmException(Exception):pass
class TmNoMatchFoundException(TmException):pass
class TmAddTuServerException(TmException):pass

class TMClient(object):
    def __init__(self,address=None,port=None):
        print "Connecting",address,port
        if address is None:
            address = default_address
        if port is None: port = default_port
        self.tus = {}
        self.server = xmlrpclib.ServerProxy("%s:%s"%(address,port,))

    def version(self):
        return self.server.version()
    
    def addTU(self,source_node,target_node):
        'add source and target nodes, index them as plain text without tag info'
        try:
            source = shallowNodeCopy(source_node)
            target = shallowNodeCopy(target_node)
            self.server.addTU(toxml(source),toxml(target))
        except xmlrpclib.Fault, fault:
            raise TmAddTuServerException(fault.faultString)

    def lookup(self,lookup):
        'return generator of match objects. @lookup is a xml node with childs'
        lookup_xml = toxml(shallowNodeCopy(lookup))
        try:
            quality,tm_source,tm_target = self.server.lookup(lookup_xml)
        except xmlrpclib.Fault, fault:
            raise TmNoMatchFoundException(fault.faultString)

        tm_source = parseString(tm_source).firstChild
        tm_target = parseString(tm_target).firstChild
        new_target = copyTags(tm_target,lookup)
        yield Match(lookup,tm_source,new_target,quality=quality)
