class TMException(Exception):pass

def getPlainText(nodelist):
    text_nodes = [node.data for node in nodelist if node.nodeType == node.TEXT_NODE]
    return ''.join(text_nodes)

def getTextWithTags(nodelist):
    'get text with placeholders for tags'
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
        elif node.nodeType == node.ELEMENT_NODE:
            if node.tagName == 'bpt': r = '<%s>'%(getNodeAttr(node,'id'),)
            elif node.tagName == 'ept': r = '</%s>'%(getNodeAttr(node,'id'),)
            elif node.tagName == 'ph': r = u'\xAB%s\xBB'%(getNodeAttr(node,'id'),)
            else: r = u'\xAB%s\xBB'%(node.tagName,)
            rc.append(r)
    return ''.join(rc)

from transolution.xmlutils import getNodeAttr

def shallowNodeCopy(target):
    'copy target with childnodes but dont recurse down into childnodes'
    new_target = target.cloneNode(False)
    for node in target.childNodes:
        new_target.appendChild(node.cloneNode(False))
    return new_target

def copyTags(tm_target,new_source):
    'try copy tags from new source into correct place in new target'
    new_target = tm_target.cloneNode(True)
    tagnodes = [node for node in new_target.childNodes if node.nodeType == node.ELEMENT_NODE]
    for node in tagnodes:
        try:
            newChild = getMatchingTagNode(new_source,node)
        except TMException,e:
            # new_source has no tag so remove it from new_target
            new_target.removeChild(node) 
        else:
            # copy new_source tag into new_target
            new_target.replaceChild( newChild.cloneNode(True), node) 
    return new_target

def getMatchingTagNode(target,node):
    'return child node of @target that has same id attr and tagname as @node'
    try:id = getNodeAttr(node,'id')
    except KeyError,e:
        raise TMException("Tag has no id")
    tagnodes = [n for n in target.childNodes if n.nodeType == n.ELEMENT_NODE]
    for child in tagnodes:
        if child.tagName == node.tagName:
            try:
                if getNodeAttr(child,'id') == id:
                    return child
            except KeyError,e:
                continue
    raise TMException("no matching node found")
