'''
<xliff>1
|
+--- <file>+
     +--- <header>?
     |    +--- <skl>?
     |    |    +--- (<internal-file> | <external-file>)1
     |    +--- <phase-group>?
     |    |    +--- <phase>+
     |    |         +--- <note>*
     |    +--- <glossary>*
     |    |    +--- (<internal-file> | <external-file>)1
     |    +--- <reference>*
     |    |    +--- (<internal-file> | <external-file>)1
     |    +--- <count-group>*
     |    |    +--- <count>*
     |    +--- <tool>*
     |    |    +--- [Extension Point]
     |    +--- <prop-group>*
     |    |    +--- <prop>*
     |    +--- [Extension Point]
     |    +--- <note>*
     +--- <body>1
          +--- <group>*
          |    +--- <context-group>*
          |    |    +--- <context>+
          |    +--- <count-group>*
          |    |    +--- <count>*
          |    +--- <prop-group>*
          |    |    +--- <prop>*
          |    +--- [Extension Point]
          |    +--- <note>*
          |    +--- At least one of: (<group>* <trans-unit>* <bin-unit>*)
          +--- <trans-unit>*
          |    +--- <source>1
          |    |    +--- [Inline Elements]
          |    +--- <target>?
          |    |    +--- [Inline Elements]
          |    +--- <context-group>*
          |    |    +--- <context>+
          |    +--- <count-group>*
          |    |    +--- <count>*
          |    +--- <prop-group>*
          |    |    +--- <prop>*
          |    +--- [Extension Point]
          |    +--- <note>*
          |    +--- <alt-trans>*
          |         +--- <context-group>*
          |         |    +--- <context>+
          |         +--- <source>?
          |         |    +--- [Inline Elements]
          |         +--- <target>+
          |         |    +--- [Inline Elements]
          |         +--- <prop-group>*
          |         |    +--- <prop>*
          |         +--- [Extension Point]
          |         +---- <note>*
          +--- <bin-unit>*
               +--- <bin-source>1 & <bin-target>?
               |    +--- (<internal-file> | <external-file>)1
               +--- <context-group>*
               |    +--- <context>+
               +--- <count-group>*
               |    +--- <count>*
               +--- <prop-group>*
               |    +--- <prop>*
               +--- [Extension Point]
               +--- <note>*
               +--- <trans-unit>*
'''
from transolution.path_mod import path
from document.segments import TuSegment,External,FileInfo,TuInfo
from transolution.xmlutils import loadXml,parseString,toxml,XmlError,getNodeAttr,setNodeAttr,setCreateNodeAttr,getNodes,getElementsByTagName
import os
import zipfile,shutil

class XliffException(Exception):pass
class XlzFileException(Exception):pass
class SklFileException(Exception):pass

class Xlz(object):
    'wraps xlz (zipped content.xml and skeleton.xml)'
    def __init__(self,fname):
        self.fname = fname
        if path(fname).isfile():
            if not zipfile.is_zipfile(fname):
                raise XlzFileException('File not a zip:%s'%(fname,))
        else:
            raise XlzFileException('not a file:%s'%(fname,))


    def read(self,fname):
        """read content-xml from the zip-archive"""
        z = zipfile.ZipFile(self.fname,"r")
        content = z.read(fname)
        z.close()
        return content

    def write(self,fname,data):
        'replace a file in a zip-archive'
        z_in = zipfile.ZipFile(self.fname,"r")
        tmpf = self.fname + '.tmp'
        z_out = zipfile.ZipFile(tmpf,"w")
        # Clone everything except the file we will write to a tempfile
        for f in z_in.namelist():
            if f == fname: continue
            zipinfo = z_in.getinfo(f)
            z_out.writestr(zipinfo,z_in.read(f))

        # Add the file to save
        zipinfo = z_in.getinfo(fname)
        z_in.close()
        z_out.writestr(zipinfo,data)
        z_out.close()
        # rename the tmp file to the orig
        if os.path.isfile(self.fname):
            os.remove(self.fname)
        os.rename(tmpf,self.fname)

from StringIO import StringIO

class XliffDocument(object):
    def __init__(self,fname):
        self.basedir = path(fname).abspath().dirname()
        try:
            self.xlz = Xlz(fname)
        except XlzFileException,e:
            self.xlz = None
        print "Parse XML"            
        self.fname = fname
        if self.xlz:
            self.xliffdoc = loadXml(StringIO(self.xlz.read('content.xlf')))
        else:
            self.xliffdoc = loadXml(self.fname)

##        print getDocTypeString(self.xliffdoc)
        print "Parse XLIFF"
        self.parse()
        print "Done"

    def parse(self):
        xliff_element = self.xliffdoc.documentElement
        file_nodes = getNodes(xliff_element,'file','+')
        self.files = []
        for file_node in file_nodes:
            self.files.append(FileNode(file_node,self.basedir,self.xlz))

    def save(self):
        'save the wole xliff dom to xml file'
        if self.xlz:
            'Save content.xlf to xlz file'
            outf = StringIO()
            data = toxml(self.xliffdoc,outf)
            self.xlz.write('content.xlf',outf.getvalue())
        else:
            toxml(self.xliffdoc,file(self.fname,'w'))

    def saveAs(self,fname):
        'save the wole xliff dom to xml file'
        if self.xlz:
            # Copy old file to new
            import shutil
            shutil.copyfile(self.fname,fname)
            self.fname = fname
            self.xlz = Xlz(self.fname)

            # Save content.xlf to xlz file
            outf = StringIO()
##            data = toxml(self.xliffdoc.documentElement,outf,addheader=True)
            data = toxml(self.xliffdoc,outf)
            self.xlz.write('content.xlf',outf.getvalue())
        else:
            self.fname = fname
            toxml(self.xliffdoc,file(fname,'w'))

    def __str__(self):
        r = []
        for f in self.files:
            r.append(unicode(f.header))
            r.append(unicode(f.header.skl))
            r.append(unicode(f.body))
            r.append("Original:%s"%(f.getOriginal()))
            r.append("Original2:%s"%(f.getNodeAttr('original')))
            r.append("Datatype:%s"%(f.getDatatype()))
            r.append("slid:%s"%(f.getSourceLang()))
            r.append("tlid:%s"%(f.getTargetLang()))
        return u'\n'.join(r)

    def __del__(self):
        print "delete dom"
        try:
            self.xliffdoc.unlink()
        except AttributeError,e:
            pass
        del self.xliffdoc

class NodeWrapper(object):
    def __init__(self,filenode):
        self.node = filenode
        self.parse()
    def parse(self):
        raise NotImplementedError('must be implemented in subclasses')

    def getNodeAttr(self,attr):
        return getNodeAttr(self.node,attr)

    def setNodeAttr(self,attr,value):
        setNodeAttr(self.node,attr,value)
  
    def __str__(self):
        return unicode(self.node)
    
class FileNode(NodeWrapper):
    def __init__(self,filenode,basedir=None,xlz=None):
        NodeWrapper.__init__(self,filenode)
        self.basedir = basedir
        self.xlz = xlz
        self.skeleton = None
        
    def parse(self):
        try:
            header_node = getNodes(self.node,'header','?')[0]
        except IndexError,e:
            self.header = None
        else:
            self.header = Header(header_node)
        self.body = Body(getNodes(self.node,'body','1')[0])

    def setTargetLang(self,tlid):
        setCreateNodeAttr(self.node,'target-language',tlid)
##        self.node.attributes[(None,'target-language')].value = tlid

    def getSourceLang(self):
        return getNodeAttr(self.node,'source-language')
    def getTargetLang(self):
        try:return getNodeAttr(self.node,'target-language')
        except KeyError,e:
            return None
    def getDatatype(self):
        try:return getNodeAttr(self.node,'datatype')
        except KeyError,e:
            return None
    def getOriginal(self):
        try:return getNodeAttr(self.node,'original')
        except KeyError,e:
            return None

    def parseSkeleton(self):
        if (not self.header) or (not self.header.skl):
            raise XliffException('Xliff has no skeleton file')
        if self.xlz:
            skl = StringIO(self.xlz.read(self.header.skl))
        else:
            skl = path(self.basedir).joinpath(self.header.skl)
        try:
            self.skeleton = Skeleton(skl)
        except (IOError,SklFileException),e:
            raise XliffException('Cant open skeleton file:%s'%(e,))
        return self.skeleton

    def generateSegments(self,use_skeleton=True):
        'Get the segment object that build a document'
        skeleton = self.skeleton
        tus = self.body.getTUs()
        if use_skeleton and skeleton:
            # TODO: only works if only one file node in skl file
            for node in skeleton.bodies[0].childNodes:
                if node.nodeType == node.ELEMENT_NODE:
                    if node.tagName == 'formatting':
                        yield External(node)
                    elif node.tagName == 'tu-placeholder':
                        id = node.attributes[(None,'id')].value
                        node = tus[id]#.getElementsByTagName('source')[0]
                        yield TuSegment(node)
        else:
            for i,tu in enumerate(self.body.tu_list):
                yield TuInfo(getNodeAttr(tu,'id'),i)
                yield TuSegment(tu)


class Skeleton(object):
    'wrapper(parser) for skeleton file'
    def __init__(self,skl_file):
        try:
            self.skldoc = loadXml(skl_file)
        except IOError,e:
            raise SklFileException('Failed to load skeleton file')
        except Exception,e:
            raise SklFileException('Skeleton file format not supported')

        try:self.parse()
        except XmlError,e:
            raise SklFileException('Skeleton file format not supported')
            
    def parse(self):
        root_element = self.skldoc.documentElement
        file_nodes = getNodes(root_element,'file','+')
        self.bodies = []
        # Todo: multiple file bodies not supported
        for file_node in file_nodes:
            body_node = getNodes(file_node,'body','1')[0]
            self.bodies.append(body_node)

    def __del__(self):
        print "delete skeleton dom"
        try:
            self.skldoc.unlink()
        except AttributeError,e:
            pass

        try:del self.skldoc
        except AttributeError,e:
            pass

class Header(NodeWrapper):
    def parse(self):
        self.skl = None
        try:
            skl_node = getNodes(self.node,'skl','?')[0]
        except IndexError,e:
            ext_file_node = None
        else:
            ext_file_node = getNodes(skl_node,'external-file','?')
        if ext_file_node:
            skl = ext_file_node[0].attributes[(None,'href')].value
            ##self.skl = path(self.basedir).joinpath(skl)
            self.skl = skl

class Body(NodeWrapper):
    def parse(self):
        # TODO: do this in a better way handling group tag etc.
        self.tu_list = getElementsByTagName(self.node,'trans-unit')
        self.tus = {}
        for tu in self.tu_list:
            id = getNodeAttr(tu,'id')
            self.tus[id] = tu

    def getTUs(self):
        'returns a dict {id:tudomnode}'
        return self.tus
    
    def __str__(self):
        return u','.join(self.tus.keys())
    
if __name__ == "__main__":
    import sys
    xml = sys.argv[1]
    xliff = XliffDocument(xml)
    xliff.parse()
    print xliff
    print list(xliff.files[0].generateSegments())
