__doc__ = '''XML Dom manipulation util functions'''
def getText(nodelist,recursive=True):
    rc = []
    if hasattr(nodelist,"nodeType"):
        raise ValueError("not a nodelist")

    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
        elif node.nodeType == node.ELEMENT_NODE:
            if recursive:
                rc += getText(node.childNodes)

    return ''.join(rc)

def getNodeSpan(start,end):
    'slice nodes in span'
    assert start.parentNode == end.parentNode, "Nodes has diff parent"
    node = start
    nodes = []
    while node and node != end:
        nodes.append(node)
        node = node.nextSibling
    return nodes

def removeNodes(start,end):
    'remove nodes in span'
    parent = start.parentNode
    for node in getNodeSpan(start,end):
        #print "Removing",node
        parent.removeChild(node)
       
def copyNodes(start,end):
    'copy nodes in span'
    parent = start.parentNode
    new_parent = parent.cloneNode(False)
   
    for node in getNodeSpan(start,end):
        #print "Copy Node:",node
        new_parent.appendChild(node.cloneNode(True))
    return new_parent.childNodes

def pasteNodes(insert,nodelist):
    'past a copy of nodes in @nodelist before the @insert node'
    parent = insert.parentNode
    #print nodelist
    for new_node in nodelist:
        #print "Paste Node",new_node
        parent.insertBefore( new_node.cloneNode(True), insert)

def splitText(node,offset):
    'split a text node at @offset. return new text node.'
    # cant use node.splitText as it is minidom specific :-( so implement it our self
    if offset < 0 or offset > len(node.data):
        raise IndexError("illegal offset value")
    new_node = node.ownerDocument.createTextNode('')
    #new_node.ownerDocument = node.ownerDocument
    new_node.data = node.data[offset:]
    next = node.nextSibling
    if node.parentNode:# and node in node.parentNode.childNodes:
        if next is None:
            node.parentNode.appendChild(new_node)
        else:
            node.parentNode.insertBefore(new_node, next)
    node.data = node.data[:offset]
    return new_node
